<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\IdMapping;
use Rc\Domain\Model\Media;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition\ToolTemplate;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\UserRepository;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;

class SystemController_Cli extends AbstractController_Cli
{

    public function init()
    {
        $this->_helper->viewRenderer->setNoRender(true);
    }

    public function flushCachesAction()
    {
        shell_exec('rm ' . cache_path('*') . ' -Rf');
    }

    public function garbageAction()
    {
        shell_exec('find ' . temp_path() . '/ -type f -mtime 1 -exec rm {} \;');
    }

    public function ensureIndexesAction()
    {
        MongoRepository::ensureIndexes();
    }

    public function copyDatabasesAction()
    {
        $mysql = config('databases.mysql');

        if (app_env('production') || config('sync.mongodb.database', 'production') == MongoRepository::getDb()) {
            throw new BadMethodCallException('Copy from cannot be used in production!');
        }

        $pdo = MySqlRepository::getPdo();
        $pdo->exec('DROP DATABASE IF EXISTS `' . $mysql['database'] . '`');
        $pdo->exec('CREATE DATABASE `' . $mysql['database'] . '` CHARACTER SET utf8 COLLATE utf8_general_ci');

        // mysql
        $this->sshTunnel(function () use ($mysql) {
            $process = new Process(sprintf(
                'mysqldump -h127.0.0.1 -P3307 -u%s -p%s %s %s | mysql -u%s -p%s %s',
                config('sync.mysql.username'),
                config('sync.mysql.password'),
                config('sync.mysql.database'),
                implode(' ', config('sync.mysql.tables')),
                $mysql['username'],
                $mysql['password'],
                $mysql['database']
            ));
            $process->setTimeout(0);
            $process->mustRun();
        }, 3306, 3307);

        $connection = MongoRepository::getClient()->getConnection();
        $connection->selectDatabase(MongoRepository::getDb())->drop();

        // mongodb
        $this->sshTunnel(function () use ($mysql) {
            $result = MongoRepository::getClient()->getConnection()->selectDatabase('admin')->command([
                'copydb'   => 1,
                'fromhost' => '127.0.0.1:27020',
                'fromdb'   => config('sync.mongodb.database'),
                'todb'     => MongoRepository::getDb(),
            ], ['socketTimeoutMS' => 60 * 60 * 1000]);

            if (empty($result['ok'])) {
                throw new Exception($result['errmsg']);
            }

            MongoRepository::ensureIndexes();
        }, 27017, 27020);
    }

    public function fixArrayNotationAction()
    {
        $connection = MongoRepository::getClient()->getConnection();

        foreach (['Popover', 'Weave', 'SimpleMedia', 'Tool', 'ToolTemplate', 'Research', 'User'] as $collection) {

            $collection = $connection->selectCollection(MongoRepository::getDb(), $collection);

            foreach (['title', 'description', 'padding', 'margin', 'border', 'background', 'shadow'] as $property) {
                $collection->update([$property => ['$eq' => []]], ['$set' => [$property => (object)[]]], ['multi' => true]);
            }
        }
    }

    public function fixUserGroupsAction()
    {
        foreach (UserRepository::findAll() as $user) {
            $user->setPortalIds($user->getPortalIds());

            UserRepository::persist($user);
        }

        UserRepository::flush();
    }

    public function fixSharedWorksAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        $exposition = ExpositionRepository::findById($this->getParam('id'));
        $exposition->setShares(
            ResearchRepository::findByIds(array_map(function (Research $research) {
                return $research->getUuid();
            }, $exposition->getShares()))
        );
        ExpositionRepository::persist($exposition);
        ExpositionRepository::flush();
    }

    /**
     * @throws Zend_Exception
     * @throws Exception
     */
    public function fixMissingIdMappingAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        $repository = $this->getParam('repository');
        if (!is_a($repository, MongoRepository::class, true)) {
            throw new Exception('You have to provide a valid repository! Given "' . $repository . '" is not valid.');
        }

        /** @var ModelInterface $model */
        $model = $repository::findById($this->getParam('id'));

        IdMappingRepository::persist(new IdMapping(get_class($model), $model->getId(), $model->getUuid()));
        IdMappingRepository::flush();

        print_r([
            'model' => get_class($model),
            'uuid'  => $model->getUuid(),
            'id'    => $model->getId(),
        ]);
    }

    /**
     * @throws Exception
     * @see https://sar.tracker.muehlschlegel.info/rc/issues/rc-983
     */
    public function fixMissingThumbAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        $tool = ToolRepository::findById($this->getParam('id'));
        if (!isset($tool)) {
            throw new Exception('Tool#' . $this->getParam('id') . ' does not exist!');
        }

        foreach ($tool->getMediaList() as $simpleMedia) {
            try {
                $media = $tool->getSimpleMediaContainer($simpleMedia);

                if ($media->hasThumb()) {
                    $media->getThumb()->getCheckSum();
                }
            } catch (\Exception $exception) {
                $tool->setImage($simpleMedia);
                echo 'fixed SimpleMedia#' . $simpleMedia->getId(), PHP_EOL;
            }
        }

        ToolRepository::persist($tool);
        ToolRepository::flush();
    }

    public function php7MigrationAction()
    {
        $this->fixIdMappingAction();
        $this->fixToolRelationMappingAction();
        $this->fixResearchRelationMappingAction();
    }

    public function fixIdMappingAction()
    {
        MongoRepository::getClient()
            ->getConnection()
            ->selectCollection(MongoRepository::getDb(), 'IdMapping')
            ->remove(['referenceType' => IdMapping::class]);

        MongoRepository::getClient()
            ->getConnection()
            ->selectCollection(MongoRepository::getDb(), 'IdMapping')
            ->remove(['referenceType' => Media::class]);

        MongoRepository::getClient()
            ->getConnection()
            ->selectCollection(MongoRepository::getDb(), 'IdMapping')
            ->remove(['referenceType' => ToolTemplate::class]);
    }

    public function fixToolRelationMappingAction()
    {
        $tools = MongoRepository::getClient()
            ->getConnection()
            ->selectCollection(MongoRepository::getDb(), 'Tool')
            ->find();

        $counter = 0;

        foreach ($tools as $tool) {
            array_walk_recursive($tool, function (&$value, $key) use (&$counter) {
                if (($key === '_class' || $key === '_doctrine_class_name') && ($value === Tool::class)) {
                    ++$counter;
                }
            });
        }

        echo $counter, PHP_EOL;

        foreach ($tools as $tool) {

            if (empty($tool['players'])) {
                continue;
            }

            $changed = false;

            foreach ($tool['players'] as $playerKey => &$player) {

                foreach (['_class', '_doctrine_class_name'] as $classKey) {
                    if (!array_key_exists($classKey, $player) || !($player[$classKey] === Tool::class || empty($player[$classKey]))) {
                        continue;
                    }

                    $changed = true;

                    $playerTool = MongoRepository::getClient()
                        ->getConnection()
                        ->selectCollection(MongoRepository::getDb(), 'Tool')
                        ->findOne(['_id' => $player['$id']]);

                    if (empty($playerTool['_class'])) {
                        unset($tool['players'][$playerKey]);
                    } else {
                        $player[$classKey] = $playerTool['_class'];
                    }
                }
            }

            if (!$changed) {
                continue;
            }

            MongoRepository::getClient()
                ->getConnection()
                ->selectCollection(MongoRepository::getDb(), 'Tool')
                ->update(['id' => $tool['id']], $tool);
        }
    }

    public function fixResearchRelationMappingAction()
    {
        $researches = MongoRepository::getClient()
            ->getConnection()
            ->selectCollection(MongoRepository::getDb(), 'Research')
            ->find();

        $counter = 0;

        foreach ($researches as $research) {
            array_walk_recursive($research, function (&$value, $key) use (&$counter) {
                if (($key === '_class' || $key === '_doctrine_class_name') && ($value === Research::class)) {
                    ++$counter;
                }
            });
        }

        echo $counter, PHP_EOL;

        foreach ($researches as $research) {

            if (empty($research['relations'])) {
                continue;
            }

            $changed = false;

            foreach ($research['relations'] as $relationKey => &$player) {

                foreach (['_class', '_doctrine_class_name'] as $classKey) {
                    if (!array_key_exists($classKey, $player) || !($player[$classKey] === Research::class || empty($player[$classKey]))) {
                        continue;
                    }

                    $changed = true;

                    $relationResearch = MongoRepository::getClient()
                        ->getConnection()
                        ->selectCollection(MongoRepository::getDb(), 'Research')
                        ->findOne(['_id' => $player['$id']]);

                    if (empty($relationResearch['_class'])) {
                        unset($research['relations'][$relationKey]);
                    } else {
                        $player[$classKey] = $relationResearch['_class'];
                    }
                }
            }

            if (!$changed) {
                continue;
            }

            MongoRepository::getClient()
                ->getConnection()
                ->selectCollection(MongoRepository::getDb(), 'Research')
                ->update(['id' => $research['id']], $research);
        }
    }

    /**
     * + remove IdMappings which don't have an identifier (id + uuid)
     * + enrich uuid if it's missing
     */
    public function normalizeIdMappingAction()
    {
        Zend_Registry::get('RC_DILPS_AUTH')->login('admin');

        $database = MongoRepository::getClient()->getConnection()->selectDatabase(MongoRepository::getDb());

        // remove entries which don't have any data
        $database->selectCollection('IdMapping')->remove([
            'referenceId'   => null,
            'referenceUuid' => null,
        ]);

        // enrich uuid
        $missingUuidMappings = $database->selectCollection('IdMapping')->find([
            'referenceUuid' => null,
        ]);
        foreach ($missingUuidMappings as $idMapping) {
            try {
                $entity = MongoRepository::findById($idMapping['referenceId']);
            } catch (\Throwable $exception) {
            }

            if (!isset($entity)) {
                echo 'entity not found [' . $idMapping['referenceType'] . '#' . $idMapping['referenceId'] . ']' . PHP_EOL;
                $database->selectCollection('IdMapping')->remove([
                    '_id' => $idMapping['_id'],
                ]);
                continue;
            }

            $database->selectCollection('IdMapping')->update(
                [
                    '_id' => $idMapping['_id'],
                ],
                [
                    '$set' => [
                        'referenceUuid' => $entity->getUuid(),
                    ],
                ]
            );
        }

        echo 'updated ', count($missingUuidMappings), PHP_EOL;
    }

    /**
     * @param callable $callback
     * @param int $sourcePort
     * @param int $destinationPort
     * @return mixed
     */
    protected function sshTunnel(callable $callback, int $sourcePort, int $destinationPort)
    {
        try {
            $tunnel = new Process('exec ' . sprintf(config('sync.ssh_tunnel_format'), $destinationPort, $sourcePort));
            $tunnel->start();

            while ($tunnel->isRunning() && !$tunnel->isTerminated()) {
                if (is_resource(@fsockopen('127.0.0.1', $destinationPort))) {
                    return $callback();
                }
                usleep(500);
            }

            throw new ProcessFailedException($tunnel);
        } finally {
            if (isset($tunnel) && $tunnel instanceof Process) {
                $tunnel->stop();
            }
        }
    }
}