<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Application\Review;
use Rc\Domain\Model\Media;
use Rc\Domain\Model\Message;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Repository\Application\ApplicationProgramRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\Research\WorkRepository;
use Rc\Domain\Repository\UserRepository;

class ApplicationController extends BasicControllerAction
{

    public function listAction()
    {
        $form = $this->_getForm();

        $this->view->assign('form', $form);
    }

    public function addAction()
    {
        list($programId, $portalId) = explode('-', $this->_getParam('application-program', '-'));
        $applicationProgram = ApplicationProgramRepository::findById($programId);
        $portal = PortalRepository::findById($portalId);

        /** @var ApplicationDefaultForm $form */
        $form = $this->_getForm($applicationProgram, null, $this->isFormSubmit());

        if ($form->isPostAndValid() && !Identity::getUser()->isLimitedUser()) {
            $application = new Application();
            $application->setOwner(Identity::getUser());
            $application->setProgram($applicationProgram);
            $application->setApplicantPortal($applicationProgram->getPortal() !== $portal ? $portal : null);

            $values = $form->normalizeValues($form->getValues());
            $application->setMetaData($values['custom']['elements']);
            $application->setRequiredDocuments($values['custom']['files']);
            $application->setOptionalDocuments($values['custom']['optionalfiles']);
            $application->setValues($values);

            ApplicationRepository::persist($application);

            $this->_handleCollaboration($application, $application->getCoAuthors(), [], 'coauthor');
            $this->_handleCollaboration($application, $application->getSupervisors(), [], 'supervisor');

            $this->_setFormActionHeader('/application/edit?application=' . $application->getId());
            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->assign('form', $form);
        }
    }

    public function editAction()
    {
        $application = $this->_getRequestedApplication();
        if (!$application->isInProgress()) {
            throw new InsufficientRightsException();
        }

        /** @var ApplicationDefaultForm $form */
        $form = $this->_getForm($application, $this->isFormSubmit());
        $form->setDefaultsFrom($application);

        if ($form->isPostAndValid()) {
            $oldCoAuthors = $application->getCoAuthors();
            $oldSupervisors = $application->getSupervisors();

            $values = $form->normalizeValues($form->getValues());

            $application->setMetaData($values['custom']['elements']);
            $application->setRequiredDocuments($values['custom']['files']);
            $application->setOptionalDocuments($values['custom']['optionalfiles']);

            $application->setValues($values);

            if ($application->isOwner(Identity::getUser())) {
                $this->_handleCollaboration($application, $application->getCoAuthors(), $oldCoAuthors, 'coauthor');
                $this->_handleCollaboration($application, $application->getSupervisors(), $oldSupervisors,
                    'supervisor');
            }

            ApplicationRepository::persist($application);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->assign('form', $form);
            $this->view->assign('application', $application);
        }
    }

    public function publishAction()
    {
        $application = $this->_getRequestedApplication();
        if (!$application->isInProgress() || !$application->isOwner(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $form = $this->_getForm($application);
        if ($form->isPostAndValid()) {
            $attachment = $this->_helper->application()->createSubmitAttachment($application);

            if (!empty($application->getApplicantPortal())) {
                $portal = $application->getApplicantPortal();
                $application->setStatus(Application::STATUS_PRE_REVIEW);
            } else {
                $portal = $application->getPortal();
                $application->setStatus(Application::STATUS_REVIEW);
            }

            $message = new Message();
            $message->setSenderId(UserRepository::findSystemAccount()->getId());
            $message->setReceiverId(Identity::getUserId());
            $message->setType(Message::TYPE_APPLICATION_SUBMITTED);
            $message->addAttachment($attachment, 'application.pdf');
            $message->setContent([
                'application' => $application->getId(),
                'portal'      => $portal->getId(),
            ]);
            $this->_sendMessage($message);

            // create notification for portal admins
            foreach ($application->getPortal()->getAdminIds() as $adminId) {
                $message = new Message();
                $message->setSenderId(UserRepository::findSystemAccount()->getId());
                $message->setReceiverId($adminId);
                $message->setType(Message::TYPE_SUBMIT_TO_PORTAL);
                $message->setContent([
                    'user'   => Identity::getUserId(),
                    'entity' => $application->getId(),
                    'type'   => 'application',
                    'portal' => $portal->getId(),
                ]);

                $this->_sendMessage($message);
            }

            $duplicateFacade = new \Rc\Utility\DuplicateFacade();
            $application->addSnapshot($duplicateFacade->applicationSnapshot($application));

            ApplicationRepository::persist($application);

            foreach ($application->getLatestSnapshot()->getResearches() as $research) {
                if ($research instanceof StatusAwareInterface) {
                    $research->removeExternalDependencies();
                }
            }

            ApplicationRepository::persist($application);

            $this->view->assign('portal', $portal);
        } else {
            $this->view->assign('form', $form);
        }
    }

    public function exportReviewsAction()
    {
        try {
            $application = $this->_getRequestedApplication();
            if (!$application->isInReview() || !$application->isReader(Identity::getUser())) {
                throw new InsufficientRightsException();
            }

            /** @var Media $media */
            $media = $this->_helper->application()->createReviewAttachment($application);

            header('X-Accel-Redirect: ' . sendfile($media->getUri()));
            header('Content-type: application/octet-stream');
            header('Content-Disposition: attachment; filename="Reviews for application ' . $application->getTitle() . '.pdf"');

            $this->_disableView();
        } catch (InsufficientRightsException $exception) {
            $this->_handleInsufficientRights();
        }
    }

    public function reviewAction()
    {
        $application = $this->_getRequestedApplication();
        if (!$application->isInReview() || !$application->isReviewer(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $review = $application->getReview(Identity::getUser());

        if (!$review->hasStatus(Review::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }

        /** @var ApplicationDefaultForm $form */
        $form = $this->_getForm($application, $this->isFormSubmit());
        $form->setDefaultsFrom($application);

        if ($form->isPostAndValid()) {
            $values = $form->normalizeValues($form->getValues());
            $review->setMetaData($values['custom']['elements']);

            $application->addReview($review);

            ApplicationRepository::persist($application);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->assign('form', $form);
        }
    }

    public function submitReviewAction()
    {
        $application = $this->_getRequestedApplication();
        if (!$application->isInReview() || !$application->isReviewer(Identity::getUser())) {
            throw new InsufficientRightsException();
        }

        $review = $application->getReview(Identity::getUser());
        if (!$review->hasStatus(Review::STATUS_PROGRESS)) {
            throw new InsufficientRightsException();
        }

        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($application->getId());

        $form = $this->_getForm($application, $this->isFormSubmit());
        $form->setDefaultsFrom($application);

        if ($form->isPostAndValid()) {
            $review->setStatus(Review::STATUS_SUBMITTED);
            $review->setSubmitted(new DateTime());

            ApplicationRepository::persist($application);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->assign('form', $form);
        }
    }

    public function searchExpositionAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (ExpositionRepository::suggest('title', $this->_getParam('name'),
            ['limit' => 10, 'isOwner' => true]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch() . ' (' . $suggest->getEntity()->getId() . ')',
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }

    public function searchWorkAction()
    {
        $this->_disableView();

        $retVal = [];
        foreach (WorkRepository::suggest('title', $this->_getParam('name'),
            ['limit' => 10, 'isOwner' => true]) as $suggest) {
            $retVal[] = [
                'value' => $suggest->getEntityId(),
                'label' => $suggest->getMatch(),
            ];
        }

        header('Content-Type: application/json');
        echo json_encode($retVal);
    }
}
