<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Core\Revision\RevisionFactory;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Generic\TextAwareInterface;
use Rc\Domain\Model\Research\Exposition\Generic\ThreeDimensionalAwareInterface;
use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaFactory;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\CommentTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Comment\ApplicationContext;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Comment\ExpositionContext;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\ContextAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\ContextRegistry;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Generic\AspectRatioAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\SimpletextTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\TextTool;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Model\Translation;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\ToolTemplateRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Service\Madek\MadekServiceFactory;

class ItemController extends BasicControllerAction
{

    /**
     * @throws Exception
     * @throws GatewayException
     * @throws InsufficientRightsException
     */
    public function addAction()
    {
        $exposition = $this->_getRequestedResearch();
        $weave = $this->getRequestedWeave();

        $tool = ToolFactory::buildByName($this->getParam('tool'));
        $toolTemplate = ToolTemplateRepository::findByNameAndResearch($tool->getName(), $exposition);
        if (isset($toolTemplate)) {
            $formTemplate = ToolFormGenerator::getEditTemplate($toolTemplate->getTool(),
                $exposition)->setDefaultsFrom($toolTemplate->getTool());
            $tool->setValues($formTemplate->getValues());
        }

        $this->addWritePermissionsForNotes($exposition, [$tool]);

        $tool->setTitle(new Translation($this->view->translate('tool.' . $tool->getName()) . ' #' . $weave->incrementItemCounter($tool)))
            ->setOwner(Identity::getUser())
            ->setWeave($weave)
            ->setLeft($this->getParam('left'))
            ->setTop($this->getParam('top'))
            ->setHeight($this->getParam('height'))
            ->setWidth($this->getParam('width'))
            ->setOpacity(ToolInterface::CSS_OPACITY_VISIBLE)
            ->setZIndex($weave->getMaxZIndex() + 1)
            ->setPosition(ToolInterface::POSITION_CENTER_CENTER)
            ->setReviewing($exposition->isStatus(Exposition::STATUS_REVIEW));

        if ($tool instanceof ContextAwareInterface) {
            ContextRegistry::fromGlobals();
            if (ContextRegistry::hasContextFor('application')) {
                $tool->setContext(ApplicationContext::from($this->_getRequestedApplication(), $tool));
            } else {
                $tool->setContext(ExpositionContext::from($tool));
            }
        }

        if (empty($this->getParam('file'))) {
            $files = [];
        } else if (is_numeric($this->getParam('file'))) {
            $files = $this->_getRequestedFiles(false);
        } else if (is_string($this->getParam('file'))) {
            $madekService = MadekServiceFactory::create();
            $media = $madekService->importMediaFile($this->getParam('file'));

            $madekMedia = $madekService->mediaEntity($this->getParam('file'));

            $simpleMedia = SimpleMediaFactory::create($media->getContentType());
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($exposition);
            $simpleMedia->setName($madekMedia->getName());
            $simpleMedia->setCopyright($madekMedia->getCopyright());
            $simpleMedia->setDescription($madekMedia->getDescription());
            $simpleMedia->setMedia($media);

            SimpleMediaRepository::persist($simpleMedia);

            $files = [$simpleMedia];
        }

        foreach ($files as $simpleMedia) {
            $this->enforceRights($simpleMedia, $exposition);

            $tool->setMedia($simpleMedia);
            if ($tool instanceof TextAwareInterface && $simpleMedia instanceof TextAwareInterface) {
                $tool->setTextContent($simpleMedia->getTextContent());
            }

            if ($tool instanceof ThreeDimensionalAwareInterface && $simpleMedia instanceof ThreeDimensionalAwareInterface) {
                $tool->setStartLevel($simpleMedia->getStartLevel());
                $tool->setLevels($simpleMedia->getLevels());
                $tool->setAutoRotate($simpleMedia->getAutoRotate());
            }
        }

        ToolRepository::persist($tool);
        WeaveRepository::persist($weave);
        $this->updateResearchLastModified($tool->getExposition());

        $this->view->item = $tool;
    }

    public function editAction()
    {
        $tool = $this->_getRequestedItem();
        $exposition = $tool->getExposition();

        $this->addWritePermissionsForNotes($exposition, [$tool]);

        $form = $this->_getForm($tool, $exposition)
            ->setDefaultsFrom($tool)
            ->setDefault('textcontent', My_View_Helper_ToolDecorator::parsePopovers($tool->getTextContent(),
                My_View_Helper_ToolDecorator::RENDER_FOR_EDITOR));

        if (!$tool->isEditableBy(Identity::getUserOrNull())) {
            $this->view->item = $tool;
            $this->view->form = $this->_createInsufficientRightsForm($tool, $form->getTitle());
        } elseif ($form->isPostAndValid()) {
            $hasTextChanged = false;
            if ($this->hasParam('textcontent')) {
                $textContent = $this->getParam('textcontent');
            } elseif (isset($this->getParam('media')['textcontent']) && $this->getParam('media')['textcontent'] != '') {
                $textContent = $this->getParam('media')['textcontent'];
            } else {
                $textContent = null;
            }
            if (isset($textContent)) {
                $hasTextChanged = ($tool->getTextContent() !== $textContent);
            }
            $tool->setValues($form->getValues());

            $backgroundImageFileId = $this->getParam('style[backgroundimagefileid]');
            if (isset($backgroundImageFileId)) {
                $this->enforceRights(SimpleMediaRepository::findById($backgroundImageFileId), $exposition);
            }

            if ($tool instanceof AspectRatioAwareInterface && $tool->isFixAspectRatio()) {
                $tool->enforceAspectRatio();
            }

            if ($hasTextChanged == true) {
                $tool->setTextAuthor(Identity::getUser());
                $tool->setTextModified(date('d.m.Y - H:i:s'));
                $tool->setTextContent($this->updateChangedPopovers($tool, $tool->getTextContent()));
            }

            ToolRepository::persist($tool);

            $this->updateResearchLastModified($exposition);

            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->item = $tool;
        }
    }

    public function removeAction()
    {
        $exposition = $this->_getRequestedResearch();
        $tools = $this->_getRequestedItems();

        if (empty($tools)) {
            return;
        }

        $this->addWritePermissionsForNotes($exposition, $tools);

        $form = $this->_getForm(reset($tools));

        foreach ($tools as $tool) {
            if (!$tool->isEditableBy(Identity::getUserOrNull())) {
                $this->view->form = $this->_createInsufficientRightsForm($tool, $form->getTitle());
                return;
            }
        }

        if ($form->isPostAndValid()) {
            foreach ($tools as $tool) {
                if (!$tool->isReviewing() || !$tool->getExposition()->isStatus(Exposition::STATUS_REVIEW)) {
                    foreach (PopoverRepository::findByWeave($tool->getWeave()) as $popover) {
                        if ($popover->getTool() && $popover->getTool()->getUuid() == $tool->getUuid()) {
                            $popover->setTool(null);
                            PopoverRepository::persist($popover);
                        }
                    }

                    RevisionFactory::create($tool, 'delete');
                }

                ToolRepository::remove($tool);
            }

            $this->updateResearchLastModified($exposition);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function convertAction()
    {
        $exposition = $this->_getRequestedResearch();
        $tools = $this->_getRequestedItems();

        $this->addWritePermissionsForNotes($exposition, $tools);

        $form = $this->_getForm();

        foreach ($tools as $item) {
            if (!$item->isEditableBy(Identity::getUserOrNull())) {
                $this->view->form = $this->_createInsufficientRightsForm($item, $form->getTitle());
                return;
            }
        }

        if ($form->isPostAndValid()) {
            foreach ($tools as $item) {
                if (!$item instanceof SimpletextTool) {
                    continue;
                }

                $item->addRevisionComment('convert');
                RevisionFactory::create($item);

                ToolRepository::convert($item, TextTool::class);
            }

            $this->updateResearchLastModified($exposition);

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @todo: restore
     */
    public function restoreAction()
    {
        $form = $this->_getForm();

        if ($form->isPostAndValid()) {
            $toolService = new \Rc\Utility\ToolService();

            $tool = $toolService->restore($this->getParam('revision'));

            $this->updateResearchLastModified($tool->getExposition());

            $this->view->item = $tool;
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function restoreDeletedAction()
    {
        $form = $this->_getForm($this->getRequestedWeave());

        $this->view->form = $form;
    }

    public function updateTextContentAction()
    {
        $tool = $this->_getRequestedItem();
        $text = $this->getParam('textcontent');

        $this->addWritePermissionsForNotes($tool->getExposition(), [$tool]);

        if ($tool->getTextContent() != $text) {
            $tool->setTextContent($text);
            $tool->setTextAuthor(Identity::getUser());
            $tool->setTextModified(date('d.m.Y - H:i:s'));
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());
        }

        $this->_disableView();
    }

    public function updateSimpleTextContentAction()
    {
        $tool = $this->_getRequestedItem();
        $text = $this->getParam('textcontent');

        if ($tool->getTextContent() != $text) {
            $text = $this->updateChangedPopovers($tool, $text);
            $tool->setTextContent($text);
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());
        }

        $this->_disableView();
    }

    public function updateTrackingAction()
    {
        $tool = $this->_getRequestedItem();

        $this->addWritePermissionsForNotes($tool->getExposition(), [$tool]);

        if ($tool instanceof CommentTool) {
            $assignee = $tool->getAssignee();
            $deadline = $tool->getDeadline();
            $tool->setAssignee($this->getParam('assignee') != '-1' ? UserRepository::findById($this->getParam('assignee')) : null);
            $tool->setResolution($this->getParam('resolution'));
            $tool->setDeadline($this->getParam('deadline'));
            if ($tool->getAssignee() && $tool->getDeadline() &&
                ($assignee != $this->getParam('assignee') || $deadline != $this->getParam('deadline'))
            ) {
                $message = new \Rc\Domain\Model\Message();
                $message->setSenderId(Identity::getUserId());
                $message->setReceiverId($tool->getAssignee()->getId());
                $message->setType(constant('Message::TYPE_TASK_ADD'));
                $message->setContent([
                    'user'     => Identity::getUserId(),
                    'research' => $tool->getExposition()->getId(),
                    'weave'    => $tool->getWeave()->getId(),
                    'deadline' => $tool->getDeadline(),
                    'comment'  => $tool->getId(),
                    'content'  => $tool->getTextContent(),
                ]);
                $this->_sendMessage($message);
            }
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());
        }

        $this->_disableView();
    }

    public function updateAction()
    {
        $tools = $this->_getRequestedItems();
        $exposition = $this->_getRequestedResearch();

        $this->addWritePermissionsForNotes($exposition, $tools);

        try {
            foreach ($tools as $id => $tool) {
                $tool->setLeft($this->getParam('left')[$id])
                    ->setTop($this->getParam('top')[$id])
                    ->setHeight($this->getParam('height')[$id])
                    ->setWidth($this->getParam('width')[$id])
                    ->setRotate($this->getParam('rotate')[$id]);
                ToolRepository::persist($tool);
            }

            $this->updateResearchLastModified($exposition);
        } catch (InsufficientRightsException $e) {
            $this->_disableView();
            return;
        }

        if ($this->hasParam('resize') && !(end($tools) instanceof CommentTool)) {
            $this->view->item = end($tools);
        } else {
            $this->_disableView();
        }
    }

    public function updateZIndexAction()
    {
        foreach ($this->getParam('index', []) as $itemId => $index) {
            ToolRepository::persist(ToolRepository::getById($itemId)->setZIndex($index));
        }

        $this->_disableView();
    }

    public function updateLockAction()
    {
        foreach ($this->getParam('lock', []) as $itemId => $lock) {
            $item = ToolRepository::getById($itemId);
            ToolRepository::persist($item->lock($lock));
        }
        $this->_disableView();
    }

    public function synchronizeAction()
    {
        $item = $this->_getRequestedItem();
        $form = $this->_getForm();

        if ($form->isPostAndValid()) {

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->item = $item;
        }
    }

    public function listAction()
    {
        if ($this->hasParam('item')) {
            $this->view->items = [$this->_getRequestedItem()];
        } else {
            $this->view->items = ToolRepository::findByExposition($this->_getRequestedResearch());
        }
    }

    public function selectResourceAction()
    {
        $tool = $this->_getRequestedItem();
        $research = $this->_getRequestedResearch();
        $form = $this->_getForm($tool, $research)->setDefaults([
            'file' => $tool->getFilePositions(),
        ]);

        if ($form->isPostAndValid()) {
            $simpleMediaList = $this->_getRequestedFiles(false);
            $tool->setMediaList($simpleMediaList);

            if ($tool instanceof AspectRatioAwareInterface && $tool->isFixAspectRatio()) {
                $tool->enforceAspectRatio();
            }

            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function selectAudioAction()
    {
        $tool = $this->_getRequestedItem();
        $research = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($tool, $research)->setDefaults([
            'audio' => $tool->getAudioId($simpleMedia),
        ]);

        if ($form->isPostAndValid()) {
            $audioId = (int)$this->getParam('audio');
            $tool->setAudio($simpleMedia, SimpleMediaRepository::findById($audioId));
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function selectImageAction()
    {
        $tool = $this->_getRequestedItem();
        $research = $this->_getRequestedResearch();
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($tool, $research)->setDefaults([
            'image' => $tool->getImageId($simpleMedia),
        ]);

        if ($form->isPostAndValid()) {
            $imageId = (int)$this->getParam('image');
            $tool->setImage($simpleMedia, SimpleMediaRepository::findById($imageId));
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
        }
    }

    public function editResourceAction()
    {
        $item = $this->_getRequestedItem();
        $simpleMedia = $this->_getRequestedFile();
        $form = $this->_getForm($item, $simpleMedia)->setDefaultsFrom($simpleMedia);

        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse !== null && $formToUse->isPostAndValid()) {
            session_write_close();
            $simpleMedia->setValues($formToUse->getValues());

            SimpleMediaRepository::persist($simpleMedia);

            $this->updateResearchLastModified($item->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->item = $item;
        }
    }

    public function uploadResourceAction()
    {
        $tool = $this->_getRequestedItem();
        $research = $this->_getRequestedResearch();
        $form = SimpleMediaFormGenerator::getAdd();

        if ($this->getRequest()->isPost()) {
            $formToUse = $this->_getSubmittedTab($form);
        } else {
            $formToUse = $form;
        }

        if ($formToUse->isPostAndValid()) {
            $simpleMedia = SimpleMediaFactory::create($formToUse->getValue('mediatype'));
            $simpleMedia->setOwner(Identity::getUser());
            $simpleMedia->setResearch($research);
            $simpleMedia->setValues($formToUse->getValues());

            SimpleMediaRepository::persist($simpleMedia);

            $tool->addFile($simpleMedia);
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());

            $this->_setFormActionHeader('/item/edit-resource?item=' . $tool->getId() . '&file=' . $simpleMedia->getId());
            $this->_setSuccessHeader();
            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->item = $tool;
        }
    }

    public function loadResourceAction()
    {
        $item = $this->_getRequestedItem();
        $form = $this->_getForm($item, $this->_getRequestedResearch());

        if ($form->isPostAndValid()) {
            $simpleMedia = $this->_getRequestedFile();
            if ($simpleMedia instanceof TextAwareInterface) {
                $this->view->textContent = $simpleMedia->getTextContent();
            }
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function updateResourcePositionAction()
    {
        $tool = $this->_getRequestedItem();
        $tool->setFilePosition($this->getParam('file'), $this->getParam('position'));
        ToolRepository::persist($tool);

        $this->updateResearchLastModified($tool->getExposition());

        $this->view->item = $tool;
        $this->render('list-resource');
    }

    public function removeResourceAction()
    {
        $simpleMedia = $this->_getRequestedFile();
        $tool = $this->_getRequestedItem();
        $form = $this->_getForm($simpleMedia);

        if ($form->isPostAndValid()) {
            $tool->removeResource($simpleMedia);
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($tool->getExposition());

            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function listResourceAction()
    {
        $this->view->item = $this->_getRequestedItem();
    }

    public function pasteErrorAction()
    {
        $form = $this->_getForm();
        if ($form->isPostAndValid()) {
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    public function copyAction()
    {
        $params = $this->getAllParams();
        $exposition = $this->_getRequestedResearch();
        $weave = $this->getRequestedWeave();
        $tools = $this->_getRequestedItems();

        $newTools = [];
        foreach ($tools as $id => $tool) {
            $newTool = clone $tool;
            $newTool->setWeave($weave);
            $newTool->setOwner(Identity::getUser());
            $newTool->setLeft($params['left'][$id]);
            $newTool->setTop($params['top'][$id]);

            foreach ($newTool->getMediaList() as $simpleMedia) {
                if (!$simpleMedia->hasWork() && $exposition !== $tool->getWeave()->getExposition()) {
                    $simpleMedia = clone $simpleMedia;
                    $simpleMedia->setOwner(Identity::getUser());
                    $simpleMedia->setResearch($exposition);
                    $simpleMedia->setName($simpleMedia->getName() . ' - copy');

                    SimpleMediaRepository::persist($simpleMedia);
                }

                $newTool->addFile($simpleMedia);
            }

            ToolRepository::persist($newTool);

            $newTools[] = $newTool;
        }

        $this->view->items = $newTools;
    }

    public function cutAction()
    {
        $params = $this->getAllParams();

        $this->_getRequestedResearch();
        $weave = $this->getRequestedWeave();
        $items = $this->_getRequestedItems();

        foreach ($items as $id => $item) {
            $item->setWeave($weave)
                ->setLeft($params['left'][$id])
                ->setTop($params['top'][$id]);
            ToolRepository::persist($item);
        }

        $this->view->items = $items;
    }

    public function processEmbedUrlAction()
    {
        $this->view->data = Tool\EmbedTool::processUrl($this->_getParam('url'));
    }

    protected function _createInsufficientRightsForm(ToolInterface $tool, $title)
    {
        $form = new DefaultForm();
        $form->setTitle($title);

        $form->addElement('html', 'info', [
            'value' => 'You have insufficient rights to edit this tool!',
        ]);

        $form->addElement('button', 'okbutton', [
            'label'   => 'button.ok',
            'onclick' => '$(this).closest(\'form\').dialog(\'close\'); return false;',
        ]);

        return $form;
    }

    protected function updateChangedPopovers(Tool $tool, $content)
    {
        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");
        $content = str_replace('<p></p>', '<p><br/></p>', $content);
        if (empty($content)) {
            return '';
        }

        $doc = new DOMDocument();
        @$doc->loadHTML($content);
        $this->_childrenWalkRecursive($tool, $doc->documentElement->childNodes);

        return $doc->saveHTML();
    }

    protected function _childrenWalkRecursive(Tool $tool, DOMNodeList $childNodeList)
    {
        /** @var DOMElement $childNode */
        foreach ($childNodeList as $childNode) {
            if ($childNode->hasAttributes() === false && $childNode->hasChildNodes() === false) {
                continue;
            }

            foreach ($childNode->attributes as $attribute) {
                $popover = null;
                if ($attribute->name != 'data-popover-options') {
                    continue;
                }
                if (!$childNode->hasAttribute('href')) {
                    $childNode->removeAttribute('data-popover-options');
                    continue;
                }
                $options = json_decode($attribute->value, true);
                $popoverId = $childNode->getAttribute('data-popover');
                if (!empty($popoverId)) {
                    try {
                        /** @var Popover $popover */
                        $popover = PopoverRepository::findById($popoverId);
                    } catch (\Exception $exception) {
                        $childNode->removeAttribute('data-popover');
                        $childNode->removeAttribute($attribute->name);
                        continue;
                    }
                }
                if (!isset($popover)) {
                    $popoverId = PopoverController::createPopover(
                        $tool->getExposition(),
                        $tool->getWeave(),
                        $tool,
                        [
                            'title'         => !empty($options['title']) ? $options['title'] : $tool->getTitle() . '-' . $tool->getWeave()->incrementItemCounter(new Popover()),
                            'creation_type' => $options['creation_type'],
                            'popover'       => $options['popover'],
                            'text'          => $options['text'],
                            'autocontent'   => $options['autocontent'],
                            'viewon'        => $options['viewon'],
                            'position'      => $options['position'],
                            'background'    => $options['background'],
                        ]
                    )->getId();
                } elseif (!isset($options['changed'])) {
                    continue;
                } else {
                    $popover->setViewOn($options['viewon']);
                    $popover->setPosition($options['position']);
                    $popover->setBackground($options['background']);
                    $popover->setAutoContent($options['autocontent']);
                    $popover->setTitle(new Translation(!empty($options['title']) ? $options['title'] : $tool->getTitle()));
                    PopoverRepository::persist($popover);
                    $popoverId = $popover->getId();
                }

                $childNode->setAttribute('data-popover', $popoverId);
                $childNode->removeAttribute('data-popover-options');
            }

            if ($childNode->hasChildNodes() === true) {
                $this->_childrenWalkRecursive($tool, $childNode->childNodes);
            }
        }
    }

    /**
     * @throws Exception
     * @return Weave
     */
    protected function getRequestedWeave()
    {
        try {
            return $this->_getRequestedWeave();
        } catch (Exception $exception) {
            try {
                return $this->_loadRequestedObject('popover', $this->getParam('weave'));
            } catch (Exception $notUsed) {
                throw $exception;
            }
        }
    }

    /**
     * @param SimpleMedia $simpleMedia
     * @param Exposition $exposition
     * @throws InsufficientRightsException
     */
    protected function enforceRights(SimpleMedia $simpleMedia, Exposition $exposition)
    {
        if ($this->hasRights($simpleMedia, $exposition)) {
            return;
        }
        if ($exposition->hasShare($simpleMedia->getResearch())) {
            return;
        }
        if ($simpleMedia->hasWork() && $simpleMedia->getResearch()->isMetaEditableBy(Identity::getUser())) {
            $exposition->addShare($simpleMedia->getResearch());
            ExpositionRepository::persist($exposition);
            return;
        }
        throw new InsufficientRightsException('You have no permission to use ' . $simpleMedia->getId() . '!');
    }

    /**
     * @param SimpleMedia $simpleMedia
     * @param Exposition $exposition
     * @return bool
     */
    protected function hasRights(SimpleMedia $simpleMedia, Exposition $exposition)
    {
        if ($simpleMedia->hasExposition()) {
            return $simpleMedia->getResearch()->getId() == $exposition->getId();
        }
        if ($simpleMedia->hasWork() && $exposition->hasShare($simpleMedia->getResearch())) {
            return true;
        }
        return false;
    }

    /**
     * @param Exposition $exposition
     * @param ToolInterface[] $tools
     */
    protected function addWritePermissionsForNotes($exposition, array $tools)
    {
        if (!$exposition->isReviewer(Identity::getUser()) && !$exposition->isSupervisor(Identity::getUser())
            && !$exposition->hasApplicationRights(Identity::getUser())) {
            return;
        }

        foreach ($tools as $tool) {
            if (!$tool instanceof CommentTool) {
                return;
            }
        }

        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($exposition->getId());
    }

}