<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\SimpletextTool;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\TextTool;
use Rc\Domain\Model\Translation;
use Rc\Domain\Repository\Research\Exposition\PopoverRepository;
use Rc\Domain\Repository\Research\Exposition\ToolTemplateRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;
use Rc\Utility\PopoverHelper;

class PopoverController extends BasicControllerAction
{

    public function addAction()
    {
        $exposition = $this->_getRequestedResearch();
        $weave = $this->_getRequestedWeave();
        $form = $this->_getForm($weave);

        if ($form->isPostAndValid()) {
            $tool = $this->_getRequestedTool();
            $popover = self::createPopover($exposition, $weave, $tool, $form->getValues());

            $tool->setPopover($popover);
            ToolRepository::persist($tool);

            $this->updateResearchLastModified($popover->getExposition());

            $this->_setSuccessHeader();

            $this->view->popover = $popover;
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @param Exposition $research
     * @param Weave $weave
     * @param Tool $tool
     * @param array $values
     * @throws Exception
     * @throws InsufficientRightsException
     * @return Popover
     */
    public static function createPopover(Exposition $research, Weave $weave, Tool $tool, array $values)
    {
        if ($values['creation_type'] == 'duplicate') {
            $popover = clone PopoverRepository::findById($values['popover']);
        } else {
            $popover = new Popover();
        }
        $popover->setOwner(Identity::getUser());
        $popover->setTool($tool);
        $popover->setWeave($weave);
        $popover->setValues($values);

        if ($values['creation_type'] == 'text') {
            $tool = ToolFactory::buildByName(ToolFactory::TOOL_SIMPLE_TEXT);
            $toolTemplate = ToolTemplateRepository::findByNameAndResearch(ToolFactory::TOOL_SIMPLE_TEXT, $research);
            if (isset($toolTemplate)) {
                $formTemplate = ToolFormGenerator::getEditTemplate($toolTemplate->getTool(), $research)
                    ->setDefaultsFrom($toolTemplate->getTool());
                $tool->setValues($formTemplate->getValues());
            }

            $boundingBox = imageftbbox(12, 0, APPLICATION_PATH . '/../resources/font/arial.ttf', $values['text']);
            $textWidth = $boundingBox[2] - $boundingBox[0];
            if ($textWidth < 150) {
                $width = 150;
                $height = 25;
            } else {
                $currentTextWidth = null;
                $currentTextHeight = null;
                for ($lines = 1; ; ++$lines) {
                    $currentTextWidth = $textWidth / $lines;
                    $currentTextHeight = $lines * 21;
                    $currentRatio = $currentTextWidth / $currentTextHeight;
                    if ($currentRatio < 4 / 3) {
                        break;
                    }
                }
                $width = $currentTextWidth + 25;
                $height = $currentTextHeight;
            }

            $popover->setHeight($height + 20);
            $popover->setWidth($width + 20);

            $tool->setOwner(Identity::getUser());
            $tool->setTitle(new Translation($tool->getTitle() . '-' . $weave->incrementItemCounter($popover)));
            $tool->setWeave($popover);
            $tool->setLeft(10);
            $tool->setTop(10);
            $tool->setHeight($height);
            $tool->setWidth($width);
            $tool->setTextContent($values['text']);
            $tool->setOpacity(Tool::CSS_OPACITY_VISIBLE);
            $tool->setZIndex($popover->getMaxZIndex() + 1);
            $tool->setPosition(Tool::POSITION_CENTER_CENTER);
            ToolRepository::persist($tool);
        }

        PopoverRepository::persist($popover);

        return $popover;
    }

    public function editAction()
    {
        $this->_getRequestedResearch();
        $popover = $this->_getRequestedPopover();
        $form = $this->_getForm($popover)->setDefaultsFrom($popover);

        if ($form->isPostAndValid()) {
            $popover->setValues($form->getValues());
            PopoverRepository::persist($popover);

            $this->updateResearchLastModified($popover->getExposition());

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->popover = $popover;
        }
    }

    public function removeAction()
    {
        $research = $this->_getRequestedResearch();
        $popover = $this->_getRequestedPopover();
        $form = $this->_getForm($popover);

        if ($form->isPostAndValid()) {
            $tool = $popover->getTool();
            if ($tool instanceof TextTool || $tool instanceof SimpletextTool) {
                $tool->setTextContent($this->removePopoverFromTextContent($popover->getId(), $tool->getTextContent()));
            }

            if ($popover->hasTool()) {
                $tool->setPopover(null);
                ToolRepository::persist($tool);
            }

            $popover->setDeleted(true);
            PopoverRepository::persist($popover);

            $this->updateResearchLastModified($research);

            $this->_disableView();
        } else {
            $this->view->form = $form;
            $this->view->popover = $popover;
        }
    }

    /**
     * @todo
     */
    public function restoreAction()
    {
        $form = $this->_getForm();

        if ($form->isPostAndValid()) {
            $popover = PopoverRepository::findById($this->_getParam('popover')/*, $this->_getParam('revision')*/);
            $popover->setDeleted(false);

            if (!PopoverHelper::toolExists($popover)) {
                $popover->setTool(null);
            }

            PopoverRepository::persist($popover);

            $research = $popover->getExposition();
            $this->updateResearchLastModified($research);

            $this->view->popover = $popover;
            $this->_setSuccessHeader();
        } else {
            $this->view->form = $form;
        }
    }

    /**
     * @todo
     */
    public function restoreDeletedAction()
    {
        $form = $this->_getForm($this->_getRequestedWeave());

        $this->view->form = $form;
    }

    protected function removePopoverFromTextContent($popoverId, $content)
    {
        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");
        $content = str_replace('<p></p>', '<p><br/></p>', $content);

        $doc = new DOMDocument();
        $doc->loadHTML($content);
        $this->_childrenWalkRecursive($popoverId, $doc->documentElement->childNodes);

        return $doc->saveHTML();
    }

    protected function _childrenWalkRecursive($popoverId, DOMNodeList $childNodeList)
    {
        /** @var DOMElement $childNode */
        foreach ($childNodeList as $childNode) {
            if ($childNode->hasAttributes() === false && $childNode->hasChildNodes() === false) {
                continue;
            }
            foreach ($childNode->attributes as $attribute) {
                if ($attribute->name != 'data-popover') {
                    continue;
                }
                if ($attribute->value != $popoverId) {
                    continue;
                }
                $newNode = $childNode->ownerDocument->createElement('span');
                while ($childNode->childNodes->length > 0) {
                    $newNode->appendChild($childNode->childNodes->item(0));
                }
                $childNode->parentNode->replaceChild($newNode, $childNode);
            }

            if ($childNode->hasChildNodes() === true) {
                $this->_childrenWalkRecursive($popoverId, $childNode->childNodes);
            }
        }
    }

}