<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\Iterator\NoteFilterIterator;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\ContextRegistry;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Metrics\Metrics;
use Rc\Utility\ResearchUtility;

class ViewController extends BasicControllerAction
{

    /**
     * @var stdClass
     */
    protected $browser;

    // legacy support for old researches: [old weaveid => new researchid]
    private $mappingArray = [
        1452  => 3106,
        13100 => 4314,
        1036  => 7344,
        1199  => 7669,
        799   => 7736,
        1372  => 7967,
        1077  => 8199,
        1585  => 8247,
        372   => 8333,
        506   => 8563,
        317   => 9434,
        6622  => 10002,
        11567 => 11076,
        12061 => 11138,
        10325 => 11423,
        16068 => 11612,
        9774  => 11733,
        9097  => 12162,
        9528  => 12219,
        8669  => 12388,
        // 15565 => 12856, HfG Projects (unpublished)
    ];

    public function __call($method, $args)
    {
        if (ctype_digit(substr($method, 0, -6))) { // show research
            return $this->forward('index', null, null, $this->_extractParamsFromRequestUrl());
        } else {
            return parent::__call($method, $args);
        }
    }

    public function init()
    {
        $bc = new phpbrowscap\Browscap(cache_path());
        $bc->doAutoUpdate = false;
        $bc->lowercase = true;

        $this->browser = $bc->getBrowser();

        ContextRegistry::fromGlobals();

        parent::init();
    }

    public function hideNavigationTooltipAction()
    {
        $this->_disableView();

        if (!Identity::exists()) {
            return;
        }

        $settings = Identity::getUser()->getSettings();
        $settings->setHideNavigationTooltip(true);
        UserRepository::persist(Identity::getUser());
    }

    public function indexAction()
    {
        $isAnonymous = $this->_getParam('identity') == 'anonymous';

        // legacy support for old researches
        if (!$this->_getParam('research') && $this->_getParam('weave')) {
            $researchId = $this->mappingArray[$this->_getParam('weave')];
            $research = ExpositionRepository::getById($researchId);
            return $this->redirect($this->view->viewUrl($research), ['options' => 301]);
        }

        $this->handleTemporaryPermissions($this->_getParam('research'));

        $research = $this->_getRequestedResearchAndCheckForInvitation();
        if (!isset($research)) {
            return $research;
        }

        if ($research->isBlocked()) {
            return $this->_handleInsufficientRights();
        }

        if (!Identity::exists() && !$research->isPublished() && $research->getAccessLevel() == Exposition::ACCESS_LEVEL_LIMITED) {
            return $this->_handleInsufficientRights();
        }

        if (!Identity::exists() && $research->isPublished() && $research->getPublishRange() == 'limited') {
            return $this->_handleInsufficientRights();
        } elseif (Identity::exists() && $research->isPublished() && $research->getPublishRange() == 'limited' &&
            !in_array($research->getPortalId(), Identity::getUser()->getPortalIds())
        ) {
            return $this->_handleInsufficientRights();
        }

        try {
            $weave = $this->_getRequestedWeave();

            if (!in_array($weave, $research->getWeaves())) {
                throw new InsufficientRightsException('The request weave is not part of the exposition!');
            }
        } catch (InsufficientRightsException $e) {
            return $this->_handleInsufficientRights();
        }

        if ($research->isPublished()) {
            $this->view->portal = \Rc\Domain\Repository\PortalRepository::findById($research->getPortalId());
        } else {
            $this->view->portal = \Rc\Domain\Repository\PortalRepository::getDefault();
        }
        $this->view->research = $research;
        $this->view->weave = $weave;
        $this->view->application = $this->hasParam('application') ? $this->_getRequestedApplication() : null;
        $this->view->aspect = [
            'x' => $this->_getParam('x', $weave->getDefaultAspectX()),
            'y' => $this->_getParam('y', $weave->getDefaultAspectY()),
        ];
        $this->view->authors = $this->_getAuthorList($research);

        $additionalCss = '';
        if ($this->browser->platform == 'MacOSX' && $this->browser->browser == 'Firefox') {
            if (in_array($weave->getId(), [27898])) {
                $additionalCss = '.mceContentBody * {line-height: 1.41 !important}';
            } elseif (in_array($weave->getId(), [345])) {
                $additionalCss = '.mceContentBody * {line-height: 1.41 !important} div[data-id="14763"] *, div[data-id="15218"] * {line-height: 1.231 !important} div[data-id="16024"] * {line-height: 1.49 !important}';
            } elseif (in_array($weave->getId(), [16062])) {
                $additionalCss = 'div[data-id="16431"] * {line-height: 1.46 !important}';
            } elseif (in_array($weave->getId(), [16023])) {
                $additionalCss = '.mceContentBody * {line-height: 1.427 !important} div[data-id="14763"] *, div[data-id="15218"] * {line-height: 1.231 !important} div[data-id="16024"] * {line-height: 1.49 !important}';
            }
        }

        if ($this->browser && !($this->browser->platform === 'MacOSX' && $this->browser->browser === 'Safari')) {
            if (in_array($weave->getId(), [28574])) {
                $additionalCss = '.mceContentBody * {line-height: 118% !important}';
            }
        }

        if ($this->browser && $this->browser->browser === 'Chrome') {
            if (in_array($weave->getId(), [56735])) {
                $additionalCss = 'div[data-id="56736"] * {letter-spacing: -0.11px}';
            }
        }
        if ($this->browser && $this->browser->platform == 'MacOSX' && $this->browser->browser === 'Firefox') {
            if (in_array($weave->getId(), [56735])) {
                $additionalCss = 'div[data-id="56736"] * {letter-spacing: -0.11px}';
            }
        }

        if ($research->isPublished() && $research->getPublishedDate(false)->getTimestamp() < strtotime('2016-12-08')) {
            $additionalCss .= '
                #content { line-height: 1.25em; }
                .mceContentBody, .mceContentBody .x-window-mc { line-height: 1.25em; }
                .html-text-editor-content, .html-text-editor-content .x-window-mc { line-height: 1.25em; }            
            ';
        }

        $this->view->additionalCss = $additionalCss;
        $this->view->includeAdditionalCss = (!$research->isPublished() || $research->getPublishedDate(false)->getTimestamp() >= 1364462257 || in_array($research->getId(),
                [21732]));

        if (!Identity::exists() || $isAnonymous) {
            $this->view->showReviewingComments = false;
        } else {
            $person = Identity::getUser();
            $this->view->showReviewingComments = !($research->isStatus(Exposition::STATUS_REVIEW_PUBLISHED) || $research->isStatus(Exposition::STATUS_PUBLISHED)) &&
                (Identity::isAdmin() || $research->isAuthor($person) || $research->isCollaborateur($person) || $research->isReviewer($person)
                    || $research->isSupervisor($person) || $research->isEditor($person) || $research->isApplicationReader($person) || $research->isApplicationReviewer($person) || $research->isApplicationSupervisor($person));
        }

        $this->view->isEditable = $research->isEditableBy(Identity::getUserOrNull()) && !$isAnonymous;

        if ($weave instanceof TextWeave) {
            $this->render('nmh-view');
        }

        Metrics::increment('analytics.exposition_hits', 1, ResearchUtility::expositionMetricTags($research, Identity::getUserOrNull()));
    }

    public function popoverAction()
    {
        try {
            list(, , , $researchId, $popoverId) = explode('/', $_SERVER['REDIRECT_URL']);

            $this->handleTemporaryPermissions($researchId);

            /** @var \Rc\Domain\Model\Research\Exposition $research */
            $research = $this->_loadRequestedObject('research', $researchId);
            if ($research->isBlocked()) {
                throw new InsufficientRightsException();
            }
            $this->view->popover = $this->_loadRequestedObject('popover', $popoverId);
        } catch (Exception $e) {
            return $this->_handleInsufficientRights();
        }
    }

    public function thumbAction()
    {
        $this->handleTemporaryPermissions($this->_getParam('research'));

        $this->_getRequestedResearch(); // Important: rights to access the entity will be granted in the research constructor
        $weave = $this->_getRequestedWeave();
        $this->view->thumb = new WeaveThumb(new NoteFilterIterator(new ArrayIterator($weave->getItems()), $this->_getParam('commentfilter')), [
            'maxwidth'  => 500,
            'maxheight' => 350,
            'padding'   => $weave->getPadding(),
        ]);
        $this->render('weave/thumb', null, true);
    }

    private function _extractParamsFromRequestUrl()
    {
        // uri format: /view/RESEARCH_ID/WEAVE_ID/X/Y
        $pieces = explode('/', $_SERVER['REDIRECT_URL']);
        $pieces = $pieces + array_fill(0, 6, null);
        return [
            'research' => $pieces[2],
            'weave'    => $pieces[3],
            'x'        => $pieces[4],
            'y'        => $pieces[5],
        ];
    }

    private function _getAuthorList(\Rc\Domain\Model\Research\Exposition $research)
    {
        $retVal = [$research->getAuthor()];
        foreach ($research->getCoAuthors() as $id) {
            $retVal[] = UserRepository::findById($id);
        }
        return $retVal;
    }

    private function handleTemporaryPermissions($expositionId)
    {
        if ($this->hasParam('application') && $this->_getRequestedApplication()->containsBoundExpositionId($expositionId)) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($expositionId);
        }

        if (Identity::exists() && Identity::getUser()->isSuperAdmin()) {
            Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryReadGroupFor($expositionId);
        }
    }
}

?>