<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Application\ApplicationForm;
use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Application\ApplicationSnapshot;
use Rc\Domain\Model\Application\Form\FormElement;
use Rc\Domain\Model\Application\Form\LayoutElement;
use Rc\Domain\Model\Application\Generic\MetaDataAwareInterface;
use Rc\Domain\Model\Application\Review;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Message;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Degree;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Repository\Application\ApplicationFavoriteRepository;
use Rc\Domain\Repository\Application\ApplicationFormRepository;
use Rc\Domain\Repository\Application\ApplicationProgramRepository;
use Rc\Domain\Repository\PortalRepository;
use Rc\Domain\Repository\Research\ExpositionRepository;
use Rc\Domain\Repository\UserRepository;
use Rc\Utility\Strings;

class AdminFormGenerator extends BasicFormGenerator
{

    // User

    public static function getUserAdd(\Rc\Domain\Model\User $user = null, Portal $portal = null, $addReviewer = false)
    {
        $form = new DefaultForm();
        $form->setTitle($addReviewer ? 'dialog.reviewer.add' : 'dialog.user.add');
        $form->setAttrib('autocomplete', 'off');

        $profileTab = $form->addTab('profile')
            ->addElement('languageBar', 'language-bar')
            //                ->addElement('file', 'media', array(
            //                    'label' => 'field.profile.picture',
            //                    'validators' => array(
            //                        'IsImage',
            //                    ),
            //                ))
            ->addElement('text', 'name', [
                'required'                  => true,
                'label'                     => 'field.name',
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url' => '/user/search?hidden-profile=true',
                ]),
                'onchange'                  => 'handleExistingUser($(this).val())',
            ])
            ->addElement('text', 'alias', [
                'label' => 'field.alias',
            ])
            ->addElement('text', 'email', [
                'required'     => true,
                'label'        => 'field.email',
                'autocomplete' => 'nope',
                'validators'   => [
                    'EmailAddress',
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new UserRepository(),
                            'field'   => 'email',
                            'exclude' => [
                                'field' => 'id',
                                'value' => isset($user) ? $user->getId() : '',
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('password', 'password', [
                'required'     => true,
                'label'        => 'field.password',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'label'       => 'field.description',
                'data-editor' => true,
            ]);
        if (isset($portal) && $portal->isDefault()) {
            $profileTab
                ->addElement('checkbox', 'sarmember', [
                    'label' => 'field.sarmember',
                ])
                ->addElement('text', 'memberinstitution', [
                    'label' => 'field.memberinstitution',
                ]);
        }

        $form->addTab('portal')
            ->addElement('multiselect', 'portalids', [
                'required'         => true,
                'value'            => isset($portal) && !$portal->isDefault() ? [
                    key(static::_getDefaultPortal()),
                    $portal->getId(),
                ] : key(static::_getDefaultPortal()),
                'label'            => 'field.portal',
                'multiOptions'     => static::_getPortalList(false),
                'data-multiselect' => true,
            ]);

        $form->addTab('roles')
            ->addElement('checkbox', 'limitedUser', [
                'label'       => 'field.limitedUser',
                'description' => 'description.limitedUser',
                'value'       => 1,
            ]);

        $tab = $form->addTab('settings');

        $tab->addElement('select', 'language', [
            'label'        => 'field.language',
            'required'     => true,
            'multiOptions' => [
                'en' => 'English',
            ],
        ])
            ->addElement('checkbox', 'announcements', [
                'label'       => 'field.announcements',
                'description' => 'description.announcements',
                'value'       => !$addReviewer,
            ])
            ->addElement('checkbox', 'digest', [
                'label'       => 'field.digest',
                'description' => 'description.digest',
                'value'       => !$addReviewer,
            ])
            ->addElement('checkbox', 'uploadNotification', [
                'label'       => 'field.upload.notification',
                'description' => 'description.upload.notification',
                'value'       => 1,
            ])
            ->addElement('checkbox', 'messageNotification', [
                'label'       => 'field.message.notification',
                'description' => 'description.message.notification',
                'value'       => 1,
            ]);

        if (isset($user) && $user->isLimitedUser() && !$user->isInvolved()) {
            $tab->addElement('checkbox', 'hide', [
                'label'       => 'field.hideProfile',
                'description' => 'description.hideProfile',
            ]);
        }

        /*$tab->addElement('checkbox', 'madek', array(
            'label' => 'field.madek',
            'description' => 'description.madek',
        ));*/

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getUserAddToPortal(\Rc\Domain\Model\User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.user.addToPortal');
        $form->addElement('html', 'info', [
            'value' => sprintf(static::_getTranslate()->translate('info.user.addToPortal'), $user->getName()),
        ]);
        $form->addCancelButton('no')->addSubmitButton('yes');
        return $form;
    }

    public static function getUserEdit(\Rc\Domain\Model\User $user, Portal $portal = null)
    {
        $form = static::getUserAdd($user, $portal);
        $form->setTitle('dialog.user.edit', $user);

        $form->getTab('profile')->getElement('password')->setRequired(false);

        return $form;
    }

    public static function getUserRemove(\Rc\Domain\Model\User $user, Portal $portal)
    {
        $form = parent::getRemove();
        if ($portal->isDefault()) {
            $form->setTitle('dialog.user.remove', $user->getName());
        } else {
            $form->setTitle('dialog.user.remove-from-portal', $user->getName());
            $form->getElement('info')->setValue(static::_getTranslate()->translate('info.entry.remove-from-portal'));
        }

        return $form;
    }

    public static function getUserSwitch(\Rc\Domain\Model\User $user)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.user.switch', $user);

        $form->addElement('html', 'info', [
            'value' => sprintf(static::_getTranslate()->translate('info.user.switch'), $user->getName()),
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    // Application

    /**
     * @param Application $application
     * @param Portal $portal
     * @return DefaultForm
     */
    public static function getApplicationEdit(Application $application, Portal $portal)
    {
        $dateHelper = new My_View_Helper_Date();
        $urlHelper = new My_View_Helper_Url();

        $form = new DefaultForm();
        $form->setTitle('dialog.application.edit');

        $form->addTab('reviewing')
            ->addElement('multiselect', 'reviewerids', [
                'label'            => 'field.reviewers',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ])
            ->addElement('multiselect', 'externalreaderids', [
                'label'            => 'field.externalreaders',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ]);

        $rating = $form->addTab('rating');
        foreach ($application->getReviews(Review::STATUS_SUBMITTED) as $reviewKey => $review) {
            $singleRating = $rating->addAccordion('review-' . $reviewKey, [], [
                'title' => '<span style="float:right">' . $dateHelper->dateTime($review->getSubmitted()) . '</span>review by ' . $review->getAuthor()->getName(),
            ]);

            foreach ($review->getFormattedMetaData(MetaDataAwareInterface::KEY_IDENTIFIER_LABEL) as $label => $value) {
                if (empty($value)) {
                    continue;
                }

                if (is_array($value)) {
                    $value = '<ul><li>' . implode('<li>', $value) . '</ul>';
                }

                $singleRating->addElement('html', $label, [
                    'value' => '<strong>' . $label . ':</strong> ' . $value,
                ]);
            }
        }

        $snapshots = $form->addTab('snapshots');
        /** @var ApplicationSnapshot $snapshot */
        foreach (array_reverse($application->getSnapshots()) as $key => $snapshot) {
            $singleSnapshot = $snapshots->addAccordion('snapshot-' . $key, [], [
                'title' => $dateHelper->dateTime($snapshot->getDate(false)) . (empty($key) ? ' (latest)' : ''),
            ]);

            if (!empty($snapshot->getExpositions())) {
                $value = '';

                foreach ($snapshot->getExpositions() as $exposition) {
                    $value .= '<li><a href="' . $urlHelper->expositionUrl($exposition) . '" target="_blank">' . htmlentities($exposition->getTitle()) . '</a>';
                }

                $singleSnapshot->addElement('html', 'expositions', [
                    'value' => 'Expositions: <ul>' . $value . '</ul>',
                ]);
            }

            if (!empty($snapshot->getWorks())) {
                $value = '';

                foreach ($snapshot->getWorks() as $work) {
                    $value .= '<li><a href="' . $urlHelper->workUrl($work) . '" target="_blank">' . htmlentities($work->getTitle()) . '</a>';
                }

                $singleSnapshot->addElement('html', 'works', [
                    'value' => 'Works: <ul>' . $value . '</ul>',
                ]);
            }
        }

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param Application $application
     * @param MediaInterface[] $attachments
     * @return DefaultForm
     */
    public static function getApplicationAccept(Application $application, $attachments = [])
    {
        $files = [];
        foreach ($attachments as $name => $attachment) {
            $files[] = '<a href="' . $attachment->getMaster()->getExternalUri() . '" target="_blank">' . $name . '</a>';
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.application.accept');

        $form
            ->addElement('html', 'info', [
                'value' => static::_getTranslate()->translate('info.application.accept'),
            ])
            ->addElement('html', 'attachment', [
                'label' => 'field.attachment',
                'value' => implode(', ', $files),
                'tag'   => 'span',
            ])
            ->addElement('textarea', 'notes', [
                'label'       => 'field.messagetoauthor',
                'data-editor' => true,
            ])
            ->addElement('hidden', 'status', [
                'required' => true,
                'value'    => StatusAwareInterface::STATUS_PUBLISHED,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param \Rc\Domain\Model\Research\Application $application
     * @param Portal $portal
     * @param MediaInterface[] $attachments
     * @return DefaultForm
     */
    public static function getApplicationDecline(Application $application, Portal $portal, $attachments = [])
    {
        $files = [];
        foreach ($attachments as $name => $attachment) {
            $files[] = '<a href="' . $attachment->getMaster()->getExternalUri() . '" target="_blank">' . $name . '</a>';
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.application.decline');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.application.decline'),
        ])
            ->addElement('html', 'attachment', [
                'label' => 'field.attachment',
                'value' => implode(', ', $files),
                'tag'   => 'span',
            ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
                'value'    => static::_getTranslate()->translate('message.subject.' . Message::TYPE_APPLICATION_DECLINED),
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
                'value'       => vsprintf(static::_getTranslate()->translate('message.content.' . Message::TYPE_APPLICATION_DECLINED),
                    [
                        $application->getOwner()->getName(),
                        $portal->getName(),
                        $application->getName(),
                        $application->getTitle(),
                    ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param \Rc\Domain\Model\Research\Application $application
     * @param Portal $portal
     * @return DefaultForm
     */
    public static function getApplicationRework(Application $application, Portal $portal)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.application.decline');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.application.rework'),
        ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
                'value'    => static::_getTranslate()->translate('message.subject.' . Message::TYPE_APPLICATION_REWORK),
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
                'value'       => vsprintf(static::_getTranslate()->translate('message.content.' . Message::TYPE_APPLICATION_REWORK),
                    [
                        $application->getOwner()->getName(),
                        $portal->getName(),
                        $application->getName(),
                        $application->getTitle(),
                    ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getApplicationForward(Application $application)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.application.forward');

        $form
            ->addElement('html', 'info', [
                'value' => static::_getTranslate()->translate('info.application.forward'),
            ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
                'value'    => static::_getTranslate()->translate('message.subject.' . Message::TYPE_APPLICATION_FORWARDED),
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'data-editor' => true,
                'value'       => vsprintf(static::_getTranslate()->translate('message.content.' . Message::TYPE_APPLICATION_FORWARDED),
                    [
                        $application->getOwner()->getName(),
                        $application->getPortal()->getName(),
                        $application->getName(),
                        $application->getTitle(),
                    ]),
            ])
            ->addElement('hidden', 'status', [
                'required' => true,
                'value'    => StatusAwareInterface::STATUS_REVIEW,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getApplicationExport(Application $application)
    {
        $program = $application->getProgram();

        $form = new DefaultForm();
        $form->setTitle('dialog.application.export');

        $tab = $form->addTab('application-favorites');
        $favorites = [];
        foreach (ApplicationFavoriteRepository::findByProgram($program, ['limit' => 25]) as $favorite) {
            $favorites[] = '<tr>
		        <td>' . $favorite->getCreated('d/m/y - H:i:s') . '
		        <td>' . $favorite->getOwner()->getTitle() . '
		        <td>' . $favorite->getLabel() . '
		        <td><button  class="ui-button ui-widget ui-state-default ui-corner-all ui-button-text-icon-primary" style="margin:0" onclick="updateApplicationExportSelection(\'' . $favorite->getUuid() . '\'); return false;">Load & Export</button>';
        }
        if (!empty($favorites)) {
            $tab->addElement(
                'html',
                'favorites',
                [
                    'value' => '<table style="width:569px;margin-top:10px"><tr><th>created at<th>creator<th>label<th>' . implode(
                            null,
                            $favorites
                        ) . '</table>',
                ]
            );
        }
        $tab->addElement('text', 'label', [
            'label' => 'field.application.favorites',
            'info'  => 'info.application.favorites',
        ]);

        $tab = $form->addTab('application-metadata');
        $tab->addElement('checkbox', 'title', [
            'description' => 'title',
            'value'       => 1,
        ]);
        $tab->addElement('checkbox', 'program', [
            'description' => 'program',
            'value'       => 1,
        ]);
        $tab->addElement('checkbox', 'applicant', [
            'description' => 'applicant',
            'value'       => 1,
        ]);
        $tab->addElement('checkbox', 'created', [
            'description' => 'created',
            'value'       => 1,
        ]);

        $tab = $form->addTab('application-form');
        foreach ($program->getApplicationForm()->getFormElements() as $key => $element) {
            if ($element::FORM_TYPE == 'file') {
                continue;
            }
            $tab->addElement('checkbox', $key, [
                'description' => $element->getLabel(),
                'value'       => 1,
            ]);
        }

        $tab = $form->addTab('review-form');
        foreach ($program->getReviewForm()->getFormElements() as $key => $element) {
            if ($element::FORM_TYPE == 'file') {
                continue;
            }
            $tab->addElement('checkbox', $key, [
                'description' => $element->getLabel(),
                'value'       => 1,
            ]);
        }

        $form->addCancelButton()->addSubmitButton('export');

        return $form;
    }

    /**
     * @param MediaInterface[] $attachments
     * @return DefaultForm
     */
    public static function getApplicationReviewMail($attachments = [])
    {
        $files = [];
        foreach ($attachments as $name => $attachment) {
            $files[] = '<a href="' . $attachment->getMaster()->getExternalUri() . '" target="_blank">' . $name . '</a>';
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.application.review-mail');

        $form->addElement('text', 'receiver', [
            'label'    => 'field.receiver',
            'value'    => self::translate('field.externalreaders'),
            'readonly' => 'readonly',
        ])
            ->addElement('html', 'attachment', [
                'label' => 'field.attachment',
                'value' => implode(', ', $files),
                'tag'   => 'span',
            ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();
        return $form;
    }

    // Application Program

    /**
     * @param ApplicationProgram|null $applicationProgram
     * @param Portal $portal
     * @return DefaultForm
     */
    public static function getApplicationProgramAdd(ApplicationProgram $applicationProgram = null, Portal $portal)
    {
        $applicationProgram = $applicationProgram ?: new ApplicationProgram();
        $applicantPortals = self::getApplicantPortalList($portal);
        $accessGroups = ApplicationProgram::getAccessGroups();
        if (empty($applicantPortals)) {
            unset($accessGroups[ApplicationProgram::ACCESS_GROUP_APPLICANT_USER]);
        }

        $form = new DefaultForm();
        $form->setTitle('dialog.application-program.add');
        $form->getView()->jQuery()->addOnLoad('initApplicationProgramPortalFilter("#frm-meta-accessgroup", "#frm-meta-applicantportalids");');

        $meta = $form->addTab('meta');
        $meta
            ->addElement('text', 'title', [
                'required'   => true,
                'label'      => 'field.title',
                'validators' => [
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new ApplicationProgramRepository(),
                            'field'   => 'title',
                            'exclude' => [
                                'field' => 'id',
                                'value' => $applicationProgram->getId(),
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('textarea', 'description', [
                'required'   => true,
                'label'      => 'field.description',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            ApplicationProgram::VAL_DESCRIPTION_LENGTH_MIN,
                            ApplicationProgram::VAL_DESCRIPTION_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('select', 'accessgroup', [
                'label'        => 'field.access-group',
                'required'     => true,
                'multiOptions' => ['' => ''] + $accessGroups,
            ])
            ->addElement('multiselect', 'applicantportalids', [
                'label'        => 'field.applicant-portal',
                'multiOptions' => $applicantPortals,
            ])
            ->addInlineElement([
                $meta->createElement('datePicker', 'startdate', [
                    'label' => 'field.start-date',
                    'style' => 'width: 175px',
                ]),
                $meta->createElement('text', 'starttime', [
                    'placeholder' => 'hh:mm:ss',
                    'style'       => 'width: 175px',
                    'value'       => '00:00:00',
                ]),
            ])
            ->addInlineElement([
                $meta->createElement('datePicker', 'enddate', [
                    'label' => 'field.end-date',
                    'style' => 'width: 175px',
                ]),
                $meta->createElement('text', 'endtime', [
                    'placeholder' => 'hh:mm:ss',
                    'style'       => 'width: 175px',
                    'value'       => '00:00:00',
                ]),
            ])
            ->addElement('select', 'status', [
                'label'        => 'field.status',
                'multiOptions' => ApplicationProgram::getStatuses(),
            ])
            ->addElement('file', 'guidelines', [
                'label' => 'field.extended-guidelines',
            ])
            ->addElement('checkbox', 'publishreview', [
                'label' => 'field.publish-review',
            ])
            ->addElement('text', 'requireddocuments', [
                'label'       => 'field.required-documents',
                'data-tagit'  => '{"allowSpaces": true}',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ])
            ->addElement('checkbox', 'allowanydocuments', [
                'label' => 'field.allow-any-documents',
            ])
            ->addElement('text', 'furtherinformation', [
                'label'       => 'field.further-information',
                'placeholder' => 'https://www.example.com',
            ]);

        $form->addTab('persons')
            ->addElement('multiselect', 'contactpersonids', [
                'label'                     => 'field.contact-persons',
                'required'                  => true,
                'multiOptions'              => static::_getLazyUserList(static::_getParam('persons.contactpersonids',
                    $applicationProgram->getContactPersonIds()), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ])
            ->addElement('multiselect', 'adminids', [
                'label'                     => 'field.admins',
                'required'                  => true,
                'multiOptions'              => static::_getLazyUserList(static::_getParam('persons.adminids',
                    $applicationProgram->getAdminIds()), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ]);

        $form->addTab('application')
            ->addElement('number', 'requiredexpositions', [
                'required'  => false,
                'label'     => 'field.required-expositions',
                'min'       => 0,
                'inclusive' => true,
            ])
            ->addElement('number', 'requiredworks', [
                'required'  => false,
                'label'     => 'field.required-works',
                'min'       => 0,
                'inclusive' => true,
            ])
            ->addElement('select', 'applicationformid', [
                'label'        => 'field.application-form',
                'required'     => true,
                'multiOptions' => ['' => ''] + static::_getApplicationFormList($portal,
                        ApplicationForm::GENRE_APPLICATION),
            ])
            ->addElement('select', 'reviewformid', [
                'label'        => 'field.review-form',
                'required'     => true,
                'multiOptions' => ['' => ''] + static::_getApplicationFormList($portal, ApplicationForm::GENRE_REVIEW),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param ApplicationProgram $applicationProgram
     * @param Portal $portal
     * @return DefaultForm
     */
    public static function getApplicationProgramEdit(ApplicationProgram $applicationProgram, Portal $portal)
    {
        $form = self::getApplicationProgramAdd($applicationProgram, $portal);
        $form->setTitle('dialog.application-program.edit', $applicationProgram);

        return $form;
    }

    /**
     * @param ApplicationProgram $applicationProgram
     * @return DefaultForm
     */
    public static function getApplicationProgramRemove(ApplicationProgram $applicationProgram = null)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.application-program.remove', $applicationProgram);
        return $form;
    }

    /**
     * @param ApplicationProgram $applicationProgram
     * @return DefaultForm
     */
    public static function getApplicationProgramRemoveError(ApplicationProgram $applicationProgram = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.application-program.remove-error', $applicationProgram);
        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.application-program.remove-error'),
        ]);
        $form->addCancelButton('ok');
        return $form;
    }

    // Application Form

    /**
     * @param ApplicationForm|null $applicationForm
     * @return ApplicationFormDefaultForm
     */
    public static function getApplicationFormAdd(ApplicationForm $applicationForm = null)
    {
        $form = new ApplicationFormDefaultForm();
        $form->setTitle('dialog.application-form.add');
        $form->getView()->jQuery()->addOnLoad('initFormElementCreator($("#tabprototype-container"));');

        $form->addTab('meta')
            ->addElement('hidden', 'genre', [
                'required'   => true,
                'validators' => [
                    [
                        'InArray',
                        false,
                        [
                            array_keys(ApplicationForm::getGenres()),
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'title', [
                'required'   => true,
                'label'      => 'name',
                'validators' => [
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new ApplicationFormRepository(),
                            'field'   => 'title',
                            'exclude' => [
                                'field' => 'id',
                                'value' => (isset($applicationForm) ? $applicationForm->getId() : ''),
                            ],
                        ],
                    ],
                ],
            ]);

        $elementsTabs = $form->addTab('elements');
        $elementsTabs->addElement('select', 'form-element-creator', [
            'multiOptions' => ['' => ''] + [
                    'form'   => FormElement::getElements(),
                    'layout' => LayoutElement::getElements(),
                ],
            'style'        => 'float:right;width:200px',
        ]);
        $elementsTabs->addElement('html', 'title-info', [
            'value' => '<i class="ui-icon ui-icon-info"></i> A default title element (short-text) is added by default!',
            'style' => 'float:left',
        ]);

        $prototypeTab = $form->addTab('prototype');
        self::addFormElementAccordions($prototypeTab);

        if (!isset($applicationForm)) {
            $form->appendFormElement('short-text', 'placeholder');
        }

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param ApplicationForm $applicationForm
     * @return ApplicationFormDefaultForm
     */
    public static function getApplicationFormEdit(ApplicationForm $applicationForm)
    {
        $form = self::getApplicationFormAdd($applicationForm);
        $form->setTitle('dialog.application-form.edit');

        if (!self::_getRequest()->isPost()) {
            foreach ($applicationForm->getElements() as $key => $element) {
                $form->appendFormElement(Strings::toCamelCase($element->getName(), '-'), $key);
            }
        }

        return $form;
    }

    /**
     * @param ApplicationForm $applicationForm
     * @return DefaultForm
     */
    public static function getApplicationFormRemove(ApplicationForm $applicationForm = null)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.application-form.remove', $applicationForm);
        return $form;
    }

    /**
     * @param ApplicationForm $applicationForm
     * @return DefaultForm
     */
    public static function getApplicationFormDuplicate(ApplicationForm $applicationForm = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.application-form.duplicate', $applicationForm);

        $form->addElement('text', 'title', [
            'required'   => true,
            'label'      => 'name',
            'validators' => [
                [
                    'NoRecordExists',
                    false,
                    [
                        'adapter' => new ApplicationFormRepository(),
                        'field'   => 'title',
                        'exclude' => [
                            'field' => 'id',
                            'value' => (isset($applicationForm) ? $applicationForm->getId() : ''),
                        ],
                    ],
                ],
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    // Portal

    public static function getPortalAdd(Portal $portal = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.portal.add');

        $form->addTab('common')
            ->addElement('file', 'media', [
                'label'      => 'field.logo',
                'validators' => [
                    ['MimeType', false, 'image'],
                ],
            ])
            ->addElement('text', 'name', [
                'required'   => true,
                'label'      => 'field.name',
                'validators' => [
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new PortalRepository(),
                            'field'   => 'name',
                            'exclude' => [
                                'field' => 'uuid',
                                'value' => (isset($portal) ? $portal->getUuid() : ''),
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'shortname', [
                'required'   => true,
                'label'      => 'field.shortname',
                'validators' => [
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new PortalRepository(),
                            'field'   => 'shortName',
                            'exclude' => [
                                'field' => 'uuid',
                                'value' => (isset($portal) ? $portal->getUuid() : ''),
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'url', [
                'required' => true,
                'label'    => 'field.url',
            ])
            ->addElement('textarea', 'description', [
                'required' => true,
                'label'    => 'field.description',
                'style'    => 'height: 47px',
            ])
            ->addElement('textarea', 'longdescription', [
                'required' => true,
                'label'    => 'field.longdescription',
            ])
            ->addElement('textarea', 'viewtext', [
                'required' => true,
                'label'    => 'field.viewtext',
                'value'    => '$author, ‘$title‘, <i>$portal</i>, $edition ($published) <a href="$url">$url</a> [accessed $now]',
                'info'     => 'available placeholders: $author, $title, $portal, $edition, $published, $url, $now',
                'style'    => 'height: 47px',
            ]);

        $form->addTab('roles')
            ->addElement('multiselect', 'adminids', [
                'label'                          => 'field.admins',
                'multiOptions'                   => static::_getLazyUserList(static::_getParam('roles.adminids',
                    (isset($portal) ? $portal->getAdminIds() : [])), false),
                'data-autocomplete'              => true,
                'data-autocomplete-options'      => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
                'concatJustValuesInErrorMessage' => true,
            ])
            ->addElement('multiselect', 'reviewerids', [
                'label'                     => 'field.reviewers',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('roles.reviewerids',
                    (isset($portal) ? $portal->getReviewerIds() : [])), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ])
            ->addElement('multiselect', 'contactpersonids', [
                    'label'                     => 'field.contactpersons',
                    'multiOptions'              => static::_getLazyUserList(static::_getParam('roles.contactpersonids',
                        (isset($portal) ? $portal->getContactPersonIds() : [])),
                        false),
                    'data-autocomplete'         => true,
                    'data-autocomplete-options' => json_encode(
                        [
                            'url'         => '/user/search?hidden-profile=true',
                            'placeholder' => 'Add user...',

                        ]
                    ),
                ]
            )
            ->addElement('textarea', 'externalcontactpersons', [
                'label'       => 'field.externalcontactpersons',
                'placeholder' => 'firstname lastname <email>',
            ]);

        if (isset($portal)) {
            $form->addTab('templates')
                ->addElement('html', 'info', [
                    'value' => '<span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span> Here you can specify which templates are available on exposition creation. All text-based expositions from portal admins are available.',
                    'style' => 'margin-top: 10px',
                ])
                ->addElement('multiselect', 'textexpositiontemplateids', [
                    'label'            => '',
                    'multioptions'     => self::getTextExpositionTemplateOptions($portal),
                    'data-multiselect' => true,
                    'style'            => 'height:300px',
                ]);
        }

        $form->addTab('doimeta')
            ->addElement('html', 'info', [
                'value' => '<span style="float: left; margin-right: .3em" class="ui-icon ui-icon-info"></span> You have to specify a DOI abbreviation. Additionally you have to specify either a portal DOI or an ISSN.',
                'style' => 'margin-top: 10px',
            ])
            ->addElement('text', 'doiabbreviation', [
                'label'      => 'field.doi-abbrevitation',
                'onchange'   => 'checkDoiRequirements()',
                'maxlength'  => 6,
                'validators' => [
                    ['Alnum'],
                    [
                        'NoRecordExists',
                        false,
                        [
                            'adapter' => new PortalRepository(),
                            'field'   => 'doiAbbrevitation',
                            'exclude' => [
                                'field' => 'uuid',
                                'value' => (isset($portal) ? $portal->getUuid() : ''),
                            ],
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'doi', [
                'label'    => 'field.doi',
                'onchange' => 'checkDoiRequirements()',
            ])
            ->addElement('text', 'issn', [
                'label'    => 'field.issn',
                'onchange' => 'checkDoiRequirements()',
            ])
            ->addElement('checkbox', 'allowExpositionDoiDeposit', [
                'label'       => ' ',
                'description' => 'description.allowExpositionDoiDeposit',
            ]);

        if (!Identity::isSuperAdmin()) {
            $form->clearAndDisableTab(['doimeta']);
        }

        $tab = $form->addTab('submissions');
        $tab->addElement('html', 'info-misc', [
            'value' => static::_getTranslate()->translate('field.allow'),
            'style' => 'margin: 15px 0 -10px',
        ]);

        $submissionElements = [
            $tab->createElement('checkbox', 'allowProfile', [
                'description' => 'description.allowProfile',
            ]),
            $tab->createElement('checkbox', 'allowLinkExternal', [
                'description' => 'description.allowLinkExternal',
            ]),
            $tab->createElement('checkbox', 'allowMembershipSelection', [
                'description' => 'description.allowMembershipSelection',
            ]),
        ];
        if (Identity::isSuperAdmin()) {
            $submissionElements[] = $tab->createElement('checkbox', 'allowApplicationProgram', [
                'description' => 'description.allowApplicationProgram',
            ]);
        }
        $tab->addInlineElement($submissionElements);
        $tab->addElement('html', 'info-partof', [
            'value' => static::_getTranslate()->translate('field.allowPartOf'),
            'style' => 'margin: 15px 0 -10px',
        ]);
        $tab->addInlineElement([
            $tab->createElement('checkbox', 'allowExpositionConnecting', [
                'description' => 'description.allowPartOf.research',
            ]),
            $tab->createElement('checkbox', 'allowProjectConnecting', [
                'description' => 'description.allowPartOf.project',
            ]),
            $tab->createElement('checkbox', 'allowDegreeConnecting', [
                'description' => 'description.allowPartOf.degree',
            ]),
            $tab->createElement('checkbox', 'allowWorkConnecting', [
                'description' => 'description.allowPartOf.work',
            ]),
        ]);
        $tab->addElement('html', 'info-publishing', [
            'value' => static::_getTranslate()->translate('field.allowPublishing'),
            'style' => 'margin: 15px 0 -10px',
        ]);

        $publishingElements = [
            $tab->createElement('checkbox', 'allowExpositionPublishing', [
                'description' => 'description.allowPublishingOf.research',
                'onchange'    => '$(this).is(":checked") ? $("#frm-submissions-allowLimitedExpositionPublishing,#frm-submissions-allowExternalContent").removeAttr("disabled") : $("#frm-submissions-allowLimitedExpositionPublishing,#frm-submissions-allowExternalContent").removeAttr("checked").attr("disabled", "disabled")',
            ]),
            $tab->createElement('checkbox', 'allowExternalContent', [
                'description' => 'description.allowExternalContent',
            ]),
            $tab->createElement('checkbox', 'allowLimitedExpositionPublishing', [
                'description' => 'description.allowLimitedPublishingOf.research',
            ]),
            $tab->createElement('checkbox', 'allowDegreePublishing', [
                'description' => 'description.allowPublishingOf.degree',
            ]),
        ];
        if (Identity::isSuperAdmin()) {
            $publishingElements[] = $tab->createElement('checkbox', 'allowApplicationPublishing', [
                'description' => 'description.allowPublishingOf.application',
                'onchange'    => '$(this).is(":checked") ? $("#frm-submissions-allowApplicationProgram").removeAttr("disabled") : $("#frm-submissions-allowApplicationProgram").removeAttr("checked").attr("disabled", "disabled")',
            ]);
        }
        $tab->addInlineElement($publishingElements);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getPortalEdit(Portal $portal)
    {
        $form = static::getPortalAdd($portal);
        $form->setTitle('dialog.portal.edit', $portal);

        return $form;
    }

    public static function getPortalSendMail(Portal $portal, $data = [])
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.portal.send.mail');

        $scope = [
            'all'          => 'field.all',
            'sarmember'    => 'field.sarmember',
            'portaladmins' => 'field.portaladmins',
            'fullaccounts' => 'field.full.accounts',
        ];
        if (!$portal->isDefault()) {
            unset($scope['sarmember']);
        }

        $template = [];
        foreach (NewsletterTemplateRepository::findAll() as $newsletterTemplate) {
            $template[$newsletterTemplate->getId()] = $newsletterTemplate->getDescription();
        }

        $required = !empty($data['mail']['template']);

        $form->addTab('mail')
            ->addElement('hidden', 'portal', [
                'value' => $portal->getId(),
            ])
            ->addElement('checkbox', 'testmode', [
                'label'       => 'field.test',
                'description' => 'sends portal mail only to <b>' . Identity::getUser()->getEmail() . '</b>',
                'value'       => true,
            ])
            ->addElement('html', 'spacer1', [
                'value' => '<div style="border-bottom: 1px solid black"></div>',
            ])
            ->addElement('text', 'receiver', [
                'value'    => 'Members of portal "' . $portal->getName() . '"',
                'readonly' => true,
                'label'    => 'field.receiver',
            ])
            ->addElement('select', 'target', [
                'label'        => 'field.type',
                'multioptions' => [
                    'default'      => 'field.default',
                    'announcement' => 'field.announcement',
                    'digest'       => 'field.digest',
                ],
                'value'        => 'default',
                'onchange'     => '$(this).val() == "default" ? $("[name=scope]").removeAttr("disabled") : $("[name=scope]").attr("disabled","disabled")',
            ])
            ->addElement('select', 'scope', [
                'label'        => 'field.scope',
                'multioptions' => $scope,
                'value'        => 'all',
            ])
            ->addElement('select', 'template', [
                'label'        => 'field.template',
                'multioptions' => ['' => ''] + $template,
            ])
            ->addElement('html', 'spacer2', [
                'value' => '<div style="border-bottom: 1px solid black"></div>',
            ])
            ->addElement('text', 'subject', [
                'required' => true,
                'label'    => 'field.subject',
                'value'    => '',
            ])
            ->addElement('file', 'banner', [
                'label'    => 'field.banner',
                'required' => $required,
            ])
            ->addElement('html', 'usebannerhelp', [
                'value' => '<label>image</label><a href="">change</a>',
                'tag'   => 'p',
            ])
            ->addElement('text', 'title', [
                'label'    => 'field.message.title',
                'value'    => '',
                'required' => $required,
            ])
            ->addElement('text', 'sender', [
                'label'       => 'field.sender',
                'value'       => '',
                'placeholder' => 'user or organisation (free text)',
                'required'    => $required,
            ])
            ->addElement('text', 'contact', [
                'label'       => 'field.contact',
                'value'       => '',
                'placeholder' => 'rc user link, rc portal link, mail address or free text',
                'required'    => $required,
            ])
            ->addElement('datePicker', 'delivery', [
                'label'        => 'field.delivery',
                'value'        => '',
                'jQueryParams' => [
                    'minDate' => date('d/m/Y', time() + 86400),
                    'maxDate' => date('d/m/Y', time() + 86400 * 30 * 3),
                ],
                'required'     => $required,
            ])
            ->addElement('textarea', 'content', [
                'required'    => true,
                'label'       => 'field.content',
                'data-editor' => true,
                'value'       => '',
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getPortalMailAccept(Newsletter $newsletter)
    {
        $form = new DefaultForm();

        $form->setTitle('dialog.newsletter.accept', $newsletter);
        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.newsletter.accept'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getPortalMailDecline(Newsletter $newsletter)
    {
        $form = new DefaultForm();

        $form->setTitle('dialog.newsletter.decline', $newsletter);
        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.newsletter.decline'),
        ])
            ->addElement('hidden', 'confirmation', [
                'value'    => 'confirmation',
                'required' => true,
            ]);

        $form->addCancelButton('no')->addSubmitButton('yes');

        return $form;
    }

    public static function getPortalRemove(Portal $portal)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.portal.remove', $portal);

        return $form;
    }

    // Issue

    public static function getIssueAdd(Issue $issue = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.issue.add');

        $form->addTab('common')
            ->addElement('file', 'media', [
                'label' => 'field.mediathumb',
            ])
            ->addElement('number', 'number', [
                'required' => true,
                'label'    => 'field.number',
            ])
            ->addElement('text', 'title', [
                'required' => true,
                'label'    => 'field.title',
            ])
            ->addElement('checkbox', 'published', [
                'required' => true,
                'label'    => 'field.published',
            ])
            ->addElement('textarea', 'description', [
                'required' => true,
                'label'    => 'field.description',
                'style'    => 'height: 47px',
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getIssueEdit(Issue $issue)
    {
        $form = static::getIssueAdd($issue);
        $form->setTitle('dialog.issue.edit', $issue);

        return $form;
    }

    public static function getIssueRemove(Issue $issue)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.issue.remove', $issue);

        return $form;
    }

    // Updates

    public static function getUpdateAdd(Update $update = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.update.add');

        $form->addTab('common')
            ->addElement('text', 'subject', [
                'required' => true,
                'label'    => 'field.subject',
            ])
            ->addElement('select', 'terminated', [
                'label'        => 'field.state',
                'multiOptions' => [
                    'field.inprogress',
                    'field.terminated',
                ],
            ])
            ->addElement('textarea', 'content', [
                'required'    => true,
                'label'       => 'field.content',
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getUpdateEdit(Update $update)
    {
        $form = static::getUpdateAdd($update);
        $form->setTitle('dialog.update.edit');

        if (!Identity::getUser()->isSuperAdmin()) {
            $form->getTab('common')->getElement('terminated')->setAttrib('disabled', 'disabled');
        }

        return $form;
    }

    public static function getUpdateRemove(Update $update)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.update.remove', $update);

        return $form;
    }

    // Reviewing

    public static function getReviewingEdit(Exposition $research = null, Portal $portal = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.edit');

        $form->addTab('reviewing')
            ->addElement('multiselect', 'reviewerids', [
                'label'            => 'field.reviewers.single.blind',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ])
            ->addElement('multiselect', 'doubleblindreviewerids', [
                'label'            => 'field.reviewers.double.blind',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ])
            ->addElement('text', 'reviewtag', [
                'label' => 'field.reviewtag',
            ])
            ->addElement('textarea', 'reviewcomment', [
                'label' => 'field.reviewcomment',
                'style' => 'height: 125px',
            ]);

        $supervisors = static::_getLazyUserList(static::_getParam('collaboration.supervisors',
            $research->getSupervisors()), false);
        foreach ($supervisors as $id => $supervisor) {
            if ($research->isInvitePending($id, 'supervisor')) {
                $supervisors[$id] = '[' . $supervisor . ']';
            } elseif ($research->isDisapprovedCollaborator($id, 'supervisor')) {
                $supervisors[$id] = '{' . $supervisor . '}';
            }
        }

        $form->addTab('collaboration')
            ->addElement('multiselect', 'supervisors', [
                'label'                     => 'field.supervisor',
                'multiOptions'              => $supervisors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ])
            ->addElement('multiselect', 'editors', [
                'label'                     => 'field.editor',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('collaboration.editors',
                    $research->getEditors()), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getReviewingAccept(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.accept');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.reviewing.accept'),
        ]);

        if ($research->isStatus(Exposition::STATUS_REVIEW)) {
            $form->addElement('select', 'portaledition', [
                'label'        => 'field.portal.edition',
                'required'     => true,
                'multioptions' => ['' => ''] + self::_getIssueList($research->getPortal()),
            ])
                ->addElement('select', 'publishrange', [
                    'label'        => 'field.publishrange',
                    'multioptions' => ['unlimited' => 'unlimited', 'limited' => 'limited to portal'],
                    'required'     => true,
                    'value'        => $research->getPublishRange(),
                ]);
        }

        $form->addElement('textarea', 'notes', [
            'label'       => 'field.additional.notes',
            'data-editor' => true,
        ])
            ->addElement('hidden', 'status', [
                'required' => true,
                'value'    => Exposition::STATUS_PUBLISHED,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getReviewingDecline(Exposition $research, Portal $portal)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.decline');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.reviewing.decline'),
        ])
            ->addElement('text', 'subject', [
                'label'    => 'field.subject',
                'required' => true,
                'value'    => static::_getTranslate()->translate('message.subject.' . Message::TYPE_SUBMISSION_DECLINED),
            ])
            ->addElement('textarea', 'content', [
                'label'       => 'field.content',
                'required'    => true,
                'data-editor' => true,
                'value'       => vsprintf(static::_getTranslate()->translate('message.content.' . Message::TYPE_SUBMISSION_DECLINED),
                    [
                        $research->getAuthor()->getName(),
                        $portal->getName(),
                        $research->getName(),
                        $research->getTitle(),
                    ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getReviewingEditDegree(Degree $degree = null, Portal $portal = null)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.edit');

        $form->addTab('reviewing')
            ->addElement('multiselect', 'reviewerids', [
                'label'            => 'field.reviewers.single.blind',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ])
            ->addElement('multiselect', 'doubleblindreviewerids', [
                'label'            => 'field.reviewers.double.blind',
                'multiOptions'     => static::_getReviewerList($portal),
                'data-multiselect' => true,
            ])
            ->addElement('text', 'reviewtag', [
                'label' => 'field.reviewtag',
            ])
            ->addElement('textarea', 'reviewcomment', [
                'label' => 'field.reviewcomment',
                'style' => 'height: 125px',
            ]);

        $supervisors = static::_getLazyUserList(static::_getParam('collaboration.supervisors',
            $degree->getSupervisors()), false);
        foreach ($supervisors as $id => $supervisor) {
            if ($degree->isInvitePending($id, 'supervisor')) {
                $supervisors[$id] = '[' . $supervisor . ']';
            } elseif ($degree->isDisapprovedCollaborator($id, 'supervisor')) {
                $supervisors[$id] = '{' . $supervisor . '}';
            }
        }

        $form->addTab('collaboration')
            ->addElement('multiselect', 'supervisors', [
                'label'                     => 'field.supervisor',
                'multiOptions'              => $supervisors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ])
            ->addElement('multiselect', 'editors', [
                'label'                     => 'field.editor',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('collaboration.editors',
                    $degree->getEditors()), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getReviewingAcceptDegree(Degree $degree)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.accept');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.reviewing.accept'),
        ]);

        $form->addElement('textarea', 'notes', [
            'label'       => 'field.additional.notes',
            'data-editor' => true,
        ])
            ->addElement('hidden', 'status', [
                'required' => true,
                'value'    => Exposition::STATUS_PUBLISHED,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getReviewingDeclineDegree(Degree $degree)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.reviewing.decline');

        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.reviewing.decline'),
        ])
            ->addElement('textarea', 'notes', [
                'label'       => 'field.additional.notes',
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    // Research

    public static function getResearchPortalComment(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.portal.comment');

        $form
            ->addElement('hidden', 'researchid', [
                'value' => $research->getId(),
            ])
            ->addElement('multiselect', 'userid', [
                'label'                     => 'field.user',
                'required'                  => true,
                'multiOptions'              => static::_getLazyUserList(static::_getParam('userid', []), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search?hidden-profile=true',
                    'placeholder' => 'Add user...',
                    'maxcount'    => 1,
                ]),
            ])
            ->addElement('textarea', 'content', [
                'required'    => true,
                'label'       => 'field.content',
                'data-editor' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getResearchEdit(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.edit');

        $form->addTab('meta')
            ->addElement('languageBar', 'language-bar')
            ->addElement('file', 'preview', [
                'label' => 'field.mediathumb',
            ])
            ->addMultiLanguageElement('text', 'title', [
                'required'   => true,
                'label'      => 'field.title',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Research::VAL_TITLE_LENGTH_MIN,
                            Research::VAL_TITLE_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('datePicker', 'date', [
                'required' => true,
                'value'    => date(static::_getLocale()->getDefaultDateFormatForPhp()),
                'label'    => 'field.date',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'required'   => true,
                'label'      => 'field.abstract',
                'validators' => [
                    [
                        'StringLength',
                        false,
                        [
                            Research::VAL_DESCRIPTION_LENGTH_MIN,
                            Research::VAL_DESCRIPTION_LENGTH_MAX,
                        ],
                    ],
                ],
            ])
            ->addElement('text', 'affiliation', [
                'label' => 'field.affiliation',
            ])
            ->addElement('text', 'keywords', [
                'label'       => 'field.keywords',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ]);

        $supervisors = static::_getLazyUserList(static::_getParam('collaboration.supervisors',
            $research->getSupervisors()), false);
        foreach ($supervisors as $id => $supervisor) {
            if ($research->isInvitePending($id, 'supervisor')) {
                $supervisors[$id] = '[' . $supervisor . ']';
            } elseif ($research->isDisapprovedCollaborator($id, 'supervisor')) {
                $supervisors[$id] = '{' . $supervisor . '}';
            }
        }

        $form->addTab('portal')
            ->addElement('html', 'hint', [
                'value' => '<p>' . static::_getTranslate()->translate('info.research.portal') . '</p>',
            ])
            ->addElement('select', 'partofportalid', [
                'label'        => 'field.assign.to.portal',
                'multioptions' => ['' => ''] + static::_getPortalList(),
            ]);

        if ($research->isPublished()) {
            $form->addTab('publication')
                ->addElement('select', 'portalissueid', [
                    'label'        => 'field.portal.edition',
                    'required'     => true,
                    'multioptions' => ['' => ''] + self::_getIssueList($research->getPortal()),
                ])
                ->addElement('select', 'publishrange', [
                    'label'        => 'field.publishrange',
                    'multioptions' => ['unlimited' => 'unlimited', 'limited' => 'limited to portal'],
                    'required'     => true,
                    'value'        => $research->getPublishRange(),
                ]);
        }

        $form->addTab('collaboration')
            ->addElement('multiselect', 'supervisors', [
                'label'                     => 'field.supervisor',
                'multiOptions'              => $supervisors,
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add user...',
                ]),
            ])
            ->addElement('multiselect', 'editors', [
                'label'                     => 'field.editor',
                'multiOptions'              => static::_getLazyUserList(static::_getParam('collaboration.editors',
                    $research->getEditors()), false),
                'data-autocomplete'         => true,
                'data-autocomplete-options' => json_encode([
                    'url'         => '/user/search',
                    'placeholder' => 'Add user...',
                ]),
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getResearchBlock(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.block', $research);

        $form->addElement('select', 'blocked', [
            'required'     => true,
            'label'        => 'field.status',
            'multiOptions' => [
                '1' => 'block',
                '0' => 'unblock',
            ],
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getPartOfRequestsAccept(Research $entity)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.' . $entity->getName() . '.accept', $entity);

        $form->addElement('textarea', 'notes', [
            'label'       => 'field.additional.notes',
            'data-editor' => true,
        ])->addElement('hidden', 'partofportalrequestid', [
            'value' => $entity->getPartOfPortalRequestId(),
        ])->addElement('hidden', 'partofportalid', [
            'value' => $entity->getPartOfPortalRequestId(),
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getPartOfRequestsDecline(Research $entity)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.' . $entity->getName() . '.decline', $entity);

        $form->addElement('textarea', 'notes', [
            'label'       => 'field.additional.notes',
            'data-editor' => true,
        ])->addElement('hidden', 'partofportalrequestid', [
            'value' => '',
        ])->addElement('hidden', 'partofportalid', [
            'value' => '',
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    public static function getResearchUnpublish(Exposition $research)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.research.unpublish', $research);

        $form->addElement('html', 'info', [
            'value' => '<br/><span style="float: left; margin-right: .3em;" class="ui-icon ui-icon-info"></span>' . static::_getTranslate()->translate('info.unpublish.research'),
        ])
            ->addElement('hidden', 'status', [
                'required' => true,
                'value'    => Exposition::STATUS_PROGRESS,
                'readonly' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    // Database

    public static function getDatabaseInit()
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.database.init');

        $form->addElement('html', 'config', [
            'value' => '',
        ]);
        $form->addElement('html', 'placeholder', [
            'value' => '<br/>',
        ]);
        $form->addElement('html', 'info', [
            'value' => static::_getTranslate()->translate('info.database.init'),
        ])
            ->addElement('hidden', 'confirm', [
                'value'    => 'remove.entry',
                'required' => true,
            ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param DefaultForm $form
     * @return DefaultForm
     */
    protected static function addFormElementAccordions(DefaultForm $form)
    {
        $elementDefinitions = [
            'form'   => [
                'checkbox'      => [],
                'money'         => [
                    $form->createElement('text', 'currency', [
                        'label'    => 'currency',
                        'required' => true,
                    ]),
                ],
                'date'          => [],
                'image'         => [],
                'number'        => [],
                'pdf'           => [],
                'range'         => [
                    $form->createElement('number', 'min', [
                        'label'    => 'min',
                        'required' => true,
                    ]),
                    $form->createElement('number', 'max', [
                        'label'    => 'max',
                        'required' => true,
                    ]),
                ],
                'select'        => [
                    $form->createElement('text', 'rawoptions', [
                        'required'    => true,
                        'label'       => 'field.options',
                        'data-tagit'  => '{"allowSpaces": true}',
                        'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
                    ]),
                ],
                'short-text'    => [],
                'text'          => [
                    $form->createElement('number', 'max', [
                        'label'    => 'max characters',
                        'required' => true,
                    ]),
                ],
                'checkbox-list' => [
                    $form->createElement('text', 'rawoptions', [
                        'required'    => true,
                        'label'       => 'field.options',
                        'data-tagit'  => '{"allowSpaces": true}',
                        'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
                    ]),
                ],
            ],
            'layout' => [
                'headline'    => [
                    $form->createElement('text', 'headline', [
                        'label'    => 'field.headline',
                        'required' => true,
                    ]),
                ],
                'description' => [
                    $form->createElement('textarea', 'description', [
                        'label'    => 'field.description',
                        'required' => true,
                    ]),
                ],
                'hr'          => [],
            ],
        ];

        foreach ($elementDefinitions as $type => $elementDefinition) {
            foreach ($elementDefinition as $name => $elements) {
                $elements = array_merge([
                    $form->createElement('hidden', 'element', [
                        'value' => $name,
                    ]),
                    $form->createElement('hidden', 'identifier', [
                        'required' => true,
                    ]),
                ], $elements);

                if ($type == 'form') {
                    $elements = array_merge([
                        $form->createElement('text', 'label', [
                            'required' => true,
                            'label'    => 'label',
                            'oninput'  => 'updateFormElementAccordionHeader($(this).closest(".ui-accordion-content").parent())',
                        ]),
                        $form->createElement('textarea', 'description', [
                            'label' => 'description',
                        ]),
                        $form->createElement('checkbox', 'required', [
                            'label' => 'required',
                        ]),
                    ], $elements);
                }

                /** @var DefaultForm[] $accordions */
                $accordions[$name] = $form->addAccordion($name, $elements,
                    ['sortable' => true, 'deletable' => true, 'collapsible' => true, 'active' => false]);
                $accordions[$name]->setElementsBelongTo('element[]');
            }
        }

        return $form;
    }

    /**
     * @param Portal $programPortal
     * @return array
     */
    protected static function getApplicantPortalList(Portal $programPortal)
    {
        $portals = [];
        foreach (PortalRepository::findAll(['sort' => ['name' => 1]]) as $portal) {
            if ($portal->getId() == $programPortal->getId()) {
                continue;
            }
            if (!$portal->getAllowApplicationPublishing()) {
                continue;
            }
            $portals[$portal->getId()] = $portal->getName();
        }
        return $portals;
    }

    /**
     * @param Portal $portal
     * @return Exposition[]
     */
    protected static function getTextExpositionTemplateOptions(Portal $portal)
    {
        $options = [];

        foreach (ExpositionRepository::findTextExpositionsByUsers($portal->getAdmins()) as $exposition) {
            $pEntry = &$options[$exposition->getOwner()->getName()];

            $pEntry[$exposition->getId()] = '<strong>' . $exposition->getTitle() . '</strong><br/>by ' . $exposition->getOwner()->getName();
        }

        return $options;
    }
}
