<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;

class WorkFormGenerator extends BasicFormGenerator
{

    /**
     * @param Work $work
     * @return DefaultForm
     */
    public static function getAdd(Work $work)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.work.add');

        $authors = static::_getLazyUserList(static::_getParam('meta.rcauthors', $work->getAuthor() ? [$work->getAuthor()->getId()] : []), false);
        $isRequired = empty($authors) && static::_getParam('meta.freetextauthor') == '';

        $tab = $form->addTab('meta');
        $tab->addElement('languageBar', 'language-bar')
            ->addMultiLanguageElement('text', 'title', [
                'required' => true,
                'label'    => 'field.title',
            ])
            ->addElement('select', 'genre', [
                'required'     => true,
                'label'        => 'field.genre',
                'multiOptions' => ['' => ''] + static::_getWorkGenreList(),
            ])
            ->addElement('datePicker', 'date', [
                'required' => true,
                'label'    => 'field.date',
            ])
            ->addInlineElement([
                $tab->createElement('multiselect', 'rcauthors', [
                    'label'                     => 'author',
                    'multiOptions'              => $authors,
                    'required'                  => $isRequired,
                    'data-autocomplete'         => true,
                    'data-autocomplete-options' => json_encode([
                        'url'         => '/user/search',
                        'placeholder' => 'Add user...',
                        'property'    => 'name',
                        'maxcount'    => 1,
                    ]),
                    'description'               => '<span class="form-description-line">use this field to select an existing rc user</span>',
                ]),
                $tab->createElement('text', 'freetextauthor', [
                    'label'       => ' ',
                    'required'    => $isRequired,
                    'class'       => 'indented',
                    'style'       => 'margin-top:10px',
                    'description' => '<span class="form-description-line">use this field for an author without rc account</span>',
                ]),
            ])
            ->addElement('text', 'publisher', [
                'label' => 'field.publisher',
            ])
            ->addElement('text', 'referenceurl', [
                'label' => 'field.reference-url',
            ])
            ->addElement('text', 'copyrightholder', [
                'required' => true,
                'label'    => 'field.copyright',
            ])
            ->addMultiLanguageElement('textarea', 'description', [
                'label' => 'field.description',
            ])
            ->addElement('text', 'keywords', [
                'label'       => 'field.keywords',
                'data-tagit'  => '',
                'description' => '<span class="form-description-line">' . self::_getTranslate()->translate('field.tagline.description') . '</span>',
            ]);

        $form->addTab('additional.infos')
            ->addElement('text', 'place', [
                'label' => 'field.place',
            ])
            ->addElement('text', 'country', [
                'label' => 'field.country',
            ])
            ->addElement('text', 'material', [
                'label' => 'field.material',
            ])
            ->addElement('text', 'technique', [
                'label' => 'field.technique',
            ])
            ->addElement('text', 'format', [
                'label' => 'field.format',
            ])
            ->addElement('text', 'reference', [
                'label' => 'field.reference',
            ]);

        self::_addMediaTab($form, $work);
        self::_addCollaborationTab($form, $work);
        self::_addRelationTab($form, $work);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param Work $work
     * @return DefaultForm
     */
    public static function getAddProfile(Work $work)
    {
        return self::getAdd($work);
    }

    /**
     * @param Work $work
     * @param bool isOwner
     * @return DefaultForm
     * @throws Exception
     * @throws Zend_Form_Exception
     */
    public static function getEdit(Work $work, $isOwner)
    {
        $form = static::getAdd($work);
        $form->setTitle('dialog.work.edit', $work);

        $form->addTab('access')
            ->addElement('multiselect', 'shares', [
                'label'            => 'field.access.researches',
                'multiOptions'     => self::_getResearchListByPerson(Identity::getUser()),
                'data-multiselect' => true,
            ]);

        if (!$work->isMetaEditableBy(Identity::getUser())) {
            $form->removeElement('submitbutton');
            $form->removeTab([
                'access',
            ]);
            foreach ($form->getTab('meta')->getElements() as $element) {
                $element->setAttrib('disabled', 'disabled');
            }
            foreach ($form->getTab('additional.infos')->getElements() as $element) {
                $element->setAttrib('disabled', 'disabled');
            }
        }

        if (!$isOwner) {
            $form->clearAndDisableTab([
                'collaboration',
            ]);
        }

        return $form;
    }

    /**
     * @param Exposition $exposition
     * @return DefaultForm
     */
    public static function getAddFile(Exposition $exposition)
    {
        $form = new DefaultForm();
        $form->setTitle('dialog.file.add');

        $form->addElement('multiselect', 'file', [
            'label'            => 'field.files',
            'multiOptions'     => self::_getFileList($exposition),
            'data-multiselect' => true,
        ]);

        $form->addCancelButton()->addSubmitButton();

        return $form;
    }

    /**
     * @param SimpleMedia $simpleMedia
     * @return DefaultForm
     */
    public static function getRemoveFile(SimpleMedia $simpleMedia)
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.file.remove', $simpleMedia);
        $form->getElement('info')->setValue(static::_getTranslate()->translate('info.entry.work.remove'));

        return $form;
    }

    /**
     * @return DefaultForm
     */
    public static function getRemove()
    {
        $form = parent::getRemove();
        $form->setTitle('dialog.work.remove');

        return $form;
    }

    /**
     * @param Research $entity
     * @return DefaultForm
     */
    public static function getShare(Research $entity = null)
    {
        $form = parent::getShare($entity);
        $form->setTitle('dialog.work.share', $entity);

        return $form;
    }

    /**
     * @param Exposition $exposition
     * @return SimpleMedia[]
     */
    protected static function _getFileList(Exposition $exposition)
    {
        $retVal = [];
        foreach (SimpleMediaRepository::findByResearch($exposition) as $simpleMedia) {
            $retVal[$simpleMedia->getId()] = $simpleMedia->getName();
        }
        return $retVal;
    }

}

?>