<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\AbstractDilpsModel;
use Rc\Domain\Model\ModelInterface;
use Rc\Service\Ingest\IngestService;
use Rc\Utility\Strings;

/**
 * @method DefaultForm setDefault($name, $value)
 * @method Zend_Form_Element[] getElements()
 */
class DefaultForm extends ZendX_JQuery_Form
{

    const PREFIX_TAB_NAME = 'tab';

    const PREFIX_FIELDSET_NAME = 'fieldset';

    const PREFIX_ACCORDION_NAME = 'accordion';

    const PREFIX_MULTILANGUAGE_NAME = 'multilanguage';

    const PREFIX_INLINE_ELEMENT_NAME = 'inline';

    const TYPE_CONTAINER = 'container';

    const TYPE_FORM = 'form';

    const TYPE_FIELDSET = 'fieldset';

    const TYPE_TAB = 'tab';

    const TYPE_ACCORDION = 'accordion';

    const TYPE_MULTILANGUAGE = 'multilanguage';

    protected $type;

    protected $elementsPrefix;

    protected $accordionContainerOptions;

    protected $tabContainerOptions;

    // Init

    public function init()
    {
        $this->accordionContainerOptions = [
            'autoHeight' => false,
        ];
        $this->tabContainerOptions = [
            'cache'  => 'true',
            'create' => new Zend_Json_Expr('function(event) {
		        var dialog = $(event.target).closest("[role=dialog]");
		        dialog.bind("resize",function() {
	                dialog.css({
	                    maxHeight: $(window).height() - 3
	                }).position({
	                    my: "top",
	                    at: "top",
	                    of: $(window),
	                    collision: "none"
	                });
	                if (dialog.position().top < 75 && dialog.height() < $(window).height()) {
	                    dialog.css({top: Math.min(($(window).height() - dialog.height() - 25)/2, 50)});
	                }
		        }).trigger("resize");
            }'),
        ];

        $this->setType(self::TYPE_FORM);
        $this->setMethod('post');
        $this->setAttrib('class', 'cmxform');
        $this->setTranslator(Zend_Registry::get('Zend_Translate'));
        $this->setElementsPrefix('frm');
        $this->setDisableLoadDefaultDecorators(true);

        $this->addPrefixPath('My_Zend_Form_Element', APPLICATION_PATH . '/library/plugin/', 'element');
        $this->addPrefixPath('My_Form_Decorator', APPLICATION_PATH . '/views/decorators/', 'decorator');
        $this->addElementPrefixPath('My_Filter', APPLICATION_PATH . '/library/filter/', 'filter');
        $this->addElementPrefixPath('My_Validate', APPLICATION_PATH . '/library/validate/', 'validate');
        $this->addElementPrefixPath('My_Form_Decorator', APPLICATION_PATH . '/views/decorators/', 'decorator');

        $this->setDecorators([
            'FormElements',
            'Form',
            'JQueryOnLoadActions',
        ]);
    }

    // Element

    /**
     * @param string|Zend_Form_Element $element
     * @param string $name
     * @param array $options
     * @return $this
     */
    public function addElement($element, $name = null, $options = null)
    {
        parent::addElement($element, $name, $options);

        $element = $element instanceof Zend_Form_Element ? $element : $this->_elements[$name];
        if ($element->getType() == 'Zend_Form_Element_File') {
            $this->addElement('hidden', $name . '_delete');
        }
        return $this;
    }

    public function createElement($type, $name = null, $options = null)
    {
        if (is_object($type)) {
            return parent::createElement($type);
        }

        $element = parent::createElement($type, $name, $options);
        $element->setAttrib('id', $this->getElementsPrefix() . $element->getId());
        $element->addFilter('StringTrim');

        $this->_setDefaultElementOptions($element, $options);

        return $element;
    }

    public function getValue($name)
    {
        $retVal = null;
        $element = $this->getElement($name);

        if (isset($element)) {
            $values = $this->_convertElementValue([$element->getName() => parent::getValue($name)], [$element]);

            if (array_key_exists($element->getName(), $values)) {
                $retVal = $values[$element->getName()];
            }
        }

        return $retVal;
    }

    public function getValues($merged = true)
    {
        $values = $this->_convertElementValue(
            parent::getValues(true)
        );

        foreach ($this->getSubForms() as $key => $subForm) {
            if ($subForm->isArray()) {
                $index = $subForm->getElementsBelongTo();
            } else {
                $index = $key;
            }

            if (($subForm->isType(self::TYPE_TAB) && $merged) ||
                ($subForm->isType(self::TYPE_ACCORDION) && !$subForm->isArray())
            ) {
                $tmp = $values[$index];
                unset($values[$index]);
                $values += $tmp;
            }
        }

        return array_map(function ($value) {
            if (isset($value) && is_string($value) && $value === '') {
                return null;
            }
            return $value;
        }, $values);
    }

    public function setDefaults(array $values)
    {
        foreach ($values as $name => $value) {
            $element = $this->getElement($name);
            if (!isset($element) || $element->getIgnore()) {
                continue;
            }

            switch ($element->getType()) {
                case 'Zend_Form_Element_File':
                    $element->setAttribs($element->getAttribs() + ['value' => $value]);
                    break;
            }
        }

        parent::setDefaults($values);

        return $this;
    }

    public function isValid($data = null)
    {
        if (!isset($data)) {
            $data = $this->_getRequest()->getParams();
            if ($this->isType(self::TYPE_TAB)) {
                $key = substr($this->getName(), strlen(self::TYPE_TAB));
                if (isset($data[$key])) {
                    $data = $data[$key];
                }
            }

            if ($this->_getRequest()->getParam('submit-async-file')) {
                $defaultValues = $this->getValues();
                foreach ($defaultValues as $key => $value) {
                    if (is_array($value) && count($value) == 0) {
                        unset($defaultValues[$key]);
                    }
                }
                $data += $defaultValues;
            }
        }

        return parent::isValid($data);
    }

    public function render(Zend_View_Interface $view = null)
    {
        if ($this->isType(self::TYPE_CONTAINER)) {
            $this->removeDecorator('Form');
            $this->addDecorator('HtmlTag', [
                'tag'   => 'div',
                'class' => 'cmxform',
            ]);
        }

        $this->_addUiContainerDecorators('accordion');
        $this->_addUiContainerDecorators('tab', 1);

        return parent::render($view);
    }

    /**
     * Given an array, an optional arrayPath and a key this method
     * dissolves the arrayPath and unsets the key within the array
     * if it exists.
     *
     * @param array $array
     * @param string|null $arrayPath
     * @param string $key
     * @return array
     */
    protected function _dissolveArrayUnsetKey($array, $arrayPath, $key)
    {
        $unset =& $array;
        $path = trim(strtr((string)$arrayPath, ['[' => '/', ']' => '']), '/');
        $segs = ('' !== $path) ? explode('/', $path) : [];

        foreach ($segs as $seg) {
            if (!is_array($unset) || !array_key_exists($seg, $unset)) {
                return $array;
            }
            $unset =& $unset[$seg];
        }
        if (array_key_exists($key, (array)$unset)) {
            unset($unset[$key]);
        }
        return $array;
    }

    public function createMultiLanguageElement($type, $name = null, $options = null)
    {
        if (!isset($options)) {
            $options = [];
        }

        $subForm = new DefaultForm();
        $subForm->setType(self::TYPE_MULTILANGUAGE);
        $subForm->setName($name);
        $subForm->setElementsBelongTo($name);
        $subForm->setElementsPrefix($subForm->getElementsPrefix() . $name);
        $subForm->setDecorators([
            'FormElements',
        ]);

        $defaultLanguage = config('app.lang');
        foreach (MyLocal::getTranslationList('language') as $key => $value) {
            $requiredOption = [
                'required' => (!empty($options['required']) && $defaultLanguage == $key),
            ];

            $element = $subForm->createElement($type, $key, $requiredOption + $options);
            $element->addDecorator('HtmlTag', [
                'tag'           => 'p',
                'data-language' => $key,
                'style'         => ($defaultLanguage != $key ? 'display:none' : ''),
            ]);

            $subForm->addElement($element);
        }

        return $subForm;
    }

    // Tab

    /**
     * @param string $type
     * @param string|null $name
     * @param array $options
     * @return $this
     */
    public function addMultiLanguageElement($type, $name = null, $options = [])
    {
        $subForm = $this->createMultiLanguageElement($type, $name, $options);

        parent::addSubForm($subForm, $subForm->getName());

        return $this;
    }

    /**
     * @param Zend_Form_Element[] $elements
     * @param string|null $name
     * @param array $options
     * @return $this
     */
    public function addInlineElement(array $elements, $name = null, $options = null)
    {
        $this->createInlineElement($elements, $name, $options);
        return $this;
    }

    public function createInlineElement(array $elements, $name = null, $options = null)
    {
        if (!isset($name)) {
            $name = self::PREFIX_INLINE_ELEMENT_NAME . '-' . count($this->getDisplayGroups());
        }
        foreach ($elements as $element) {
            $element->addDecorator('HtmlTag', ['tag' => 'span', 'class' => 'inline']);
        }
        reset($elements)->removeDecorator('HtmlTag');

        $this->addDisplayGroup($elements, $name, $options);

        return $this->getDisplayGroup($name)
            ->setDecorators([
                'FormElements',
                ['HtmlTag', ['tag' => 'p']],
            ]);
    }

    /**
     * @param string $value
     * @param null $label
     * @return DefaultForm
     */
    public function addSubmitButton($value = 'submit', $label = null)
    {
        if (!isset($label)) {
            $label = 'button.' . $value;
        }

        return $this->addElement('button', $value . 'button', [
            'type'  => 'submit',
            'label' => $label,
        ]);
    }

    /**
     * @param string $value
     * @return DefaultForm
     */
    public function addCancelButton($value = 'cancel')
    {
        return $this->addElement('button', $value . 'button', [
            'type'    => 'cancel',
            'label'   => 'button.' . $value,
            'onclick' => '$(this).closest(\'form\').dialog(\'close\'); return false;',
        ]);
    }

    public function addFieldset($name = null, array $elements = [], $options = [])
    {
        $subForm = $this->_addUiContainer('fieldset', $name, $elements, $options);

        if (!array_key_exists('elementsBelongTo', $options)) {
            $subForm->setElementsBelongTo($name);
        } elseif ($options['elementsBelongTo'] !== null) {
            $subForm->setElementsBelongTo($options['elementsBelongTo']);
        }

        return $subForm;
    }

    /**
     * @param string|null $name
     * @param array $elements
     * @param array $options
     * @return DefaultForm
     */
    public function addTab($name = null, array $elements = [], $options = [])
    {
        $subForm = $this->_addUiContainer('tab', $name, $elements, $options);

        if (!array_key_exists('elementsBelongTo', $options)) {
            $subForm->setElementsBelongTo($name);
        } elseif ($options['elementsBelongTo'] !== null) {
            $subForm->setElementsBelongTo($options['elementsBelongTo']);
        }

        return $subForm;
    }

    public function pushTab($tab)
    {
        $this->addSubForm($tab, self::PREFIX_TAB_NAME . '-' . substr($tab->getName(), 3));
        return $this;
    }

    public function addTabs($tabs, $options = [])
    {
        return $this->_addUiContainers('tab', $tabs, $options);
    }

    /**
     * @param string $name
     * @return DefaultForm
     */
    public function getTab($name)
    {
        return $this->_getUiContainer('tab', $name);
    }

    /**
     * @return $this[]
     */
    public function getTabs()
    {
        return $this->_getUiContainers('tab');
    }

    public function addTabContainerOptions($options)
    {
        $this->setTabContainerOptions(
            $options + $this->getTabContainerOptions()
        );
        return $this;
    }

    public function getTabContainerOptions($key = null)
    {
        if (isset($key) && isset($this->tabContainerOptions[$key])) {
            $retVal = $this->tabContainerOptions[$key];
        } elseif (!isset($key)) {
            $retVal = $this->tabContainerOptions;
        } else {
            $retVal = null;
        }

        return $retVal;
    }

    public function setTabContainerOptions($options)
    {
        $this->tabContainerOptions = $options;
        return $this;
    }

    public function enableTab($tabs = null)
    {
        $tabs = $this->_initTabsIfEmpty($tabs);
        $existingTabs = array_values($this->getTabs());
        $disabled = (array)$this->getTabContainerOptions('disabled');

        foreach ($tabs as $value) {
            $tab = $this->getTab($value);
            if (empty($tab)) {
                continue;
            }

            $index = array_search($tab, $existingTabs);
            if ($index !== false && ($index = array_search($index, $disabled)) !== false) {
                unset($disabled[$index]);
            }
        }

        $this->addTabContainerOptions([
            'disabled' => $disabled,
        ]);

        return $this;
    }

    public function disableTab($tabs = null)
    {
        $tabs = $this->_initTabsIfEmpty($tabs);
        $existingTabs = array_values($this->getTabs());
        $disabled = (array)$this->getTabContainerOptions('disabled');

        foreach ($tabs as $value) {
            $tab = $this->getTab($value);
            if (empty($tab)) {
                continue;
            }

            foreach ($tab->getElements() as $element) {
                $element->setAttrib('readonly', true);
            }

            $index = array_search($tab, $existingTabs);
            if ($index !== false && array_search($index, $disabled) === false) {
                $disabled[] = $index;
            }
        }

        $this->addTabContainerOptions([
            'disabled' => $disabled,
        ]);

        return $this;
    }

    // Accordion

    public function removeTab($tabs)
    {
        $tabs = $this->_initTabsIfEmpty($tabs);

        foreach ($tabs as $tab) {
            $this->removeSubForm(self::PREFIX_TAB_NAME . '-' . $tab);
        }

        return $this;
    }

    public function clearTab($tabs = null)
    {
        $tabs = $this->_initTabsIfEmpty($tabs);

        foreach ($tabs as $tab) {
            $this->getTab($tab)->clearElements();
        }

        return $this;
    }

    public function clearTabPerFlag($flag, $tabs = null)
    {
        if ($flag) {
            $this->clearTab($tabs);
        }

        return $this;
    }

    public function clearAndDisableTab($tabs = null)
    {
        $this->clearTab($tabs);
        $this->disableTab($tabs);

        return $this;
    }

    public function clearAndDisableTabPerFlag($flag, $tabs = null)
    {
        if ($flag) {
            $this->clearAndDisableTab($tabs);
        }

        return $this;
    }

    public function addAccordion($name = null, array $elements = [], $options = [])
    {
        $subForm = $this->_addUiContainer('accordion', $name, $elements, $options);

        return $subForm;
    }

    public function addAccordions($accordions, $options = [])
    {
        return $this->_addUiContainers('accordion', $accordions, $options);
    }

    // Values

    /**
     * @param string $name
     * @return DefaultForm
     */
    public function getAccordion($name)
    {
        return $this->_getUiContainer('accordion', $name);
    }

    /**
     * @return DefaultForm[]
     */
    public function getAccordions()
    {
        return $this->_getUiContainers('accordion');
    }

    public function addAccordionContainerOptions($options)
    {
        $this->setAccordionContainerOptions(array_merge($options, $this->getAccordionContainerOptions()));
        return $this;
    }

    public function getAccordionContainerOptions($key = null)
    {
        if (isset($key) && isset($this->accordionContainerOptions[$key])) {
            $retVal = $this->accordionContainerOptions[$key];
        } elseif (!isset($key)) {
            $retVal = $this->accordionContainerOptions;
        } else {
            $retVal = null;
        }

        return $retVal;
    }

    public function setAccordionContainerOptions($options)
    {
        $this->accordionContainerOptions = $options;
        return $this;
    }

    // jQuery

    /**
     * @param \Rc\Domain\Model\Core\FormValuesTrait $entity
     * @param bool $forSubForms
     * @param array $attributes
     * @return $this
     * @throws Exception
     */
    public function setDefaultsFrom($entity, $forSubForms = null, array $attributes = [])
    {
        if (!isset($forSubForms)) {
            $subForms = $this->getSubForms();
        } elseif (is_string($forSubForms)) {
            if ($forSubForms[0] == '!') {
                $subForms = $this->getSubForms() + $this->getAccordions() + $this->_getUiContainers('multilanguage');
                $subFormToIgnore = $this->getTab(substr($forSubForms, 1));
                foreach ($subForms as $key => $subForm) {
                    if ($subForm == $subFormToIgnore) {
                        unset($subForms[$key]);
                    }
                }
            } else {
                $subForms = [$this->getTab($forSubForms)] + $this->getAccordions() + $this->_getUiContainers('multilanguage');
            }
        } elseif (is_object($forSubForms)) {
            $subForms = [$forSubForms];
        } else {
            throw new Exception('UNSUPPORTED_PARAMETER_TYPE');
        }

        $this->_setDefaultsFrom($entity, $attributes);
        foreach ($subForms as $key => $subForm) {
            $subForm->setDefaultsFrom($entity, null, $attributes);
        }
        return $this;
    }

    public function isPostAndValid(array $data = null)
    {
        return $this->_getRequest()->isPost() && $this->isValid($data);
    }

    /**
     * @param array $options
     * @return $this
     */
    public function setJQueryParams(array $options)
    {
        $this->setAttrib('jQueryParams', $options);
        return $this;
    }

    /**
     * @return array
     */
    public function getJQueryParams()
    {
        return (array)$this->getAttrib('jQueryParams');
    }

    /**
     * @param string $key
     * @param mixed $param
     * @return $this
     */
    public function addJQueryParam($key, $param)
    {
        $this->setAttrib('jQueryParams', [$key => $param] + $this->getJQueryParams());
        return $this;
    }

    /**
     * @param string $key
     * @return mixed|null
     */
    public function getJQueryParam($key)
    {
        $jQueryParams = $this->getJQueryParams();
        if (isset($jQueryParams[$key])) {
            $retVal = $jQueryParams[$key];
        } else {
            $retVal = null;
        }

        return $retVal;
    }

    public function getElementsPrefix()
    {
        return $this->elementsPrefix;
    }

    // Misc

    public function setElementsPrefix($elementPrefix)
    {
        $this->elementsPrefix = $elementPrefix . '-';
    }

    public function getType()
    {
        return $this->type;
    }

    public function setType($type)
    {
        $this->type = $type;
        return $this;
    }

    public function isType($type)
    {
        return ($this->type == $type);
    }

    public function setTitle($title, $suffix = null)
    {
        $title = $this->getTranslator()->translate($title);

        if ($suffix instanceof AbstractDilpsModel) {
            $this->setAttrib('data-subtitle', $suffix->getTitle() . ' [' . $suffix->getId() . ']');
        } elseif ($suffix instanceof ModelInterface) {
            $title .= ' ' . $suffix->getId();
        } elseif (is_string($suffix)) {
            $title = sprintf($title, $suffix);
        }

        $this->setAttrib('title', $title);
        return $this;
    }

    public function getTitle()
    {
        return $this->getAttrib('title');
    }

    /**
     * @param  string $name
     * @return bool
     */
    public function hasSubForm($name)
    {
        return isset($this->_subForms[$name]);
    }

    protected function _initTabsIfEmpty($tabs = null)
    {
        if (!isset($tabs)) {
            if ($this->isType(self::TYPE_TAB)) {
                $retVal = [str_replace(self::PREFIX_TAB_NAME, '', $this->getName())];
            } else {
                throw new Exception('No tab specified!');
            }
        } else {
            $retVal = (array)$tabs;
        }

        return $retVal;
    }

    // UiContainer

    /**
     * @param \Rc\Domain\Model\Core\FormValuesTrait $entity
     * @param array $attributes
     * @return $this
     */
    protected function _setDefaultsFrom($entity, array $attributes = [])
    {
        if (count($attributes) == 0) {
            $attributes = [];

            foreach ($this->getElements() as $element) {
                if ($element->getType() == 'Zend_Form_Element_Button' || $element->getIgnore()) {
                    continue;
                }

                $attributes[$element->getName()] = $element->getName();
            }

            if ($this->isType(self::TYPE_MULTILANGUAGE)) {
                $attributes = [$this->getName() => $attributes];
            }
        }

        $defaults = [];

        foreach ($attributes as $name => $attribute) {
            $element = $this->getElement($name);
            if (isset($element) && $element->getType() == 'Zend_Form_Element_File' && $element->getName() == 'media') {
                $method = 'getThumb';
            } else {
                $method = 'get' . Strings::toCamelCase($name, '-', true);
            }

            // multi language attribute
            if (is_array($attribute) && method_exists($entity, $method . 'List')) {
                $value = $entity->{$method . 'List'}();
                if (is_object($value) && method_exists($value, 'toArray')) {
                    $value = $value->toArray();
                } else {
                    $info = [];
                    if (PHP_SAPI == 'cli') {
                        $info[] = 'cli: ' . implode(' ', $_SERVER['argv']);
                    } else {
                        $info[] = 'url: ' . config('app.url') . $_SERVER['REQUEST_URI'];
                        if (!empty($_SERVER['HTTP_REFERER'])) {
                            $info[] = 'referer: ' . $_SERVER['HTTP_REFERER'];
                        }
                    }
                    if (\Identity::exists()) {
                        $info[] = 'identity: ' . \Identity::getUserId();
                    }
                    trigger_error(implode(' , ', $info) . "\tMISSING TRANSLATION FOR #" . $entity->getId() . " " . $method . 'List()', E_USER_WARNING);
                }
                $defaults = (array)$value;
            } elseif (is_array($attribute) && method_exists($entity, $method)) {
                $defaults = $entity->{$method}()->toArray();
            } else {
                if (method_exists($entity, $method) == false) {
                    $method = 'is' . Strings::toCamelCase($name, '-', true);
                    if (method_exists($entity, $method) == false) {
                        continue;
                    }
                }

                $defaults[$name] = $entity->$method();
            }
        }

        $this->setDefaults($defaults);

        return $this;
    }

    protected function _convertElementValue(array $values, array $elements = [])
    {
        if (count($elements) == 0) {
            $elements = $this->getElements();
        }

        foreach ($elements as $element) {
            switch ($element->getType()) {
                case 'Zend_Form_Element_Password':
                    if ($element->getValue() == null) {
                        unset($values[$element->getName()]);
                    }
                    break;

                case 'Zend_Form_Element_File':
                    unset($values[$element->getName()]);
                    if ($element->getValue() != null) {
                        if (IngestService::isArchive($element->getFileName())) {
                            $values[$element->getName() . 'List'] = IngestService::mastersFromArchive($element->getFileName());
                        } else {
                            $values[$element->getName()] = IngestService::masterFromFile($element->getFileName());
                        }
                    } elseif (!empty($values[$element->getName() . '_delete'])) {
                        $values[$element->getName()] = null;
                    }
                    unset($values[$element->getName() . '_delete']);
                    break;

                case 'Zend_Form_Element_Multiselect':
                    if ($element->getValue() == null) {
                        $values[$element->getName()] = [];
                    }
                    break;
            }
        }

        return $values;
    }

    protected function _addUiContainer($type, $name = null, array $elements = [], $jQueryParams = [])
    {
        $containerName = constant('self::PREFIX_' . strtoupper($type) . '_NAME') . '-' . $name;

        $subForm = $this->getSubForm($containerName);
        if (!$subForm) {
            $subForm = new DefaultForm();
            $subForm->setJQueryParams($jQueryParams);
            $subForm->setType(constant('self::TYPE_' . strtoupper($type)));
            $subForm->setElementsPrefix($subForm->getElementsPrefix() . $name);
            $subForm->setDecorators([
                'FormElements',
            ]);

            $this->addSubForm($subForm, $containerName);
        } elseif (!empty($jQueryParams)) {
            $subForm->setJQueryParams($jQueryParams);
        }

        if (count($elements) > 0) {
            foreach ($elements as $key => $element) {
                if (get_class($element) == 'Zend_Form_DisplayGroup') {
                    $subForm->addDisplayGroup($element->getElements(), $element->getName());
                    $subForm->getDisplayGroup($element->getName())->setDecorators($element->getDecorators());

                    $element->clearElements();
                    $element->clearDecorators();
                    $this->removeDisplayGroup($element->getName());
                } elseif (get_class($element) == 'DefaultForm') {
                    $subForm->addSubForm($element, $element->getName());
                } else {
                    $subForm->addElement($element);
                }
            }
        }

        return $subForm;
    }

    protected function _addUiContainers($type, array $uiContainers, $options = [])
    {
        foreach ($uiContainers as $uiContainer) {
            $this->_addUiContainer($type, $uiContainer[0], $uiContainer[1], $options);
        }

        return $this;
    }

    /**
     * @param string $type
     * @param string $name
     * @return DefaultForm|null
     */
    protected function _getUiContainer($type, $name)
    {
        $retVal = null;

        foreach ($this->_getUiContainers($type) as $key => $uiContainer) {
            if ($key == $type . '-' . $name || $uiContainer->getName() == $type . $name) {
                $retVal = $uiContainer;
                break;
            }
        }

        return $retVal;
    }

    protected function _getUiContainers($type)
    {
        $retVal = [];

        foreach ($this->getSubForms() as $key => $subForm) {
            if ($subForm->isType(constant('self::TYPE_' . strtoupper($type)))) {
                $retVal[$key] = $subForm;
            }
        }

        return $retVal;
    }

    // Helper

    protected function _addUiContainerDecorators($type, $minCount = null)
    {
        $uiContainers = $this->_getUiContainers($type);
        $optionsMethod = 'get' . ucfirst($type) . 'ContainerOptions';

        if ((!isset($minCount) && count($uiContainers) > 0) || (isset($minCount) && count($uiContainers) > $minCount)) {
            $orderCount = 0;
            foreach ($this->_order as $key => &$order) {
                if (strpos($key, constant('self::PREFIX_' . strtoupper($type) . '_NAME') . '-') !== false) {
                    $order = $orderCount++;
                }
            }

            $lastUiContainer = end($uiContainers);

            foreach ($uiContainers as $key => $uiContainer) {
                $uiContainer->setJQueryParams($uiContainer->getJQueryParams() + [
                        'containerId' => $lastUiContainer->getName(),
                        'title'       => str_replace('-', '.', $key),
                    ]);

                $uiContainer->addDecorator('Error');
                $uiContainer->addDecorator(ucfirst($type) . 'Pane');
            }

            $lastUiContainer->addDecorator(ucfirst($type) . 'Container', [
                'jQueryParams' => $this->{$optionsMethod}(),
            ]);
        }

        return $this;
    }

    protected function _getUiContainerElements($types)
    {
        $retVal = [];

        foreach ((array)$types as $type) {
            foreach ($this->_getUiContainers($type) as $uiContainer) {
                foreach ($uiContainer->getElements() as $name => $element) {
                    $retVal[$name] = $element;
                }
            }
        }

        return $retVal;
    }

    protected function _setDefaultElementOptions(Zend_Form_Element $element, $options = null)
    {
        if (isset($options['decorators'])) {
            $element->setDecorators($options['decorators']);
            return $element;
        }

        $htmlTagOptions = ['tag' => 'p'];
        if (isset($options['data-language'])) {
            $htmlTagOptions += [
                'data-language' => $options['data-language'],
                'style'         => $options['style'],
            ];
        }

        switch ($element->getType()) {
            case 'ZendX_JQuery_Form_Element_DatePicker':
                $element->setOptions([
                        'placeholder'  => $this->_getLocale()->getDefaultDateFormatForUser(),
                        'jQueryParams' => [
                            'changeMonth' => true,
                            'changeYear'  => true,
                            'dateFormat'  => $this->_getLocale()->getDefaultDateFormatForJs(),
                        ],
                        'validators'   => [
                            ['SimpleDate', false, static::_getLocale()->getDefaultDateFormatForUser()],
                        ],
                    ] + $options);
                $element->setDecorators([
                    'Label',
                    'UiWidgetElement',
                    'Error',
                    ['HtmlTag', $htmlTagOptions],
                ]);
                break;

            case 'Zend_Form_Element_Hidden':
            case 'Zend_Form_Element_Button':
            case 'My_Zend_Form_Element_LanguageBar':
                $element->setDecorators([
                    'DefaultElement',
                ]);
                break;
            case 'My_Zend_Form_Element_Html':
                if (!empty($options['label'])) {
                    $element->setDecorators([
                        'Label',
                        ['DefaultElement'],
                        ['HtmlTag', $htmlTagOptions],
                    ]);
                } else {
                    $element->setDecorators([
                        'DefaultElement',
                    ]);
                }
                break;
            case 'Zend_Form_Element_File':
            case 'My_Zend_Form_Element_Image':
            case 'My_Zend_Form_Element_Pdf':
                $element->setDecorators([
                    'DefaultFile',
                    'Label',
                    'DefaultElement',
                    'Error',
                    ['HtmlTag', $htmlTagOptions],
                ]);
                break;
            case 'Zend_Form_Element_Text':
                if ($this->_getRequest()->getParams()['controller'] != 'admin') {
                    $element->addFilter('HtmLawed');
                }
                $element->setDecorators([
                    'Label',
                    'DefaultElement',
                    'Error',
                    ['HtmlTag', $htmlTagOptions],
                ]);
                break;
            default:
                $element->setDecorators([
                    'Label',
                    'DefaultElement',
                    'Error',
                    ['HtmlTag', $htmlTagOptions],
                ]);
        }

        return $element;
    }

    /**
     * @return null|Zend_Controller_Request_Abstract|Zend_Controller_Request_Http
     */
    protected function _getRequest()
    {
        return Zend_Controller_Front::getInstance()->getRequest();
    }

    protected function _getLocale()
    {
        return Zend_Registry::get('Zend_Locale');
    }
}

?>