<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

use Rc\Domain\Model\User;
use Rc\Domain\Repository\UserRepository;

class Identity
{

    /**
     * @var User
     */
    private static $user;

    /**
     * @var Zend_Session_Namespace
     */
    private static $session;

    private final function __construct()
    {
    }

    /**
     * @param User $user
     * @return bool
     */
    public static function canSwitchUser(User $user)
    {
        if (!self::exists()) {
            return false;
        }
        if (!self::isAdmin() || self::isUserSwitched()) {
            return false;
        }
        if ($user->isSuperAdmin() && !self::getUser()->isSuperAdmin()) {
            return false;
        }

        return true;
    }

    /**
     * @param User $user
     * @return bool
     */
    public static function switchUser(User $user)
    {
        if (!self::canSwitchUser($user)) {
            return false;
        }

        $session = self::_getSession();
        $switchedUser = Identity::getUserId();

        if (self::setIdentity($user->getEmail(), $user->getPassword(), true)) {
            self::$user = null;

            $session->switchedUser = $switchedUser;

            return true;
        } else {
            return false;
        }
    }

    /**
     * @return bool
     */
    public static function isUserSwitched()
    {
        return (bool)self::_getSession()->switchedUser;
    }

    /**
     * @return bool
     */
    public static function switchUserBack()
    {
        if (!self::isUserSwitched()) {
            return false;
        }

        $session = self::_getSession();
        $user = UserRepository::findById($session->switchedUser);

        if (self::setIdentity($user->getEmail(), $user->getPassword(), true)) {
            self::$user = null;

            $session->switchedUser = null;

            return true;
        }

        return false;
    }

    /**
     * @return bool|User
     */
    public static function getSwitchedUser()
    {
        if (!self::isUserSwitched()) {
            return false;
        }

        return UserRepository::getById(self::_getSession()->switchedUser);
    }

    /**
     * @return bool
     */
    public static function isSwitchedUserAdmin()
    {
        $switchedUser = self::getSwitchedUser();

        if (!$switchedUser) {
            return self::isAdmin();
        }

        return $switchedUser->isAdmin();
    }


    /**
     * @return User
     * @throws Exception
     */
    public static function getUser()
    {
        if (!self::exists()) {
            throw new Exception('AUTHENTIFICATION_REQUIRED');
        }

        if (!isset(self::$user)) {
            self::$user = UserRepository::findById(self::_getSession()->userId);
        }

        if (!isset(self::$user)) {
            self::clear();
            throw new Exception('AUTHENTIFICATION_REQUIRED');
        }

        return self::$user;
    }

    /**
     * @return User|null
     */
    public static function getUserOrNull()
    {
        return self::exists() ? self::getUser() : null;
    }

    /**
     * @deprecated
     * @return User
     * @throws Exception
     */
    public static function getPerson()
    {
        return self::getUser();
    }

    /**
     * @return int
     */
    public static function getUserId()
    {
        return self::_getSession()->userId;
    }

    /**
     * @return string
     */
    public static function getUserUuid()
    {
        return self::getUser()->getUuid();
    }

    /**
     * @return bool
     */
    public static function isAdmin()
    {
        return (bool)self::_getSession()->isAdmin;
    }

    /**
     * @return bool
     */
    public static function isSuperAdmin()
    {
        return (bool)self::getUser()->isSuperAdmin();
    }

    /**
     * @return bool
     */
    public static function isUserSupport()
    {
        return self::getUserId() == 45585 || self::getUser()->isSuperAdmin();
    }

    /**
     * @param string $email
     * @param string $password
     * @param bool $hashed
     * @return bool
     */
    public static function setIdentity($email, $password, $hashed = false)
    {
        if (empty($email) || empty($password)) {
            return false;
        }

        if (!$hashed) {
            $password = md5(config('app.salt') . $password);
        }

        $user = UserRepository::findByLoginInformation($email, $password);
        if (!isset($user)) {
            return false;
        }

        $session = self::_getSession();
        $session->userId = $user->getId();
        $session->isAdmin = $user->isAdmin();

        return true;
    }

    /**
     * @return bool
     */
    public static function exists()
    {
        return isset(self::_getSession()->userId);
    }

    /**
     * @return void
     */
    public static function clear()
    {
        Zend_Session::destroy();
    }

    /**
     * @return Zend_Session_Namespace
     */
    protected static function _getSession()
    {
        if (!isset(self::$session)) {
            self::$session = new Zend_Session_Namespace('rc.session');
        }
        return self::$session;
    }

    private final function __clone()
    {
    }

}