<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

class MySqlRepository extends AbstractRepository
{
    const MAX_ROW_COUNT = 30;
    const DATABASE_PREFIX = 'rc_';

    protected static $defaultConnection;

    protected static $query;

    protected static $numRows;

    protected static $foundRows;

    protected static $maxRowCount;

    protected static $orderBy = [
        'field' => 'id',
        'order' => 'ASC',
    ];

    /**
     * @var PDO
     */
    protected static $pdo;

    public static function establishConnection($config)
    {
        $connection = mysqli_init();
        if (mysqli_real_connect($connection, $config['host'], $config['user'], $config['password'], $config['database']) == false) {
            throw new Exception('MYSQL_CONNECTION_FAILED [' . mysqli_connect_errno() . ': ' . mysqli_connect_error() . ']');
        }
        mysqli_query($connection, 'SET NAMES utf8');
        mysqli_query($connection, 'SET SESSION sql_mode=(SELECT REPLACE(@@sql_mode,"ONLY_FULL_GROUP_BY",""));');
        mysqli_report(MYSQLI_REPORT_ERROR | MYSQLI_REPORT_STRICT);

        static::$pdo = new Pdo(
            'mysql:host=' . $config['host'] . ';dbname=' . $config['database'],
            $config['user'],
            $config['password'],
            [PDO::ATTR_ERRMODE => Pdo::ERRMODE_EXCEPTION]
        );
        static::$pdo->exec('SET NAMES utf8');

        return $connection;
    }

    /**
     * @return PDO
     */
    public static function getPdo()
    {
        return static::$pdo;
    }

    public static function getDefaultConnection()
    {
        return static::$defaultConnection;
    }

    public static function setDefaultConnection($defaultConnection)
    {
        if (is_array($defaultConnection)) {
            $defaultConnection = static::establishConnection($defaultConnection);
        }
        static::$defaultConnection = $defaultConnection;
    }

    public static function hasDefaultConnection()
    {
        return (boolean)static::getDefaultConnection();
    }

    public static function getNumRows()
    {
        return static::$numRows;
    }

    public static function getFoundRows()
    {
        return static::$foundRows;
    }

    public static function getPageCount()
    {
        if (empty(static::$maxRowCount)) {
            return null;
        }

        return ceil(static::$foundRows / static::$maxRowCount);
    }

    public static function findById($id, $limit = null)
    {
        if (empty($id)) {
            return null;
        }
        return static::_findBy('id', $id, 0, $limit, null);
    }

    /**
     * @param array $ids
     * @param int $page
     * @param int $limit
     * @param string $orderBy
     * @return \Rc\Domain\Model\User[]
     */
    public static function findByIds(array $ids, $page, $limit = null, $orderBy = null)
    {
        if (count($ids) == 0) {
            return [];
        }

        $result = static::_query('
            SELECT
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                `id` IN (' . static::_escape(implode(',', $ids)) . ')
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    public static function findAll($page = 0, $limit = null, $orderBy = null)
    {
        $result = static::_query('
            SELECT SQL_CALC_FOUND_ROWS
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                1
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        return static::_fetchAndFreeResults($result);
    }

    public static function update(MySqlEntity $object)
    {
        return static::_update($object->getId(), $object->getProperties(false));
    }

    public static function add(MySqlEntity $object)
    {
        $result = static::_add($object->getProperties(false), $id);
        $object->setId($id);

        return $result;
    }

    public static function remove(MySqlEntity $object)
    {
        return static::_remove($object->getId());
    }

    public static function query($query)
    {
        return static::_query($query);
    }

    public static function getConnection()
    {
        if (static::hasDefaultConnection()) {
            return static::getDefaultConnection();
        } else {
            throw new Exception('NO_MYSQL_CONNECTION_ESTABLISHED');
        }
    }

    protected static function _getModelName()
    {
        return substr(get_called_class(), 0, -strlen('Repository'));
    }

    protected static function _getMainTable()
    {
        return static::DATABASE_PREFIX . strtolower(static::_getModelName());
    }

    protected static function _getJoinedTables()
    {
        return '`' . static::_getMainTable() . '`';
    }

    protected static function _getFields()
    {
        $modelName = static::_getModelName();
        $object = new $modelName;
        $fields = [];

        foreach ($object->getProperties(false) as $property => $value) {
            $fields[] = '`' . $property . '`';
        }

        return implode(',', $fields);
    }

    protected static function _getLimit($page, $limit = null)
    {
        if (empty($limit)) {
            $limit = static::MAX_ROW_COUNT;
        }

        static::$maxRowCount = $limit;

        return (int)$page * (int)$limit . ',' . (int)$limit;
    }

    protected static function _getOrder()
    {
        return '`name` ASC';
    }

    protected static function _fetchAndFreeResults($result)
    {
        return (array)static::_fetchAndFreeResult($result, true);
    }

    protected static function _fetchAndFreeResult($result, $return_as_array = false)
    {
        if (empty($result)) {
            static::_printError();
            return null;
        }

        $modelName = static::_getModelName();
        if (class_exists($modelName) == false) {
            throw new Exception('UNKNOWN MODEL NAME [' . $modelName . ']');
        }

        if ($return_as_array == false) {
            $retVal = null;
            if (mysqli_num_rows($result) > 0) {
                $row = $result->fetch_object();
                $retVal = new $modelName($row);
            }
        } else {
            $retVal = [];
            while ($row = $result->fetch_object()) {
                $retVal[] = new $modelName($row);
            }
        }

        static::$numRows = mysqli_num_rows($result);

        if (strpos(static::$query, 'SQL_CALC_FOUND_ROWS') === false) {
            static::$foundRows = null;
        } else {
            $result = static::_query('SELECT FOUND_ROWS() foundRows');
            $row = $result->fetch_object();
            static::$foundRows = $row->foundRows;
        }

        mysqli_free_result($result);

        return $retVal;
    }

    protected static function _printError()
    {
        if (APPLICATION_ENV == 'development') {
            throw new Exception('MySql-Error: ' . mysqli_error(static::getConnection()) . '</p><p><b>Query:</b>' . static::$query);
        }
    }

    protected static function _query($query)
    {
        static::$query = $query;
        return mysqli_query(static::getConnection(), $query);
    }

    protected static function _getInsertId()
    {
        return mysqli_insert_id(static::getConnection());
    }

    protected static function _escape($value, $trim = true)
    {
        if ($trim) {
            $value = trim($value);
        }
        return static::getConnection()->real_escape_string($value);
    }

    protected static function _update($id, $values)
    {
        foreach ($values as $key => &$value) {
            if (is_null($value)) {
                $value = '`' . $key . '` = NULL';
            } else {
                $value = '`' . $key . '` = "' . static::_escape($value) . '"';
            }
        }

        $result = static::_query('
            UPDATE
                ' . static::_getJoinedTables() . '
            SET
                ' . implode(',', $values) . '
            WHERE
                `id` = ' . static::_escape($id) . '
        ');

        if (empty($result)) {
            static::_printError();
            return false;
        }

        return (boolean)$result;
    }

    protected static function _add($values, &$id)
    {
        foreach ($values as $key => &$value) {
            if (is_null($value)) {
                $value = '`' . $key . '` = NULL';
            } else {
                $value = '`' . $key . '` = "' . static::_escape($value) . '"';
            }
        }

        $result = static::_query('
            INSERT
                ' . static::_getJoinedTables() . '
            SET
                ' . implode(',', $values) . '
        ');

        if (empty($result)) {
            static::_printError();
            return false;
        }

        if (func_num_args() >= 2) {
            $id = static::_getInsertId();
        }

        return (boolean)$result;
    }

    protected static function _remove($id)
    {
        $result = static::_query('
            DELETE FROM
                ' . static::_getJoinedTables() . '
            WHERE
                `id` = ' . static::_escape($id) . '
        ');

        if (empty($result)) {
            static::_printError();
            return false;
        }

        return (boolean)$result;
    }

    protected static function _findBy($field, $value, $page = null, $limit = null, $orderBy = null)
    {
        $fields = (array)$field;
        $values = (array)$value;
        $condition = [];

        foreach ($fields as $key => $field) {
            if (!static::_isField($field)) {
                throw new Exception('UNKNOWN_FIELD [' . $field . ']');
            }

            if ($values[$key] === null) {
                $condition[] = '`' . $field . '` IS NULL';
            } else {
                $condition[] = '`' . $field . '` = "' . static::_escape($values[$key]) . '"';
            }
        }

        $result = static::_query('
            SELECT SQL_CALC_FOUND_ROWS
                ' . static::_getFields() . '
            FROM
                ' . static::_getJoinedTables() . '
            WHERE
                ' . implode(' AND ', $condition) . '
            ORDER BY
                ' . static::_getValidOrderBy($orderBy) . '
            LIMIT
                ' . static::_getLimit($page, $limit) . '
        ');

        if (array_search('id', $fields) !== false && count($fields) == 1) {
            return static::_fetchAndFreeResult($result);
        } else {
            return static::_fetchAndFreeResults($result);
        }
    }

    protected static function _isField($field)
    {
        $reflection = new ReflectionClass(static::_getModelName());
        $properties = $reflection->getProperties(ReflectionProperty::IS_PROTECTED);

        foreach ($properties as $property) {
            if (strtolower($field) == strtolower($property->getName())) {
                return true;
            }
        }
        return false;
    }

    protected static function _getValidOrderBy($orderBy = null)
    {
        foreach ((array)$orderBy as $field => $order) {
            if ($order != 'ASC' && $order != 'DESC') {
                continue;
            }

            if (static::_isField($field) === false) {
                continue;
            }

            $retVal[] = '`' . $field . '` ' . $order;
        }

        if (isset($retVal) == false) {
            $retVal[] = '`' . static::$orderBy['field'] . '` ' . static::$orderBy['order'];
        }

        return implode(',', $retVal);
    }

}

?>