<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Doctrine\ODM\MongoDB\Event;

use Doctrine\ODM\MongoDB\Event\PreUpdateEventArgs;

class PreUpdatedEventHelper
{
    /**
     * @var PreUpdateEventArgs
     */
    protected $args;

    /**
     * @var object
     */
    protected $object;

    /**
     * @param PreUpdateEventArgs $args
     * @param object $object
     */
    public function __construct(PreUpdateEventArgs $args, $object)
    {
        $this->args = $args;
        $this->object = $object;
    }

    /**
     * @param array $fields
     * @return bool
     */
    public function hasChangedFields(array $fields)
    {
        foreach ($fields as $field) {
            if ($this->args->hasChangedField($field)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param array|string $field
     * @return array
     */
    public function getNewValue($field)
    {
        if (is_array($field)) {
            return $this->mergeFields(function ($field) {
                return $this->getNewValue($field);
            }, $field);
        }

        if ($this->args->hasChangedField($field)) {
            return $this->args->getNewValue($field);
        }

        return $this->object->{'get' . $field}(false);
    }

    /**
     * @param array|string $field
     * @return array
     */
    public function getOldValue($field)
    {
        if (is_array($field)) {
            return $this->mergeFields(function ($field) {
                return $this->getOldValue($field);
            }, $field);
        }


        if ($this->args->hasChangedField($field)) {
            return $this->args->getOldValue($field);
        }

        return $this->object->{'get' . $field}(false);
    }

    /**
     * @param array|string $field
     * @return array
     */
    public function getRemovedValues($field)
    {
        if (is_array($field)) {
            return $this->mergeFields(function ($field) {
                return $this->getRemovedValues($field);
            }, $field);
        }

        return $this->diffLists($this->getOldValue($field), $this->getNewValue($field));
    }

    /**
     * @param array|string $field
     * @return array
     */
    public function getAddedValues($field)
    {
        if (is_array($field)) {
            return $this->mergeFields(function ($field) {
                return $this->getAddedValues($field);
            }, $field);
        }

        return $this->diffLists($this->getNewValue($field), $this->getOldValue($field));
    }

    /**
     * @param array $list1
     * @param array $list2
     * @return array
     */
    protected function diffLists($list1, $list2)
    {
        $list1 = $this->getListValue($list1);
        $list2 = $this->getListValue($list2);

        if (!isset($list1) || !isset($list2)) {
            throw new \InvalidArgumentException('Cannot calculate diff between given list because at least one of them is not traversable!');
        }

        return array_diff($list1, $list2);
    }

    /**
     * @param callable $callback
     * @param array $fields
     * @return array
     */
    protected function mergeFields(callable $callback, array $fields)
    {
        $result = [];
        foreach (array_map($callback, $fields) as $values) {
            $result = array_merge($result, $this->getListValue($values));
        }
        return $result;
    }

    /**
     * @param mixed $value
     * @return array|null
     */
    protected function getListValue($value)
    {
        if (!isset($value)) {
            return null;
        }

        if (is_array($value)) {
            return $value;
        }

        if ($value instanceof \Traversable) {
            return iterator_to_array($value);
        }

        return null;
    }
}