<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Exception;

abstract class AbstractDilpsModel extends AbstractModel
{
    const ATTRIBUTE_THUMB_FILE_NAME = 'thumb_file_id';

    const MAX_DESCRIPTION_LENGTH = 1000;

    const DEFAULT_LANGUAGE = 'en';

    const DEFAULT_DATE_FORMAT = 'm/d/Y';

    const DEFAULT_THUMB_NAME = 'placeholder.png';

    const DEFAULT_THUMB_WIDTH = 150;

    const DEFAULT_THUMB_HEIGHT = 150;

    const ATTRIBUTE_FILE_POSITIONS_NAME = 'file-positions';

    const GROUP_OWNER_FORMAT = 'user%s';

    const MEDIA_TYPE_IMAGE = 'image';

    const MEDIA_TYPE_VIDEO = 'video';

    const MEDIA_TYPE_AUDIO = 'audio';

    const MEDIA_TYPE_TEXT = 'text';

    const MEDIA_TYPE_PDF = 'pdf';

    const MEDIA_TYPE_SHAPE = 'shape';

    const MEDIA_TYPE_ARCHIVE = 'archive';

    const MEDIA_TYPE_SIMPLE_TEXT = 'simpletext';

    const MEDIA_TYPE_EMBED = 'embed';

    const ACCESS_LEVEL_PRIVATE = 0;

    const ACCESS_LEVEL_LIMITED = 1;

    const ACCESS_LEVEL_PUBLIC = 2;

    const ACCESS_LEVEL_PORTAL = 3;

    /**
     * @var Translation
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\Translation")
     */
    protected $title;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     * @ODM\Index(unique=false)
     * @ODS\Type(name="Rc\Serializer\Type\Owner")
     */
    protected $owner;

    /**
     * @var User
     * @ODS\Exclude()
     */
    protected $originalOwner;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     * @ODS\Type(name="Rc\Serializer\Type\Owner")
     */
    protected $author;

    /**
     * @var Translation
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\Translation")
     */
    protected $description;

    /**
     * @var User
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $textAuthor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $textModified;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $date;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $text;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $mediaType;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $lastModified;

    /**
     * AbstractDilpsModel constructor.
     */
    public function __construct()
    {
        parent::__construct();
        $this->title = new Translation([]);
        $this->description = new Translation([]);
        $this->lastModified = time();
    }

    /**
     * @deprecated
     */
    public function __call($method, $args)
    {
        if (substr($method, 0, 3) == 'get' && substr($method, -4) == 'List' && isset($method[8])) {
            return $this->{substr($method, 0, -4)}();
        }
        throw new Exception('unsupported method call: ' . $method);
    }

    /**
     * @return User
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner)
    {
        if (!isset($this->originalOwner)) {
            $this->originalOwner = $this->owner;
        }

        $this->owner = $owner;

        return $this;
    }

    /**
     * @param User $owner
     * @return bool
     */
    public function isOwner(User $owner)
    {
        return $this->isEqual($this->owner ?: $this->author, $owner);
    }

    /**
     * @param User $owner
     * @return bool
     */
    public function wasOwner(User $owner)
    {
        return $this->isEqual($this->originalOwner, $owner);
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return get_called_class() . '#' . $this->getUuid();
    }

    /**
     * @return Translation
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param Translation $title
     * @return $this
     */
    public function setTitle(Translation $title)
    {
        $this->title = $title;
        return $this;
    }

    /**
     * @deprecated use getOwner() instead
     * @return User|null
     */
    public function getAuthor()
    {
        return $this->author ?: $this->owner;
    }

    /**
     * @deprecated
     * @todo: check the amount of entities which differs in owner/author and maybe we can remove the author support
     * @param User $author
     * @return $this
     */
    public function setAuthor(User $author = null)
    {
        if ($this->isEqual($author, $this->owner)) {
            $this->author = null;
        } else {
            $this->author = $author;
        }
        return $this;
    }

    /**
     * @deprecated
     * @param User $author
     * @return bool
     */
    public function isAuthor(User $author)
    {
        return $this->isEqual($this->getAuthor(), $author);
    }

    /**
     * @return Translation
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param Translation $description
     * @return $this
     */
    public function setDescription(Translation $description)
    {
        $this->description = $description;
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getDate($format = true)
    {
        return $this->formatDateTime($this->date, $format);
    }

    /**
     * @param string $date
     * @return void
     */
    public function setDate($date)
    {
        $this->date = $this->castToDateTime($date);
    }

    /**
     * @return string
     */
    public function getTextContent()
    {
        return $this->text;
    }

    /**
     * @param string $text
     */
    public function setTextContent($text)
    {
        $this->setMediaType(self::MEDIA_TYPE_TEXT);
        $this->text = $text;
    }

    /**
     * @return bool
     */
    public function hasTextContent()
    {
        return isset($this->text);
    }

    /**
     * @return User
     */
    public function getTextAuthor()
    {
        return $this->textAuthor;
    }

    /**
     * @param User $user
     */
    public function setTextAuthor(User $user)
    {
        $this->textAuthor = $user;
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isTextAuthor(User $user)
    {
        return $this->isEqual($user, $this->getTextAuthor());
    }

    /**
     * @return string
     */
    public function getTextModified()
    {
        return $this->textModified;
    }

    /**
     * @param string $date
     */
    public function setTextModified($date)
    {
        $this->textModified = $date;
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isMetaEditableBy(User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        return $this->isOwner($user);
    }

    /**
     * @return $this
     */
    public function getMediaEntity()
    {
        return $this;
    }

    /**
     * @return null|string
     */
    public function getMediaType()
    {
        if (!isset($this->mediaType)) {
            return null;
        }
        return $this->mediaType;
    }

    /**
     * @param string $mediaType
     */
    public function setMediaType($mediaType)
    {
        $this->mediaType = $mediaType;
    }

    /**
     * manual modified flag, e.g. for research ordering
     *
     * @return int
     */
    public function getLastModified()
    {
        return $this->lastModified;
    }

    /**
     * manual modified flag, e.g. for research ordering
     *
     * @param integer|null $timestamp
     */
    public function setLastModified($timestamp = null)
    {
        $this->lastModified = $timestamp ?: time();
    }
}