<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use DateTime;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\Core\DateTimeTrait;
use Rc\Domain\Model\Core\FormValuesTrait;
use Rc\Domain\Model\Core\OrmAwareTrait;
use Rc\Domain\Repository\MongoRepository;

/**
 * @ODM\Document
 * @ODM\HasLifecycleCallbacks
 */
abstract class AbstractModel implements ModelInterface
{
    use OrmAwareTrait;
    use FormValuesTrait;
    use DateTimeTrait;

    /**
     * @var string
     * @ODM\Id()
     * @ODS\Type(name="Rc\Serializer\Type\Uuid")
     */
    protected $uuid;

    /**
     * @var int
     * @ODM\Field(type="int")
     * @ODM\Index(order="asc")
     * @ODS\Type(name="Rc\Serializer\Type\Id")
     */
    protected $id;

    /**
     * @var ModelInterface
     * @ODM\ReferenceOne(storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $source;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $revision;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $sourceId;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $created;

    /**
     * @var DateTime
     * @ODM\Field(type="date")
     */
    protected $modified;

    /**
     * @var bool
     * @ODM\Field(type="bool")
     */
    protected $deleted = false;

    /**
     * @var bool
     * @ODS\Exclude()
     */
    private $new = true;

    public function __construct()
    {
        $this->setCreated(new DateTime());
        $this->setModified(new DateTime());
    }

    /**
     * @return string
     */
    public function getName()
    {
        $pieces = explode('\\', get_called_class());
        return strtolower(end($pieces));
    }

    /**
     * @return string
     */
    public function getUuid()
    {
        return $this->uuid;
    }

    /**
     * @internal
     * @param string $uuid
     * @return void
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @internal
     * @param int $id
     * @return void
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @param string|bool $format
     * @return DateTime|string
     */
    public function getCreated($format = true)
    {
        return $this->formatDateTime($this->created, $format);
    }

    /**
     * @param DateTime $created
     * @return void
     */
    public function setCreated(DateTime $created = null)
    {
        $this->created = $created;
    }

    /**
     * @param string|bool $format
     * @return DateTime|string
     */
    public function getModified($format = true)
    {
        return $this->formatDateTime($this->modified, $format);
    }

    /**
     * @param DateTime $modified
     * @return void
     */
    public function setModified(DateTime $modified = null)
    {
        $this->modified = $modified;
    }

    /**
     * @return boolean
     */
    public function getDeleted()
    {
        return $this->deleted;
    }

    /**
     * @param boolean $deleted
     * @return void
     */
    public function setDeleted($deleted)
    {
        $this->deleted = $deleted;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return get_class($this) . '#' . $this->getId();
    }

    /**
     * @deprecated
     * @return array
     */
    public function getValues()
    {
        throw new \LogicException('not implemented, but not needed');
    }

    /**
     * @return int
     */
    public function getSourceId()
    {
        return $this->sourceId;
    }

    /**
     * @param int $sourceId
     */
    public function setSourceId($sourceId)
    {
        $this->sourceId = $sourceId;
    }

    /**
     * @return ModelInterface
     */
    public function getSource()
    {
        return $this->source;
    }

    /**
     * @param ModelInterface $model
     */
    public function setSource(ModelInterface $model)
    {
        $this->source = $model;
    }

    /**
     * @return DateTime
     */
    public function getRevision()
    {
        return $this->revision;
    }

    /**
     * @param DateTime|null $revision
     */
    public function setRevision(DateTime $revision = null)
    {
        $this->revision = $revision;
    }

    /**
     * @return void
     */
    public function __clone()
    {
        $this->sourceId = $this->id ?: null;
        $this->uuid = null;
        $this->id = null;
        $this->new = true;
    }

    /**
     * @ODM\PrePersist
     */
    public function generateAutoIncrementId()
    {
        if (isset($this->id)) {
            return;
        }

        $this->id = MongoRepository::enforceAutoIncrementId(ModelInterface::class);
    }

    /**
     * @ODM\PostPersist
     */
    public function maintainIdMapping()
    {
        if ($this->new && $this->id) {
            MongoRepository::persistIdMapping(new IdMapping(get_class($this), $this->id, $this->uuid));
        }

        $this->new = false;
    }

    /**
     * @return bool
     */
    protected function isNew()
    {
        return $this->new;
    }

    /**
     * @param bool $flag
     */
    public function setNew(bool $flag)
    {
        $this->new = $flag;
    }

}
