<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Application;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\AbstractModel;
use Rc\Domain\Model\Application\Form\Element;
use Rc\Domain\Model\Application\Form\FormElement;
use Rc\Domain\Model\Application\Form\LayoutElement;
use Rc\Domain\Model\Portal;
use Rc\Utility\Strings;

/**
 * @ODM\Document
 */
class ApplicationForm extends AbstractModel
{
    const GENRE_APPLICATION = 'application';
    const GENRE_REVIEW = 'review';

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $title;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $genre;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $portal;

    /**
     * @var ArrayCollection|ApplicationProgram[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Application\ApplicationProgram", mappedBy="applicationForm", storeAs="dbRef")
     */
    protected $applicationPrograms;

    /**
     * @var ArrayCollection|ApplicationProgram[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Application\ApplicationProgram", mappedBy="reviewForm", storeAs="dbRef")
     */
    protected $reviewPrograms;

    /**
     * @var ArrayCollection|Element[]
     * @ODM\EmbedMany(strategy="set")
     */
    protected $elements;

    public function __construct()
    {
        parent::__construct();
        $this->applicationPrograms = new ArrayCollection();
        $this->reviewPrograms = new ArrayCollection();
        $this->elements = new ArrayCollection();
    }

    /**
     * @return array
     */
    public static function getGenres()
    {
        return [
            self::GENRE_APPLICATION => 'category.application-form',
            self::GENRE_REVIEW      => 'category.review-form',
        ];
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     */
    public function setPortal(Portal $portal)
    {
        $this->portal = $portal;
    }

    /**
     * @return string
     */
    public function getGenre()
    {
        return $this->genre;
    }

    /**
     * @param string $genre
     */
    public function setGenre($genre)
    {
        $this->genre = $genre;
    }

    /**
     * @return Element[]
     */
    public function getElements()
    {
        return $this->elements->toArray();
    }

    /**
     * @return FormElement[]
     */
    public function getFormElements()
    {
        $formElements = [];
        foreach ($this->elements as $key => $element) {
            if (!$element instanceof FormElement) {
                continue;
            }
            $formElements[$key] = $element;
        }
        return $formElements;
    }

    /**
     * @param Element[] $elements
     */
    public function setElements($elements)
    {
        $this->elements = new ArrayCollection();
        foreach ($elements as $element) {
            $this->addElement($element);
        }
    }

    /**
     * @return array
     */
    public function getElementArray()
    {
        $elements = [];
        foreach ($this->elements as $key => $element) {
            $elements[$key] = $element->toArray();
        }
        return $elements;
    }

    /**
     * @param Element $element
     */
    public function addElement(Element $element)
    {
        if (!$this->elements->offsetExists($element->getIdentifier())) {
            $this->elements->set($element->getIdentifier(), $element);
        }
    }

    /**
     * @return ApplicationForm[]
     */
    public function getPrograms()
    {
        return $this->getProgramList()->toArray();
    }

    /**
     * @return bool
     */
    public function hasPrograms()
    {
        return !$this->getProgramList()->isEmpty();
    }

    /**
     * @param array $values
     * @return $this
     */
    public function setValues(array $values)
    {
        if (isset($values['element'])) {
            $elements = [];
            foreach ($values['element'] as $element) {
                $instance = $this->createElementInstance($element);
                if (!isset($instance)) {
                    continue;
                }
                $instance->setValues($element);
                $elements[] = $instance;
            }
            $this->setElements($elements);
            unset($values['element']);
        }

        parent::setValues($values);

        return $this;
    }

    /**
     * @return ArrayCollection|ApplicationForm[]
     */
    protected function getProgramList()
    {
        if (!isset($this->genre)) {
            throw new \LogicException('Application form does not have a genre set!');
        }
        switch ($this->genre) {
            case self::GENRE_APPLICATION:
                return $this->applicationPrograms;
            case self::GENRE_REVIEW:
                return $this->reviewPrograms;
            default:
                throw new \LogicException('Application form type "' . $this->genre . '" is not supported!');
        }
    }

    /**
     * @param array $element
     * @return Element|null
     */
    protected function createElementInstance($element)
    {
        $classNames = [
            'form'   => FormElement::class . '\\' . Strings::toCamelCase($element['element'], '-', true) . 'FormElement',
            'layout' => LayoutElement::class . '\\' . Strings::toCamelCase($element['element'], '-', true) . 'LayoutElement',
        ];

        foreach ($classNames as $className) {
            if (!is_a($className, Element::class, true)) {
                continue;
            }
            return new $className();
        }
        return null;
    }
}