<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core\Acl;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\Core\OrmAwareTrait;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\User;
use Zend_Registry;

/**
 * @mixin OrmAwareTrait
 * @mixin AclAwareInterface
 * @mixin ModelInterface
 */
trait AclAwareTrait
{

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $aclv2 = [
        'role' => [
            'read'  => [],
            'write' => [],
        ],
        'user' => [
            'read'  => [],
            'write' => [],
        ],
    ];

    /**
     * @var bool
     * @ODS\Exclude()
     */
    protected $generateOptimizedAcl = false;

    /**
     * @return void
     */
    public function generateOptimizedAcl()
    {
        if (!$this->generateOptimizedAcl) {
            return;
        }

        $userList = [];
        foreach ($this->aclv2['role']['write'] as $tmpUserList) {
            foreach ($tmpUserList as $tmpUser) {
                $userList[$tmpUser] = true;
            }
        }
        $this->aclv2['user']['write'] = array_keys($userList);

        foreach ($this->aclv2['role']['read'] as $tmpUserList) {
            foreach ($tmpUserList as $tmpUser) {
                $userList[$tmpUser] = true;
            }
        }
        $this->aclv2['user']['read'] = array_keys($userList);
    }

    /**
     * @param array  $group
     * @param string $context
     */
    public function addReadAcl(array $group, $context)
    {
        if (empty($group)) {
            $this->removeReadAcl($context);
            return;
        }

        if (!isset($this->aclv2['role']['read'][$context]) || $this->aclv2['role']['read'][$context] != $group) {
            $this->aclv2['role']['read'][$context] = $group;
            $this->generateOptimizedAcl = true;
        }
    }

    /**
     * @param array  $group
     * @param string $context
     */
    public function addWriteAcl(array $group, $context)
    {
        if (empty($group)) {
            $this->removeWriteAcl($context);
            return;
        }

        if (!isset($this->aclv2['role']['write'][$context]) || $this->aclv2['role']['write'][$context] != $group) {
            $this->aclv2['role']['write'][$context] = $group;
            $this->generateOptimizedAcl = true;
        }
    }

    /**
     * @param string $context
     * @return void
     */
    public function removeReadAcl($context)
    {
        unset($this->aclv2['role']['read'][$context]);
        $this->generateOptimizedAcl = true;
    }

    /**
     * @param string $context
     * @return void
     */
    public function removeWriteAcl($context)
    {
        unset($this->aclv2['role']['write'][$context]);
        $this->generateOptimizedAcl = true;
    }

    /**
     * @param string $context
     * @return void
     */
    public function removeAcl($context)
    {
        $this->removeReadAcl($context);
        $this->removeWriteAcl($context);
    }

    /**
     * @return void
     */
    public function removeAllReadAcl()
    {
        $this->aclv2['role']['read'] = [];
        $this->aclv2['user']['read'] = [];
        $this->generateOptimizedAcl = true;
    }

    /**
     * @return void
     */
    public function removeAllWriteAcl()
    {
        $this->aclv2['role']['write'] = [];
        $this->aclv2['user']['write'] = [];
        $this->generateOptimizedAcl = true;
    }

    /**
     * @return void
     */
    public function removeAllAcl()
    {
        $this->removeAllReadAcl();
        $this->removeAllWriteAcl();
    }

    /**
     * @return void
     */
    public function convertAllWriteAclToReadAcl()
    {
        $this->aclv2['user']['read'] = array_merge($this->aclv2['user']['read'], $this->aclv2['user']['write']);
        foreach ($this->aclv2['role']['write'] as $role => $acl) {
            if (empty($this->aclv2['role']['read'][$role])) {
                $this->aclv2['role']['read'][$role] = $acl;
            } else {
                $this->aclv2['role']['read'][$role] = array_unique(array_merge($this->aclv2['role']['read'][$role], $acl));
            }
        }

        $this->removeAllWriteAcl();
    }

    /**
     * @param User   $user
     * @param string $type
     * @return bool
     */
    public function isAllowed(User $user = null, $type = self::TYPE_WRITE)
    {
        if ($type == self::TYPE_READ && !empty(Zend_Registry::get('RC_DILPS_AUTH')->getGroups()[$this->getId() . '_reader'])) {
            return true;
        } elseif ($type == self::TYPE_WRITE && !empty(Zend_Registry::get('RC_DILPS_AUTH')->getGroups()[$this->getId() . '_writer'])) {
            return true;
        }

        $this->generateOptimizedAcl();

        $groups = $user ? $user->getGroups() : AclGroup::RC_GUEST();
        $allowedGroups = array_flip(isset($this->aclv2['user'][$type]) ? $this->aclv2['user'][$type] : []);
        foreach ($groups as $group) {
            if (isset($allowedGroups[$group])) {
                return true;
            }
        }

        return false;
    }
}
