<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core;

use Doctrine\Common\Collections\Collection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Exception;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\MongoRepository;

trait OrmAwareTrait
{

    /**
     * @param Collection $collection
     * @return int[]|integer[]
     */
    protected function createIdList(Collection $collection)
    {
        $idList = [];
        foreach ($collection as $document) {
            try {
                $idList[] = $document->getId();
            } catch (DocumentNotFoundException $exception) {
                trigger_error($exception->getMessage(), E_USER_WARNING);
            }
        }
        return $idList;
    }

    /**
     * @param string $type
     * @param array $list
     * @return array
     * @throws Exception
     */
    protected function createReferenceList($type, array $list)
    {
        foreach ($list as $key => $value) {
            $list[$key] = $this->createReference($type, $value);
        }
        return array_filter($list); #todo: remove filter
    }

    /**
     * @param string $type
     * @param string|object $id
     * @return mixed|object
     * @throws Exception
     */
    protected function createReference($type, $id)
    {
        if (!isset($id) || $id === '') {
            return null;
        }

        if (is_object($id)) {
            return $id;
        }

        $mappedId = $this->mapId($type, $id);
        if (!isset($mappedId)) {
            return null;
        }

        // workaround to prevent doctrine from creating abstract classes. we should find a cleaner solution to avoid performance problems
        if (IdMappingRepository::isAbstractClass($type)) {
            return MongoRepository::getClient()->find($type, (string)$mappedId);
        } else {
            return MongoRepository::getClient()->getReference($type, (string)$mappedId);
        }
    }

    /**
     * @todo: type hint
     * @param ModelInterface $model1
     * @param ModelInterface $model2
     * @return bool
     */
    protected function isEqual($model1 = null, $model2 = null)
    {
        if (!isset($model1) || !isset($model2)) {
            return false;
        }
        return $model1->getId() == $model2->getId();
    }

    /**
     * @param array $list
     * @return int[]
     */
    protected function castListToInt(array $list)
    {
        return array_map(function ($value) {
            return (int)$value;
        }, $list);
    }

    /**
     * @param array $list
     * @return string[]
     */
    protected function castListToString(array $list)
    {
        return array_map(function ($value) {
            return (string)$value;
        }, $list);
    }

    /**
     * @param string $type
     * @param mixed $id
     * @return string
     */
    protected function mapId($type, $id)
    {
        if (!is_numeric($id)) {
            return $id;
        }

        $idMapping = IdMappingRepository::getByTypeAndReferenceId($type, $id);
        if (!isset($idMapping)) {
            return null;
        }

        return $idMapping->getReferenceUuid();
    }
}
