<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Core\Revision;

use Doctrine\ODM\MongoDB\MongoDBException;
use Doctrine\ODM\MongoDB\UnitOfWork;
use Exception;
use Identity;
use MongoDBRef;
use MongoException;
use MongoId;
use Rc\Domain\Model\Core\Revision;
use Rc\Domain\Repository\MongoRepository;

class RevisionFactory
{

    /**
     * @param RevisionAwareInterface|Revision $document
     * @param string $label
     * @throws MongoDBException
     * @throws MongoException
     */
    public static function create($document, $label = null)
    {
        if (PHP_SAPI == 'cli') {
            return;
        }

        if (!Identity::exists()) {
            return;
        }

        $label = $label ?: implode(',', $document->getRevisionComments());
        if (empty($label)) {
            return;
        }

        $client = MongoRepository::getClient();
        if ($client->getUnitOfWork()->getDocumentState($document) != UnitOfWork::STATE_MANAGED) {
            return;
        }

        $aggregationBuilder = $client->getDocumentCollection(get_class($document))->createAggregationBuilder();
        $aggregationBuilder->match()
            ->field('_id')
            ->equals(new MongoId($document->getUuid()));

        $project = $aggregationBuilder->project();
        $project->field('_id')->expression(['$literal' => new MongoId()]);
        $project->field('label')->expression(['$literal' => (string)$label]);
        $project->field('created')->expression(['$literal' => time()]);
        $project->field('user')->expression(['$literal' => self::getIdentityDbRef()]);
        $project->field('document')->expression('$$ROOT');
        $project->field('_class')->expression(['$literal' => Revision::class]);

        $revisions = [];
        foreach ($aggregationBuilder->execute(['cursor' => true])->toArray() as $revision) {
            $revision['document']['_doctrine_class_name'] = $revision['document']['_class'];
            $revisions[] = $revision;
        }

        if (empty($revisions)) {
            throw new \LogicException(
                'Can not create revision for document ' . get_class($document) . '#' . $document->getUuid() . '!'
            );
        }

        $client->getDocumentCollection(Revision::class)->batchInsert($revisions);
    }

    /**
     * @return array
     * @throws MongoDBException
     * @throws MongoException
     * @throws Exception
     */
    protected static function getIdentityDbRef()
    {
        $identity = Identity::getUser();
        $documentName = MongoRepository::getClient()->getDocumentCollection(get_class($identity))->getName();

        return MongoDBRef::create($documentName, new MongoId($identity->getUuid()));
    }
}