<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\Core\Acl\AclAwareInterface;
use Rc\Domain\Model\Core\Acl\AclAwareTrait;
use Rc\Domain\Model\Core\Acl\AclContext;
use Rc\Domain\Model\Core\Acl\AclGroup;
use Rc\Domain\Model\Generic\KeywordAwareInterface;
use Rc\Domain\Model\Generic\KeywordAwareTrait;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\MediaContainer;
use Rc\Domain\Model\Research\SimpleMediaAwareInterface;
use Rc\Domain\Model\Research\SimpleMediaAwareTrait;
use Rc\Domain\Repository\ResearchRepository;

/**
 * @ODM\Document(collection="Research")
 * @ODM\InheritanceType("SINGLE_COLLECTION")
 * @ODM\HasLifecycleCallbacks()
 * @ODM\DiscriminatorField("_class")
 * @ODM\Index(keys={"aclv2.user.read"="asc", "_class"="asc", "status"="asc", "accesslevel"="asc"})
 * @ODM\Index(name="text", keys={
 *     "title.en"="text",
 *     "affiliation"="text",
 *     "reference"="text",
 *     "referenceUrl"="text",
 *     "tocAlignment"="text",
 *     "description.en"="text",
 *     "text"="text",
 *     "copyright"="text",
 *     "place"="text",
 *     "material"="text",
 *     "technique"="text",
 *     "format"="text",
 *     "publisher"="text",
 *     "freeTextAuthor"="text",
 *     "mainFieldOfStudy"="text",
 *     "mainFieldOfStudyType"="text",
 *     "area"="text",
 *     "fullText"="text"
 * })
 */
abstract class Research extends AbstractDilpsModel implements AclAwareInterface, SimpleMediaAwareInterface, KeywordAwareInterface
{
    use KeywordAwareTrait;
    use AclAwareTrait;
    use SimpleMediaAwareTrait;

    const VAL_TITLE_LENGTH_MIN = 4;
    const VAL_TITLE_LENGTH_MAX = 255;
    const VAL_DESCRIPTION_LENGTH_MIN = 10;
    const VAL_DESCRIPTION_LENGTH_MAX = 5000;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $hide = false;

    /**
     * relations between researches, works, projects and degrees
     *
     * @var ArrayCollection
     * @ODM\ReferenceMany()
     */
    protected $relations;

    /**
     * @var int
     * @ODM\Field(type="int")
     */
    protected $accessLevel = self::ACCESS_LEVEL_PRIVATE;

    /**
     * @var ArrayCollection|Portal[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $accessLevelPortals;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $partOfPortalRequest;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $partOfPortal;

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $pendingInvites = [];

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $disapprovedCollaborators = [];

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $coAuthors;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $contributors;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $supervisors;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $editors;

    /**
     * @var array
     * @ODS\Exclude()
     */
    protected $collaboratorRoles = ['editor', 'supervisor', 'coauthor', 'contributor'];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $fullText = '';

    public function __construct()
    {
        parent::__construct();
        $this->coAuthors = new ArrayCollection();
        $this->contributors = new ArrayCollection();
        $this->supervisors = new ArrayCollection();
        $this->editors = new ArrayCollection();
        $this->accessLevelPortals = new ArrayCollection();
        $this->relations = new ArrayCollection();
        $this->mediaList = new ArrayCollection();
    }

    /**
     * @ODM\PreUpdate
     * @ODM\PrePersist
     */
    public function optimizeAclOnPersist()
    {
        $this->updateAclForContributors();
        $this->generateOptimizedAcl();
    }

    /**
     * @return SimpleMediaInterface[]
     */
    public function getMediaList()
    {
        foreach ($this->mediaList as $media) {
            try {
                $media->getDescription();
            } catch (DocumentNotFoundException $exception) {
                trigger_error($exception->getMessage(), E_USER_WARNING);

                $this->mediaList->removeElement($media);
                ResearchRepository::persist($this);
            }
        }
        return $this->mediaList->toArray();
    }

    /**
     * @inheritdoc
     */
    public function getMediaContainer()
    {
        if (!isset($this->thumb)) {
            return new MediaContainer();
        }

        try {
            $this->thumb->getId();
        } catch (DocumentNotFoundException $exception) {
            trigger_error($exception->getMessage(), E_USER_WARNING);
            return new MediaContainer();
        }

        return new MediaContainer(
            $this->thumb->getMedia(),
            $this->thumb->getThumb(),
            $this->thumb->getName(),
            $this->thumb->getId()
        );
    }

    /**
     * @return boolean
     */
    public function isHidden()
    {
        return $this->hide;
    }

    /**
     * @return array
     */
    public function getCollaborationRoles()
    {
        return $this->collaboratorRoles;
    }

    /**
     * @param boolean $hide
     * @return void
     */
    public function setHide($hide)
    {
        $this->hide = $hide;
    }

    /**
     * @return bool
     */
    public function isPartOfPortal()
    {
        return isset($this->partOfPortal);
    }

    /**
     * @return int
     */
    public function getPartOfPortalId()
    {
        if (!isset($this->partOfPortal)) {
            return null;
        }
        return $this->partOfPortal->getId();
    }

    /**
     * @return \Rc\Domain\Model\Portal|null
     */
    public function getPartOfPortal()
    {
        return $this->partOfPortal;
    }

    /**
     * @param int $portalId
     */
    public function setPartOfPortalId($portalId)
    {
        $this->partOfPortal = $this->createReference(Portal::class, $portalId);
    }

    /**
     * @return string
     */
    public function getPartOfPortalRequestId()
    {
        if (!isset($this->partOfPortalRequest)) {
            return null;
        }
        return $this->partOfPortalRequest->getId();
    }

    /**
     * @param string $portalId
     */
    public function setPartOfPortalRequestId($portalId)
    {
        $this->partOfPortalRequest = $this->createReference(Portal::class, $portalId);
    }

    /**
     * @return int
     */
    public function getAccessLevel()
    {
        return $this->accessLevel;
    }

    /**
     * @todo
     * @param int $accessLevel
     */
    public function setAccessLevel($accessLevel)
    {
        switch ($accessLevel) {
            case self::ACCESS_LEVEL_PUBLIC:
                $this->addReadAcl(AclGroup::RC_GUEST(), AclContext::ACCESS_LEVEL());
                break;
            case self::ACCESS_LEVEL_LIMITED:
                $this->addReadAcl(AclGroup::RC_USER(), AclContext::ACCESS_LEVEL());
                break;
            case self::ACCESS_LEVEL_PORTAL:
                $this->addReadAcl(AclGroup::PORTAL($this->getAccessLevelPortalsIds()), AclContext::ACCESS_LEVEL());
                break;
            default:
                $this->removeReadAcl(AclContext::ACCESS_LEVEL());
                break;
        }
        $this->accessLevel = $accessLevel;
    }

    /**
     * @param string[] $portalIds
     */
    public function setAccessLevelPortalsIds($portalIds)
    {
        $this->accessLevelPortals = new ArrayCollection($this->createReferenceList(Portal::class, $portalIds));
        $this->setAccessLevel($this->getAccessLevel());
    }

    /**
     * @return integer[]
     */
    public function getAccessLevelPortalsIds()
    {
        return $this->createIdList($this->accessLevelPortals);
    }

    /**
     * @return integer[]
     */
    public function getRelations()
    {
        return $this->createIdList($this->relations);
    }

    /**
     * @param string[] $relations
     */
    public function setRelations(array $relations)
    {
        $this->relations = new ArrayCollection($this->createReferenceList(Research::class, $relations));
    }

    /**
     * @param User $collaborator
     * @return bool
     */
    public function isInvitedAsCollaborator(User $collaborator)
    {
        return $this->isInvitePending($collaborator->getId());
    }

    /**
     * @param int $userId
     * @param string $role
     */
    public function addDisapprovedCollaborator($userId, $role)
    {
        $this->{'set' . ucfirst($role) . 's'}(array_merge($this->{'get' . ucfirst($role) . 's'}(), [$userId]));
        $this->disapprovedCollaborators = array_merge($this->disapprovedCollaborators, [$role => [$userId => true]]);
    }

    /**
     * @param int $userId
     * @param string $role
     */
    public function removeDisapprovedCollaborator($userId, $role)
    {
        if (isset($this->disapprovedCollaborators[$role][$userId])) {
            unset($this->disapprovedCollaborators[$role][$userId]);
        }
    }

    /**
     * @param int[] $userIds
     * @param string $role
     */
    public function removeDisapprovedCollaborators(array $userIds, $role)
    {
        foreach ($userIds as $userId) {
            $this->removeDisapprovedCollaborator($userId, $role);
        }
    }

    /**
     * @param string $userId
     * @param string|array|null $roles
     * @return bool
     */
    public function isDisapprovedCollaborator($userId, $roles = null)
    {
        if (!isset($roles)) {
            $roles = $this->collaboratorRoles;
        } else {
            if (!is_array($roles)) {
                $roles = (array)$roles;
            }
        }
        foreach ($roles as $role) {
            if (isset($this->disapprovedCollaborators[$role][$userId])) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return array
     */
    public function getDisapprovedCollaborators()
    {
        return $this->disapprovedCollaborators + $this->collaboratorRoles;
    }

    /**
     * @param array $disapprovedCollaborators
     */
    public function setDisapprovedCollaborators(array $disapprovedCollaborators)
    {
        $this->disapprovedCollaborators = $disapprovedCollaborators;
    }

    public function getPendingInvites()
    {
        return $this->pendingInvites + $this->collaboratorRoles;
    }

    /**
     * @param array $pendingInvites
     */
    public function setPendingInvites(array $pendingInvites)
    {
        $this->pendingInvites = $pendingInvites;
    }

    /**
     * @param int $userId
     * @param string|array|null $roles
     * @return bool
     */
    public function isInvitePending($userId, $roles = null)
    {
        if (!isset($roles)) {
            $roles = $this->collaboratorRoles;
        } else {
            if (!is_array($roles)) {
                $roles = (array)$roles;
            }
        }

        foreach ($roles as $role) {
            if (isset($this->pendingInvites[$role][$userId])) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param array $userIds
     * @param string $role
     */
    public function addPendingInvites(array $userIds, $role)
    {
        if (empty($userIds) || $role === 'editor') {
            return;
        }

        foreach ($userIds as $userId) {
            if (!isset($this->pendingInvites[$role])) {
                $this->pendingInvites[$role] = [];
            }
            $this->pendingInvites[$role][$userId] = true;
        }

        $this->{'set' . $role . 's'}($this->{'get' . $role . 's'}()); // update acls
    }

    /**
     * @param int[] $userIds
     * @param string $role
     */
    public function removePendingInvites(array $userIds, $role)
    {
        if (empty($userIds) || $role === 'editor') {
            return;
        }

        foreach ($userIds as $userId) {
            if (isset($this->pendingInvites[$role][$userId])) {
                unset($this->pendingInvites[$role][$userId]);
            }
        }

        $this->{'set' . $role . 's'}($this->{'get' . $role . 's'}()); // update acls
    }

    /**
     * @return void
     */
    public function removePendingContributors()
    {
        foreach ($this->pendingInvites as $role => $userIds) {
            $this->{'remove' . $role . 'sByIds'}(array_keys($userIds));
        }
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isMetaEditableBy(User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        if (!$this->isAllowed($user)) {
            return false;
        }

        return $this->isOwner($user) || $this->isCoAuthor($user);
    }

    /**
     * @param User $owner
     * @return $this
     */
    public function setOwner(User $owner)
    {
        parent::setOwner($owner);

        $this->addWriteAcl(AclGroup::USER([$owner->getId()]), AclContext::OWNER());
        return $this;
    }

    /**
     * @param User $author
     * @return $this
     */
    public function setAuthor(User $author = null)
    {
        parent::setAuthor($author);

        $this->addWriteAcl(AclGroup::USER($author ? [$author->getId()] : []), AclContext::AUTHOR());
        return $this;
    }

    /**
     * @return bool
     */
    public function hasCollaborateurs()
    {
        return count($this->getCollaborateurs()) > 0;
    }

    /**
     * @return array
     */
    public function getCollaborateurs()
    {
        return array_merge(
            $this->createIdList($this->coAuthors),
            $this->createIdList($this->contributors),
            $this->createIdList($this->supervisors),
            $this->createIdList($this->editors)
        );
    }

    /**
     * @return User[]
     */
    public function getCollaborateurList()
    {
        return array_merge(
            $this->coAuthors->toArray(),
            $this->contributors->toArray(),
            $this->supervisors->toArray(),
            $this->editors->toArray()
        );
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isCollaborateur(User $user)
    {
        return $this->isCoAuthor($user) || $this->isContributor($user);
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isCoAuthor(User $user)
    {
        return !$this->isInvitePending($user->getId(), 'coauthor') && $this->coAuthors->contains($user);
    }

    /**
     * @param bool $asIdList
     * @return int[]
     */
    public function getCoAuthors($asIdList = true)
    {
        if (!$asIdList) {
            return $this->coAuthors->toArray();
        }

        return $this->createIdList($this->coAuthors);
    }

    /**
     * @param array $userList
     */
    public function setCoAuthors(array $userList)
    {
        $idList = $this->getCoAuthors();
        $this->removePendingInvites(array_diff($idList, $userList), 'coauthor');
        $this->removeDisapprovedCollaborators(array_diff($userList, $idList), 'coauthor');
        $this->coAuthors = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addWriteAcl(AclGroup::USER($userList), AclContext::COAUTHOR());
    }

    /**
     * @return bool
     */
    public function hasCoAuthors()
    {
        return !$this->coAuthors->isEmpty();
    }

    /**
     * @param User $user
     */
    public function removeCoAuthor(User $user)
    {
        $this->coAuthors->removeElement($user);
        $this->updateProfileGroup($user);

        $this->setCoAuthors($this->getCoAuthors());
    }

    /**
     * @param array $userIds
     */
    public function removeCoAuthorsByIds(array $userIds)
    {
        foreach ($this->coAuthors as $user) {
            if (in_array($user->getId(), $userIds)) {
                $this->removeCoAuthor($user);
            }
        }
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isEditor(User $user)
    {
        return !$this->isInvitePending($user->getId(), 'editor') && $this->editors->contains($user);
    }

    /**
     * @param bool $asIdList
     * @return int[]
     */
    public function getEditors($asIdList = true)
    {
        if (!$asIdList) {
            return $this->editors->toArray();
        }

        return $this->createIdList($this->editors);
    }

    /**
     * @param array $userList
     */
    public function setEditors(array $userList)
    {
        $idList = $this->getEditors();
        $this->removePendingInvites(array_diff($idList, $userList), 'editor');
        $this->removeDisapprovedCollaborators(array_diff($userList, $idList), 'editor');
        $this->editors = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addWriteAcl(AclGroup::USER($userList), AclContext::EDITOR());
    }

    /**
     * @return bool
     */
    public function hasEditors()
    {
        return !$this->editors->isEmpty();
    }

    /**
     * @param User $user
     */
    public function removeEditor(User $user)
    {
        $this->editors->removeElement($user);
        $this->updateProfileGroup($user);

        $this->setEditors($this->getEditors());
    }

    /**
     * @param array $userIds
     */
    public function removeEditorsByIds(array $userIds)
    {
        foreach ($this->editors as $user) {
            if (in_array($user->getId(), $userIds)) {
                $this->removeEditor($user);
            }
        }
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isContributor(User $user)
    {
        return !$this->isInvitePending($user->getId(), 'contributor') && $this->contributors->contains($user);
    }

    /**
     * @param bool $asIdList
     * @return int[]
     */
    public function getContributors($asIdList = true)
    {
        if (!$asIdList) {
            return $this->contributors->toArray();
        }

        return $this->createIdList($this->contributors);
    }

    /**
     * @param array $userList
     */
    public function setContributors(array $userList)
    {
        $idList = $this->getContributors();
        $this->removePendingInvites(array_diff($idList, $userList), 'contributor');
        $this->removeDisapprovedCollaborators(array_diff($userList, $idList), 'contributor');
        $this->contributors = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addWriteAcl(AclGroup::USER($userList), AclContext::CONTRIBUTOR());
    }

    /**
     * @param User $user
     */
    public function removeContributor(User $user)
    {
        $this->contributors->removeElement($user);
        $this->updateProfileGroup($user);

        $this->setContributors($this->getContributors());
    }

    /**
     * @param array $userIds
     */
    public function removeContributorsByIds(array $userIds)
    {
        foreach ($this->contributors as $user) {
            if (in_array($user->getId(), $userIds)) {
                $this->removeContributor($user);
            }
        }
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isSupervisor(User $user)
    {
        return !$this->isInvitePending($user->getId(), 'supervisor') && $this->supervisors->contains($user);
    }

    /**
     * @param bool $asIdList
     * @return int[]
     */
    public function getSupervisors($asIdList = true)
    {
        if (!$asIdList) {
            return $this->supervisors->toArray();
        }
        return $this->createIdList($this->supervisors);
    }

    /**
     * @param array $userList
     */
    public function setSupervisors(array $userList)
    {
        $idList = $this->getSupervisors();
        $this->removePendingInvites(array_diff($idList, $userList), 'supervisor');
        $this->removeDisapprovedCollaborators(array_diff($userList, $idList), 'supervisor');
        $this->supervisors = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addReadAcl(AclGroup::USER($userList), AclContext::SUPERVISOR());
    }

    /**
     * @param User $user
     */
    public function removeSupervisor(User $user)
    {
        $this->supervisors->removeElement($user);
        $this->updateProfileGroup($user);

        $this->setSupervisors($this->getSupervisors());
    }

    /**
     * @param array $userIds
     */
    public function removeSupervisorsByIds(array $userIds)
    {
        foreach ($this->supervisors as $user) {
            if (in_array($user->getId(), $userIds)) {
                $this->removeSupervisor($user);
            }
        }
    }

    /**
     * @param User $user
     * @return array
     */
    public function getRoleNames(User $user = null)
    {
        if (!isset($user)) {
            return [];
        }

        $roles = [];
        $roles['author'] = $this->isAuthor($user);
        $roles['coauthor'] = $this->isCoAuthor($user);
        $roles['contributor'] = $this->isContributor($user);
        $roles['editor'] = $this->isEditor($user);
        $roles['supervisor'] = $this->isSupervisor($user);

        return array_filter($roles);
    }

    /**
     * @return string
     */
    public function getFullText()
    {
        return $this->fullText;
    }

    /**
     * @param string $fullText
     */
    public function setFullText($fullText)
    {
        $this->fullText = $fullText;
    }

    /**
     * Deletes all external references from mongodb, so that only the research remains alive. Needed to replace a research with all references during a restore process.
     *
     * Should be overwritten by the concrete implementation
     */
    public function deleteRelations() {

    }

    /**
     * @return void
     */
    protected function updateAclForContributors()
    {
        if (!$this->generateOptimizedAcl) {
            return;
        }

        foreach ($this->pendingInvites as $role => $pendingInvites) {
            foreach ($pendingInvites as $userId => $isInvitePending) {
                $userGroup = AclGroup::USER($userId);

                if (!empty($this->aclv2['role']['read'][$role])) {
                    foreach ($this->aclv2['role']['read'][$role] as $key => $value) {
                        if ($value == $userGroup) {
                            unset($this->aclv2['role']['read'][$role][$key]);
                        }
                    }
                }

                if (!empty($this->aclv2['role']['write'][$role])) {
                    foreach ($this->aclv2['role']['write'][$role] as $key => $value) {
                        if ($value == $userGroup) {
                            unset($this->aclv2['role']['write'][$role][$key]);
                        }
                    }
                }
            }
        }
    }

    /**
     * @param User $user
     */
    protected function updateProfileGroup(User $user)
    {
        if ($this->isOwner($user) || $this->isAuthor($user)) {
            return;
        }
        if ($this->isCollaborateur($user) || $this->isSupervisor($user)) {
            return;
        }
        \ProfileGroupElementRepository::removeByUserIdAndEntityIds($user->getId(), [$this->getId()]);
    }

}
