<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Application\ApplicationForm;
use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Application\ApplicationSnapshot;
use Rc\Domain\Model\Application\Form\FormElement\FileFormElement;
use Rc\Domain\Model\Application\Form\FormElement\PdfFormElement;
use Rc\Domain\Model\Application\Form\FormElementValueContainer;
use Rc\Domain\Model\Application\Generic\MetaDataAwareTrait;
use Rc\Domain\Model\Application\Review;
use Rc\Domain\Model\Core\Acl\AclContext;
use Rc\Domain\Model\Core\Acl\AclGroup;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\ResearchRepository;
use Zend_Registry;

/**
 * @ODM\Document
 * @ODM\HasLifecycleCallbacks
 */
class Application extends Research implements StatusAwareInterface
{
    use StatusAwareTrait;
    use MetaDataAwareTrait;

    /**
     * @var FormElementValueContainer[]
     * @ODM\EmbedMany(targetDocument="Rc\Domain\Model\Application\Form\FormElementValueContainer", strategy="set")
     */
    protected $requiredDocuments;

    /**
     * @var FormElementValueContainer[]
     * @ODM\EmbedMany(targetDocument="Rc\Domain\Model\Application\Form\FormElementValueContainer", strategy="set")
     */
    protected $optionalDocuments;

    /**
     * @var ArrayCollection|Review[]
     * @ODM\EmbedMany(targetDocument="Rc\Domain\Model\Application\Review", strategy="set")
     */
    protected $reviews;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $portal;

    /**
     * @var ApplicationProgram
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Application\ApplicationProgram", storeAs="dbRef")
     */
    protected $program;

    /**
     * @var ArrayCollection|Exposition[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition", storeAs="dbRef")
     */
    protected $expositions;

    /**
     * @var ArrayCollection|Work[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Work", storeAs="dbRef")
     */
    protected $works;

    /**
     * @var ArrayCollection|ApplicationSnapshot[]
     * @ODM\EmbedMany(targetDocument="Rc\Domain\Model\Application\ApplicationSnapshot")
     */
    protected $snapshots;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $reviewers;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     */
    protected $externalReaders;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     */
    protected $applicantPortal;

    /**
     * @var bool
     * @ODM\Field(type="bool")
     */
    protected $valid;

    public function __construct()
    {
        parent::__construct();
        $this->expositions = new ArrayCollection();
        $this->works = new ArrayCollection();
        $this->snapshots = new ArrayCollection();
        $this->reviews = new ArrayCollection();
        $this->metaData = new ArrayCollection();
        $this->requiredDocuments = new ArrayCollection();
        $this->optionalDocuments = new ArrayCollection();
        $this->externalReaders = new ArrayCollection();
        $this->reviewers = new ArrayCollection();
        $this->status = self::STATUS_PROGRESS;
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        // add in review step immediately publication read rights for portal admins, because
        // they will always have access to application (also if the application will be declined)

        switch ($status) {
            case self::STATUS_PRE_REVIEW:
                $this->addReadAcl(AclGroup::PORTAL_REVIEWER([$this->getApplicantPortalId()]), AclContext::REVIEW());
                $this->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getApplicantPortalId()]), AclContext::PUBLICATION());
                break;

            case self::STATUS_REVIEW:
                $this->addReadAcl(AclGroup::PORTAL_REVIEWER([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::REVIEW());
                $this->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::PUBLICATION());
                break;

            case self::STATUS_PUBLISHED:
                if (!$this->isPublished()) {
                    $this->setLastModified();
                }

                $this->setReviewerIds([]);
                $this->removeExternalDependencies();

                $this->convertAllWriteAclToReadAcl();
                $this->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::PUBLICATION());
                break;

            case self::STATUS_REWORK:
                $this->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::PUBLICATION());
                $this->removeAcl(AclContext::REVIEW());
                break;

            case self::STATUS_DECLINED:
                $this->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::PUBLICATION());
                $this->removeAcl(AclContext::REVIEW());
                break;

            case self::STATUS_PROGRESS:
                $this->removeAcl(AclContext::REVIEW());
                $this->removeAcl(AclContext::PUBLICATION());
                break;

            default:
                throw new \InvalidArgumentException('Unknown status change to "' . $status . '"!', 1496038944);
        }

        $this->status = $status;
    }

    /**
     * @ODM\PreUpdate
     * @ODM\PrePersist
     */
    public function optimizeAclOnPersist()
    {
        $this->setStatus($this->getStatus());
        parent::optimizeAclOnPersist();
        $this->updateAclForLinkedResearches();
    }

    /**
     * @return ArrayCollection|MediaInterface[]
     */
    public function getRequiredDocuments()
    {
        $requiredDocuments = [];
        foreach ($this->requiredDocuments as $key => $container) {
            $requiredDocuments[$key] = $container->getValue();
        }
        return $requiredDocuments;
    }

    /**
     * @param array|MediaInterface[] $metaData
     * @param bool                   $preserveFiles
     */
    public function setRequiredDocuments(array $metaData, $preserveFiles = true)
    {
        foreach ($metaData as $identifier => $value) {
            if ($value === null) {
                unset($metaData[$identifier]);
                unset($this->requiredDocuments[$identifier]);
                continue;
            }

            $element = $this->getRequiredDocument($identifier);
            if (!isset($element) || $value === '') {
                unset($metaData[$identifier]);
                continue;
            }

            if ($value instanceof MediaInterface) {
                $value->setEmbedded(true);
            }

            $metaData[$identifier] = new FormElementValueContainer($element, $value);
        }

        if ($preserveFiles) {
            foreach ($this->requiredDocuments as $key => $container) {
                if ($container->getElement()->requiresUpload() && !isset($metaData[$key])) {
                    $metaData[$key] = $container;
                }
            }
        }

        $this->requiredDocuments = new ArrayCollection($metaData);
    }

    /**
     * @return ArrayCollection|MediaInterface[]
     */
    public function getOptionalDocuments()
    {
        $optionalDocuments = [];
        foreach ($this->optionalDocuments as $key => $container) {
            $optionalDocuments[$key] = $container->getValue();
        }
        return $optionalDocuments;
    }

    /**
     * @param array|MediaInterface[] $metaData
     * @param bool                   $preserveFiles
     */
    public function setOptionalDocuments(array $metaData, $preserveFiles = true)
    {
        foreach ($metaData as $identifier => $value) {
            if ($value === null) {
                unset($metaData[$identifier]);
                unset($this->optionalDocuments[$identifier]);
                continue;
            }

            $element = $this->getOptionalDocument($identifier);
            if (!isset($element) || $value === '') {
                unset($metaData[$identifier]);
                continue;
            }

            if ($value instanceof MediaInterface) {
                $value->setEmbedded(true);
            }

            $metaData[$identifier] = new FormElementValueContainer($element, $value);
        }

        if ($preserveFiles) {
            foreach ($this->optionalDocuments as $key => $container) {
                if ($container->getElement()->requiresUpload() && !isset($metaData[$key])) {
                    $metaData[$key] = $container;
                }
            }
        }

        $this->optionalDocuments = new ArrayCollection($metaData);
    }

    /**
     * @param Review $review
     */
    public function addReview(Review $review)
    {
        $this->reviews->set($review->getAuthor()->getId(), $review);
    }

    /**
     * @param User $user
     * @return Review
     */
    public function getReview(User $user)
    {
        $review = $this->reviews->get($user->getId());
        if (!isset($review)) {
            $review = new Review($this);
            $review->setAuthor($user);

            $this->addReview($review);
        }
        return $review;
    }

    /**
     * @param User   $user
     * @param string $status
     * @return bool
     */
    public function hasReview(User $user, $status = Review::STATUS_SUBMITTED)
    {
        return !$this->reviews->filter(function (Review $review) use ($user, $status) {
            return $review->getAuthor() === $user && $review->hasStatus($status);
        })->isEmpty();
    }

    /**
     * @param string $status
     * @return Review[]
     */
    public function getReviews($status = null)
    {
        if (!isset($status)) {
            return $this->reviews;
        }
        return $this->reviews->filter(function (Review $comment) use ($status) {
            return $comment->hasStatus($status);
        })->toArray();
    }

    /**
     * @param Review[] $reviews
     */
    public function setReviews($reviews)
    {
        $this->reviews->clear();
        array_map([$this, 'addReview'], $reviews);
    }

    /**
     * @return bool
     */
    public function isReviewed()
    {
        return count($this->getReviews(Review::STATUS_SUBMITTED)) == $this->reviewers->count() && $this->reviewers->count() > 0;
    }

    /**
     * @return ApplicationProgram
     */
    public function getProgram()
    {
        return $this->program;
    }

    /**
     * @param ApplicationProgram $program
     */
    public function setProgram($program)
    {
        $this->program = $program;
        $this->portal = $program->getPortal();
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     */
    public function setPortal(Portal $portal)
    {
        throw new \LogicException('The portal of an application cannot be changed directly! Replace the application program instead.');
    }

    /**
     * @return int
     */
    public function getPortalId()
    {
        if (!isset($this->portal)) {
            return null;
        }
        return $this->portal->getId();
    }

    /**
     * @return Portal
     */
    public function getApplicantPortal()
    {
        return $this->applicantPortal;
    }

    /**
     * @param Portal $applicantPortal
     */
    public function setApplicantPortal($applicantPortal)
    {
        $this->applicantPortal = $applicantPortal;
    }

    /**
     * @return int
     */
    public function getApplicantPortalId()
    {
        if (!isset($this->applicantPortal)) {
            return null;
        }
        return $this->applicantPortal->getId();
    }

    /**
     * @return bool
     */
    public function hasApplicantPortal()
    {
        return isset($this->applicantPortal);
    }

    /**
     * @return Exposition[]
     */
    public function getExpositions()
    {
        return $this->expositions->toArray();
    }

    /**
     * @return Exposition[]
     */
    public function getBoundExpositions()
    {
        if (!$this->hasSnapshots()) {
            return $this->getExpositions();
        }

        return $this->getLatestSnapshot()->getExpositions();
    }

    /**
     * @param int $id
     * @return bool
     */
    public function containsBoundExpositionId($id) {
        foreach ($this->getBoundExpositions() as $exposition) {
            if ($exposition->getId() == $id) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param integer[] $expositionIds
     */
    public function setExpositionIds($expositionIds)
    {
        foreach ($this->expositions as $exposition) {
            $exposition->removeAcl(AclContext::APPLICATION($this));
            $this->persistLinkedResearch($exposition);
        }

        $this->expositions = new ArrayCollection($this->createReferenceList(Exposition::class, $expositionIds));
    }

    /**
     * @return integer[]
     */
    public function getExpositionIds()
    {
        return $this->createIdList($this->expositions);
    }

    /**
     * @return Work[]
     */
    public function getWorks()
    {
        return $this->works->toArray();
    }

    /**
     * @return Work[]
     */
    public function getBoundWorks()
    {
        if (!$this->hasSnapshots()) {
            return $this->getWorks();
        }

        return $this->getLatestSnapshot()->getWorks();
    }

    /**
     * @param int $id
     * @return bool
     */
    public function containsBoundWorkId($id) {
        foreach ($this->getBoundWorks() as $work) {
            if ($work->getId() == $id) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param integer[] $workIds
     */
    public function setWorkIds($workIds)
    {
        foreach ($this->works as $work) {
            $work->removeAcl(AclContext::APPLICATION($this));
            $this->persistLinkedResearch($work);
        }

        $this->works = new ArrayCollection($this->createReferenceList(Work::class, $workIds));
    }

    /**
     * @return integer[]
     */
    public function getWorkIds()
    {
        return $this->createIdList($this->works);
    }

    /**
     * @return Research[]
     */
    public function getResearches()
    {
        return array_merge($this->expositions->toArray(), $this->works->toArray());
    }

    /**
     * @return ApplicationSnapshot[]
     */
    public function getSnapshots()
    {
        if (!isset($this->snapshots)) {
            $this->snapshots = new ArrayCollection();
        }

        return $this->snapshots->toArray();
    }

    /**
     * @return bool
     */
    public function hasSnapshots()
    {
        if (!isset($this->snapshots)) {
            $this->snapshots = new ArrayCollection();
        }

        return !$this->snapshots->isEmpty();
    }

    /**
     * @return ApplicationSnapshot
     */
    public function getLatestSnapshot()
    {
        if (!isset($this->snapshots)) {
            $this->snapshots = new ArrayCollection();
        }

        return $this->snapshots->last() ?: null;
    }

    /**
     * @param ApplicationSnapshot $snapshot
     */
    public function addSnapshot(ApplicationSnapshot $snapshot)
    {
        if (!isset($this->snapshots)) {
            $this->snapshots = new ArrayCollection();
        }

        foreach ($snapshot->getResearches() as $research) {
            $research->removeAllAcl();
            $research->addReadAcl(AclGroup::PORTAL_ADMIN([$this->getPortalId(), $this->getApplicantPortalId()]), AclContext::PUBLICATION());
        }
        $this->snapshots->add($snapshot);
    }

    /**
     * @return ArrayCollection|User[]
     */
    public function getReviewers()
    {
        return $this->reviewers;
    }

    /**
     * @return integer[]
     */
    public function getReviewerIds()
    {
        return $this->createIdList($this->reviewers);
    }

    /**
     * @param integer[] $reviewerIds
     */
    public function setReviewerIds($reviewerIds)
    {
        $this->reviewers = new ArrayCollection($this->createReferenceList(User::class, $reviewerIds));

        $this->addWriteAcl(AclGroup::USER($reviewerIds), AclContext::REVIEWER());
    }

    /**
     * @return ArrayCollection|User[]
     */
    public function getExternalReaders()
    {
        return $this->externalReaders;
    }

    /**
     * @return bool
     */
    public function hasExternalReaders()
    {
        return !$this->externalReaders->isEmpty();
    }

    /**
     * @return integer[]
     */
    public function getExternalReaderIds()
    {
        return $this->createIdList($this->externalReaders);
    }

    /**
     * @param integer[] $externalReaderIds
     */
    public function setExternalReaderIds($externalReaderIds)
    {
        $this->externalReaders = new ArrayCollection($this->createReferenceList(User::class, $externalReaderIds));

        $this->addWriteAcl(AclGroup::USER($externalReaderIds), AclContext::EXTERNAL_READER());
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isReviewer(User $user)
    {
        return $this->reviewers->contains($user);
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isReader(User $user)
    {
        return $this->externalReaders->contains($user);
    }

    /**
     * @return bool
     */
    public function isValid()
    {
        return $this->valid;
    }

    /**
     * @param bool $valid
     */
    public function setValid($valid)
    {
        $this->valid = $valid;
    }

    /**
     * @return void
     */
    protected function updateAclForLinkedResearches()
    {
        if (!$this->generateOptimizedAcl) {
            return;
        }

        foreach ($this->works as $work) {
            $work->addReadAcl($this->aclv2['user']['read'], AclContext::APPLICATION($this));
            $this->persistLinkedResearch($work);
        }

        foreach ($this->expositions as $exposition) {
            $exposition->addReadAcl($this->aclv2['user']['read'], AclContext::APPLICATION($this));
            $this->persistLinkedResearch($exposition);
        }
    }

    /**
     * @return ApplicationForm
     */
    protected function getMetadataForm()
    {
        return $this->program->getApplicationForm();
    }

    /**
     * @param string $identifier
     * @return \Rc\Domain\Model\Application\Form\FormElement\PdfFormElement
     */
    protected function getRequiredDocument($identifier)
    {
        if (!in_array($identifier, $this->program->getRequiredDocuments(true))) {
            return null;
        }

        $element = new PdfFormElement();
        $element->setIdentifier($identifier);
        $element->setLabel($identifier);
        $element->setRequired(true);
        return $element;
    }

    /**
     * @param string $identifier
     * @return \Rc\Domain\Model\Application\Form\FormElement\FileFormElement
     */
    protected function getOptionalDocument($identifier)
    {
        $element = new FileFormElement();
        $element->setIdentifier($identifier);
        $element->setLabel($identifier);
        return $element;
    }

    /**
     * @param Research $research
     */
    private function persistLinkedResearch(Research $research): void
    {
        Zend_Registry::get('RC_DILPS_AUTH')->addTemporaryEditGroupFor($research->getId());
        ResearchRepository::persist($research, false);
    }
}
