<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Identity;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\Core\Acl\AclContext;
use Rc\Domain\Model\Core\Acl\AclGroup;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\TableOfContent;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\MongoRepository;
use Rc\Domain\Repository\Portal\IssueRepository;
use Rc\Domain\Repository\Research\ApplicationRepository;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;

/**
 * @ODM\Document
 */
class Exposition extends Research implements StatusAwareInterface, DoiAwareInterface, ObjectShareAwareInterface
{
    use StatusAwareTrait;
    use DoiAwareTrait;
    use ObjectShareAwareTrait;

    const EDITOR_TYPE_TEXT_BASED = 'text-based';
    const EDITOR_TYPE_GRAPHICAL = 'graphical';

    /**
     * @var MediaInterface
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Media", cascade={"persist"}, storeAs="dbRef")
     */
    protected $preview;

    /**
     * @var Weave
     * @ODM\ReferenceOne(storeAs="dbRef")
     */
    protected $defaultWeave;

    /**
     * @var ArrayCollection|Weave[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition\Weave", mappedBy="exposition", cascade={"all"}, storeAs="dbRef")
     * @ODS\Type(name="ArrayCollection<Rc\Serializer\Type\Weave>")
     */
    protected $weaves;

    /**
     * @var ArrayCollection|SimpleMedia[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition\SimpleMedia", mappedBy="research", cascade={"all"}, storeAs="dbRef")
     * @ODS\Type(name="ArrayCollection<Rc\Serializer\Type\Embed>")
     */
    protected $simpleMedias;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $portal;

    /**
     * @var ArrayCollection|Portal[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $additionalPortals;

    /**
     * @var Portal
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal", storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $pendingAdditionalPortal;

    /**
     * @var string
     * @ODM\Field(type="string")
     * @ODS\Exclude()
     */
    protected $portalEdition;

    /**
     * @var Issue
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Portal\Issue", inversedBy="expositions")
     * @ODS\Exclude()
     */
    protected $portalIssue;

    /**
     * @var Research[]|Collection
     * @ODM\ReferenceMany(storeAs="dbRef")
     */
    protected $shares;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $reviewers;

    /**
     * @var ArrayCollection|User[]
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\User", storeAs="dbRef")
     * @ODS\Exclude()
     */
    protected $doubleBlindReviewers;

    /**
     * @var array
     * @ODM\Field(type="collection")
     */
    protected $groups = [];

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $affiliation;

    /**
     * @var \DateTime
     * @ODM\Field(type="date")
     */
    protected $publishDate;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $blocked = false;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $reference;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $referenceUrl;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $abstractOnly = false;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $reviewTag;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $reviewComment;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $publishRange;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $tocAlignment;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $displayMenuBarPermanently = false;

    /**
     * @var TableOfContent
     * @ODM\EmbedOne(targetDocument="\Rc\Domain\Model\Research\Exposition\TableOfContent")
     */
    protected $tableOfContent;

    /**
     * @deprecated is only used as readonly field if $this->>tableOfContent isn't initialized.
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $chapterList = [];

    public function __construct()
    {
        parent::__construct();
        $this->weaves = new ArrayCollection();
        $this->reviewers = new ArrayCollection();
        $this->doubleBlindReviewers = new ArrayCollection();
        $this->additionalPortals = new ArrayCollection();
        $this->mediaList = new ArrayCollection();
        $this->shares = new ArrayCollection();
        $this->simpleMedias = new ArrayCollection();
        $this->status = self::STATUS_PROGRESS;
    }

    /**
     * @return array
     */
    public static function getEditorTypes()
    {
        return [
            self::EDITOR_TYPE_GRAPHICAL  => 'editor.types.graphical',
            self::EDITOR_TYPE_TEXT_BASED => 'editor.types.text-based',
        ];
    }

    /**
     * @return MediaInterface
     */
    public function getPreview()
    {
        return $this->preview;
    }

    /**
     * @param MediaInterface $preview
     */
    public function setPreview($preview)
    {
        $this->preview = $preview;
    }

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        return new MediaContainer($this->preview, null);
    }

    /**
     * @inheritdoc
     */
    public function deleteRelations()
    {
        WeaveRepository::removeCollection($this->weaves);
        $this->weaves = new ArrayCollection();

        SimpleMediaRepository::removeCollection($this->simpleMedias);
        $this->simpleMedias = new ArrayCollection();
    }

    /**
     * @param string $status
     */
    public function setStatus($status)
    {
        switch ($status) {
            case self::STATUS_REVIEW:
                $this->addWriteAcl([AclGroup::PORTAL_ADMIN($this->getPortalId())], AclContext::REVIEW());
                $this->addReadAcl([AclGroup::PORTAL_REVIEWER($this->getPortalId())], AclContext::REVIEW());
                break;

            case self::STATUS_REVIEW_PUBLISHED:
                $this->addWriteAcl([AclGroup::PORTAL_ADMIN($this->getAdditionalPendingPortalId())], AclContext::REVIEW());
                $this->addReadAcl([AclGroup::PORTAL_REVIEWER($this->getAdditionalPendingPortalId())], AclContext::REVIEW());
                break;

            case self::STATUS_PUBLISHED:
                if (!$this->isPublished()) {
                    $this->setPublishedDate(new \DateTime());
                    $this->setLastModified();
                }

                $this->resetSubmissionFields();
                $this->removeExternalDependencies();

                $this->addReadAcl(AclGroup::RC_GUEST(), AclContext::PUBLICATION());
                $this->removeAcl(AclContext::REVIEW());
                $this->convertAllWriteAclToReadAcl();
                break;

            case self::STATUS_PROGRESS:
                $this->setPortalId(null);
                $this->setAdditionalPortalIds([]);
                $this->setAdditionalPendingPortalId(null);
                $this->setPublishedDate(null);

                $this->resetSubmissionFields();

                $this->removeAcl(AclContext::REVIEW());
                $this->removeAcl(AclContext::PUBLICATION());
                break;

            default:
                throw new \InvalidArgumentException('Unknown status change to "' . $status . '"!', 1496038944);
        }

        $this->status = $status;
    }

    /**
     * @inheritdoc
     */
    public function removeExternalDependencies()
    {
        foreach ($this->weaves as $weave) {
            $weave->removeExternalDependencies();
        }
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isEditableBy(User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        if (!$this->isStatus(self::STATUS_PROGRESS)) {
            return false;
        }

        return $this->isOwner($user) || $this->isCollaborateur($user);
    }

    /**
     * @return string
     */
    public function getAffiliation()
    {
        return $this->affiliation;
    }

    /**
     * @param string $affiliation
     * @return $this
     */
    public function setAffiliation($affiliation)
    {
        $this->affiliation = $affiliation;
        return $this;
    }

    /**
     * @param Exposition\Weave $weave
     */
    public function addWeave($weave)
    {
        if (!$this->weaves->contains($weave)) {
            $this->weaves->add($weave);
            $weave->setExposition($this);
        }
    }

    /**
     * @return Weave[]
     */
    public function getWeaves()
    {
        return $this->weaves->toArray();
    }

    /**
     * @param Weave[] $weaves
     */
    public function setWeaves(array $weaves)
    {
        $this->weaves = new ArrayCollection($weaves);
    }

    /**
     * @return Weave
     */
    public function getDefaultWeave()
    {
        if (isset($this->defaultWeave)) {
            return $this->defaultWeave;
        }
        if (!$this->weaves->isEmpty()) {
            return $this->weaves->first();
        }
        return null;
    }

    /**
     * @param Weave $defaultWeave
     * @return $this
     */
    public function setDefaultWeave(Weave $defaultWeave = null)
    {
        $this->defaultWeave = $defaultWeave;
        return $this;
    }

    /**
     * @return int
     */
    public function getDefaultWeaveId()
    {
        $weave = $this->getDefaultWeave();
        if (!isset($weave)) {
            return null;
        }

        return $weave->getId();
    }

    /**
     * @param int $defaultWeaveId
     */
    public function setDefaultWeaveId($defaultWeaveId)
    {
        $this->setDefaultWeave(WeaveRepository::getById($defaultWeaveId));
    }

    /**
     * @return string
     */
    public function getReferenceUrl()
    {
        return $this->referenceUrl;
    }

    /**
     * @param string $referenceUrl
     * @return $this
     */
    public function setReferenceUrl($referenceUrl)
    {
        $this->referenceUrl = $referenceUrl;
        return $this;
    }

    /**
     * @return string
     */
    public function getTocAlignment()
    {
        return $this->tocAlignment;
    }

    /**
     * @param string $alignment
     * @return $this
     */
    public function setTocAlignment($alignment)
    {
        $this->tocAlignment = $alignment;
        return $this;
    }

    /**
     * @return bool
     */
    public function getDisplayMenubarPermanently()
    {
        return $this->displayMenuBarPermanently;
    }

    /**
     * @param bool $display
     * @return $this
     */
    public function setDisplayMenubarPermanently($display)
    {
        $this->displayMenuBarPermanently = $display;
        return $this;
    }

    /**
     * @return string
     */
    public function getPublishRange()
    {
        return $this->publishRange;
    }

    /**
     * @param string $publishRange
     * @return $this
     */
    public function setPublishRange($publishRange)
    {
        $this->publishRange = $publishRange;
        return $this;
    }

    /**
     * @param string|bool $format
     * @return \DateTime|string
     */
    public function getPublishedDate($format = true)
    {
        return $this->formatDateTime($this->publishDate, $format);
    }

    /**
     * @param string $publishedDate
     * @return $this
     */
    public function setPublishedDate($publishedDate)
    {
        $this->publishDate = $this->castToDateTime($publishedDate, false);
        return $this;
    }

    /**
     * @param User $user
     * @return bool
     */
    public function hasEditRights(User $user)
    {
        if ($this->isEqual($user, $this->getAuthor())) {
            return true;
        }

        return $this->isAllowed($user, self::TYPE_WRITE);
    }

    /**
     * @return bool
     */
    public function isBlocked()
    {
        return $this->blocked;
    }

    /**
     * @param bool $blocked
     * @return $this
     */
    public function setBlocked($blocked)
    {
        $this->blocked = $blocked;
        return $this;
    }

    /**
     * @return int|null
     */
    public function getPortalId()
    {
        if (!isset($this->portal)) {
            return null;
        }
        return $this->portal->getId();
    }

    /**
     * @param int $portalId
     * @return $this
     */
    public function setPortalId($portalId)
    {
        $this->portal = $this->createReference(Portal::class, $portalId);
        return $this;
    }

    /**
     * @return Portal
     */
    public function getPortal()
    {
        return $this->portal;
    }

    /**
     * @param Portal $portal
     * @return $this
     */
    public function setPortal(Portal $portal)
    {
        $this->portal = $portal;
        return $this;
    }

    /**
     * @return Portal|null
     */
    public function getReviewPortal()
    {
        if ($this->isStatus(self::STATUS_REVIEW)) {
            return $this->portal;
        }

        if ($this->isStatus(self::STATUS_REVIEW_PUBLISHED)) {
            return $this->additionalPortals->last();
        }

        return null;
    }

    /**
     * @return bool
     */
    public function hasReviewPortal()
    {
        return $this->getReviewPortal() !== null;
    }

    /**
     * @return string
     */
    public function getPortalEdition()
    {
        if (!isset($this->portalIssue)) {
            return null;
        }
        return $this->portalIssue->getNumber();
    }

    /**
     * @param string $portalEdition
     * @return $this
     */
    public function setPortalEdition($portalEdition)
    {
        $issue = IssueRepository::findById($portalEdition);
        $this->portalIssue = $issue;
        $this->portalEdition = $issue->getNumber();
        return $this;
    }

    /**
     * @return Issue
     */
    public function getPortalIssue()
    {
        return $this->portalIssue;
    }

    /**
     * @param Issue $portalIssue
     */
    public function setPortalIssue(Issue $portalIssue)
    {
        $this->portalIssue = $portalIssue;
    }

    /**
     * @return string
     */
    public function getPortalIssueId()
    {
        if (!isset($this->portalIssue)) {
            return null;
        }
        return $this->portalIssue->getId();
    }

    /**
     * @param string $portalIssueId
     */
    public function setPortalIssueId($portalIssueId)
    {
        $this->portalIssue = IssueRepository::findById($portalIssueId);
    }

    /**
     * @return int[]
     */
    public function getAdditionalPortalIds()
    {
        return $this->createIdList($this->additionalPortals);
    }

    /**
     * @param array $portalIds
     * @return $this
     */
    public function setAdditionalPortalIds(array $portalIds)
    {
        $this->additionalPortals = new ArrayCollection($this->createReferenceList(Portal::class, $portalIds));
        return $this;
    }

    /**
     * @param int $portalId
     */
    public function addAdditionalPortalId($portalId)
    {
        $portal = $this->createReference(Portal::class, $portalId);
        if (isset($portal) && !$this->additionalPortals->contains($portal)) {
            $this->additionalPortals->add($portal);
        }
    }

    /**
     * @return Portal[]
     */
    public function getAdditionalPortals()
    {
        return $this->additionalPortals->toArray();
    }

    /**
     * @return int
     */
    public function getAdditionalPendingPortalId()
    {
        if (!isset($this->pendingAdditionalPortal)) {
            return null;
        }
        return $this->pendingAdditionalPortal->getId();
    }

    /**
     * @param int $portalId
     * @return $this
     */
    public function setAdditionalPendingPortalId($portalId)
    {
        $this->pendingAdditionalPortal = $this->createReference(Portal::class, $portalId);
        return $this;
    }

    /**
     * @return integer[]
     */
    public function getPublishedInPortalIds()
    {
        return $this->createIdList(new ArrayCollection($this->getPublishedInPortals()));
    }

    /**
     * @return Portal[]
     */
    public function getPublishedInPortals()
    {
        $portals = $this->additionalPortals->toArray();
        if (isset($this->portal)) {
            $portals[] = $this->portal;
        }
        return $portals;
    }

    /**
     * @return User[]
     */
    public function getReviewers()
    {
        return $this->reviewers->toArray();
    }

    /**
     * @return integer[]
     */
    public function getReviewerIds()
    {
        return $this->createIdList($this->reviewers);
    }

    /**
     * @param array $userList
     */
    public function setReviewerIds(array $userList)
    {
        $this->reviewers = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addReadAcl(AclGroup::USER($userList), AclContext::REVIEWER());
    }

    /**
     * @return User[]
     */
    public function getDoubleBlindReviewers()
    {
        return $this->doubleBlindReviewers->toArray();
    }

    /**
     * @return array
     */
    public function getDoubleBlindReviewerIds()
    {
        return $this->createIdList($this->doubleBlindReviewers);
    }

    /**
     * @param array $userList
     */
    public function setDoubleBlindReviewerIds(array $userList)
    {
        $this->doubleBlindReviewers = new ArrayCollection($this->createReferenceList(User::class, $userList));

        $this->addReadAcl(AclGroup::USER($userList), AclContext::DOUBLE_BLIND_REVIEWER());
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isReviewer(User $user)
    {
        return $this->reviewers->contains($user) || $this->isDoubleBlindReviewer($user);
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isDoubleBlindReviewer(User $user = null)
    {
        if (!isset($user)) {
            if (!Identity::exists()) {
                return false;
            }
            $user = Identity::getUser();
        }
        return $this->doubleBlindReviewers->contains($user);
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isApplicationReader(User $user)
    {
        return !empty(ApplicationRepository::findByUserForReading($user, $this));
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isApplicationReviewer(User $user)
    {
        return !empty(ApplicationRepository::findByUserForReviewing($user, $this));
    }

    /**
     * @param User $user
     * @return bool
     */
    public function isApplicationSupervisor(User $user)
    {
        return !empty(ApplicationRepository::findSupervisedByUser($user, $this));
    }

    /**
     * @param User $user
     * @return bool
     */
    public function hasApplicationRights(User $user)
    {
        $groups = $user->getGroups();
        foreach ($this->aclv2['role']['read'] as $role => $allowedGroups) {
            if (strpos($role, AclContext::APPLICATION('')) === false) {
                continue;
            }

            $allowedGroups = array_flip($allowedGroups);
            foreach ($groups as $group) {
                if (isset($allowedGroups[$group])) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @return Research[]
     */
    public function getShares()
    {
        return MongoRepository::initializeCollection($this->shares)->toArray();
    }

    /**
     * @param array $shares
     */
    public function setShares(array $shares)
    {
        $this->shares = new ArrayCollection($shares);
    }

    /**
     * @param Research $work
     * @return $this
     */
    public function addShare(Research $work)
    {
        if (!$this->shares->contains($work)) {
            $this->shares->add($work);
        }
        return $this;
    }

    /**
     * @param Research $work
     * @return bool
     */
    public function hasShare(Research $work)
    {
        return $this->shares->contains($work);
    }

    /**
     * @return TableOfContent
     */
    public function getTableOfContent()
    {
        if (!isset($this->tableOfContent)) {
            $this->setChapterList($this->chapterList);
        }

        return $this->tableOfContent;
    }

    /**
     * @param TableOfContent $tableOfContent
     */
    public function setTableOfContent(TableOfContent $tableOfContent)
    {
        $this->tableOfContent = $tableOfContent;
        $this->chapterList = null;
    }

    /**
     * @return array
     */
    public function getChapterList()
    {
        return $this->getTableOfContent()->toArray();
    }

    /**
     * @param array $chapterList
     * @return $this
     */
    public function setChapterList(array $chapterList)
    {
        $this->tableOfContent = new TableOfContent($chapterList);
        return $this;
    }

    /**
     * @return bool
     */
    public function hasComments()
    {
        return count(\CommentRepository::findByResearchId($this->getId(), 0)) > 0;
    }

    /**
     * @return mixed
     */
    public function getComments()
    {
        return \CommentRepository::findByResearchId($this->getId(), 0, 1e4);
    }

    /**
     * @return string
     */
    public function getReference()
    {
        return $this->reference;
    }

    /**
     * @param string $reference
     * @return $this
     */
    public function setReference($reference)
    {
        $this->reference = $reference;
        return $this;
    }

    /**
     * @return bool
     */
    public function isAbstractOnly()
    {
        return $this->abstractOnly;
    }

    /**
     * @param bool $abstractOnly
     * @return $this
     */
    public function setAbstractOnly($abstractOnly)
    {
        $this->abstractOnly = $abstractOnly;
        return $this;
    }

    /**
     * @return string
     */
    public function getReviewTag()
    {
        return $this->reviewTag;
    }

    /**
     * @param string $reviewTag
     * @return $this
     */
    public function setReviewTag($reviewTag)
    {
        $this->reviewTag = $reviewTag;
        return $this;
    }

    /**
     * @return string
     */
    public function getReviewComment()
    {
        return $this->reviewComment;
    }

    /**
     * @param string $reviewComment
     * @return $this
     */
    public function setReviewComment($reviewComment)
    {
        $this->reviewComment = $reviewComment;
        return $this;
    }

    /**
     * @return bool
     */
    public function trackChangesForLastModified()
    {
        if ($this->isPublished()) {
            return false;
        }
        if ($this->getLastModified() >= time() - 7200) {
            return false;
        }
        $user = Identity::getUser();
        return $this->isAuthor($user) || $this->isCoAuthor($user) || $this->isContributor($user);
    }

    /**
     * @return bool
     */
    public function hasExternalContent()
    {
        foreach ($this->getWeaves() as $weave) {
            foreach ($weave->getItems() as $item) {
                if ($item instanceof Weave\Tool\EmbedTool) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @return bool
     */
    public function hasItems()
    {
        foreach ($this->getWeaves() as $weave) {
            if ($weave->hasItems()) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return void
     */
    protected function resetSubmissionFields()
    {
        $this->setAdditionalPendingPortalId(null);
        $this->setReviewTag(null);
        $this->setReviewComment(null);
        $this->setReviewerIds([]);
        $this->setDoubleBlindReviewerIds([]);
        $this->setSupervisors([]);
        $this->setEditors([]);
    }
}
