<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research\Exposition\Weave\Generic;

use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\ImageSimpleMedia;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Model\Research\Exposition\Weave\Style\BackgroundStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Tool;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use Rc\Domain\Repository\Research\Exposition\Weave\ToolRepository;
use Rc\Domain\Repository\Research\Exposition\WeaveRepository;

trait BackgroundAwareTrait
{

    /**
     * @var BackgroundStyle
     * @ODM\EmbedOne(targetDocument="Rc\Domain\Model\Research\Exposition\Weave\Style\BackgroundStyle")
     */
    protected $background;

    /**
     * @return array
     */
    public static function getBackgroundImageStyles()
    {
        return [
            'repeat'    => 'css.bg.repeat',
            'no-repeat' => 'css.bg.norepeat',
        ];
    }

    /**
     * @return array
     */
    public static function getBackgroundImagePositions()
    {
        return [
            'left top'      => 'css.bg.left-top',
            'left center'   => 'css.bg.left-center',
            'left bottom'   => 'css.bg.left-bottom',
            'center top'    => 'css.bg.center-top',
            'center center' => 'css.bg.center-center',
            'center bottom' => 'css.bg.center-bottom',
            'right top'     => 'css.bg.right-top',
            'right center'  => 'css.bg.right-center',
            'right bottom'  => 'css.bg.right-bottom',
        ];
    }

    /**
     * @return array
     */
    public static function getBackgroundImageSizes()
    {
        return [
            'auto'    => 'css.bg.auto',
            'cover'   => 'css.bg.cover',
            'contain' => 'css.bg.contain',
        ];
    }

    /**
     * @return string
     */
    public function getBackgroundColor()
    {
        return $this->background->getColor();
    }

    /**
     * @param string $color
     */
    public function setBackgroundColor($color)
    {
        $this->background->setColor($color);
    }

    /**
     * @return string
     */
    public function getBackgroundImageStyle()
    {
        return $this->background->getImageStyle();
    }

    /**
     * @param string $imageStyle
     */
    public function setBackgroundImageStyle($imageStyle)
    {
        $this->background->setImageStyle($imageStyle);
    }

    /**
     * @return string
     */
    public function getBackgroundImagePosition()
    {
        return $this->background->getImagePosition();
    }

    /**
     * @param string $imagePosition
     */
    public function setBackgroundImagePosition($imagePosition)
    {
        $this->background->setImagePosition($imagePosition);
    }

    /**
     * @return string
     */
    public function getBackgroundImageSize()
    {
        return $this->background->getImageSize();
    }

    /**
     * @param string $imageSize
     */
    public function setBackgroundImageSize($imageSize)
    {
        $this->background->setImageSize($imageSize);
    }

    /**
     * @return ImageSimpleMedia|null
     */
    public function getBackgroundImage()
    {
        return $this->background->getImage();
    }

    /**
     * @param ImageSimpleMedia|null $image
     */
    public function setBackgroundImage(ImageSimpleMedia $image = null)
    {
        $this->background->setImage($image);
    }

    /**
     * @return bool
     */
    public function hasBackgroundImage()
    {
        try {
            return $this->background->hasImage() && $this->background->getImage()->getName();
        } catch (DocumentNotFoundException $exception) {
            trigger_error($exception->getMessage(), E_USER_WARNING);

            $this->background->setImage(null);
            if ($this instanceof GraphicalWeave) {
                WeaveRepository::persist($this);
            } elseif ($this instanceof Tool) {
                ToolRepository::persist($this);
            }
            return false;
        }
    }

    /**
     * @return int|null
     */
    public function getBackgroundImageFileId()
    {
        if (!$this->hasBackgroundImage()) {
            return null;
        }
        return $this->getBackgroundImage()->getId();
    }

    /**
     * @param int $fileId
     */
    public function setBackgroundImageFileId($fileId)
    {
        $image = !empty($fileId) ? SimpleMediaRepository::findById($fileId) : null;

        $this->background->setImage($image);
    }

    /**
     * @param int|null $width
     * @param int|null $height
     * @return string|null
     */
    public function getBackgroundImageUrl($width = null, $height = null)
    {
        if (!$this->hasBackgroundImage() || !$this->getBackgroundImage()->hasMedia()) {
            return null;
        }

        if (!isset($width) && !isset($height)) {
            /** @var ImageResource $master */
            $master = $this->getBackgroundImage()->getMedia()->getMaster();
            if ($this->getBackgroundImageStyle() == 'repeat') {
                $width = $master->getWidth();
                $height = $master->getHeight();
            } else {
                $width = min($this->getWidth(), $master->getWidth());
                $height = min($this->getHeight(), $master->getHeight());
            }
        }

        return $this->getBackgroundImage()->getMedia()->getBestFittingSplashScreen($width, $height)->getExternalUri();
    }

    /**
     * @return void
     */
    protected function removeExternalDependencyForBackgroundImage()
    {
        if (!$this->background->hasImage() || !$this->background->getImage()->hasWork()) {
            return;
        }

        $simpleMedia = clone $this->background->getImage();
        $simpleMedia->setResearch($this->getExposition());
        $simpleMedia->setSourceId($this->background->getImage()->getId());
        SimpleMediaRepository::persist($simpleMedia);

        $this->background->setImage($simpleMedia);
    }

    /**
     * @return Exposition
     */
    abstract public function getExposition();
}