<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Identity;
use JMS\Serializer\Annotation as ODS;
use Rc\Domain\Model\ModelInterface;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\BackgroundAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\BackgroundAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\BorderAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\BorderAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\MarginAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\MarginAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\PaddingAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\PaddingAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Style\BackgroundStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Style\BorderStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Style\PositionStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\ContextAwareInterface;
use Rc\Domain\Model\User;
use WeaveThumb;

/**
 * @ODM\Document(collection="Weave")
 */
class GraphicalWeave extends Weave implements BackgroundAwareInterface, BorderAwareInterface, MarginAwareInterface, PaddingAwareInterface
{
    use PaddingAwareTrait;
    use MarginAwareTrait;
    use BorderAwareTrait;
    use BackgroundAwareTrait;

    const DEFAULT_TITLE = 'default page';

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $iframe;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $hyperlinkColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $hyperlinkVisitedColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $hyperlinkHoverColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $footnoteColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $footnoteVisitedColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $footnoteHoverColor;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $rawCss;

    /**
     * @var array
     * @ODM\Field(type="hash")
     */
    protected $defaultAspect = ['x' => 0, 'y' => 0];

    /**
     * @var ArrayCollection|ToolInterface[]
     * @ODM\ReferenceMany(
     *     targetDocument="Rc\Domain\Model\Research\Exposition\Weave\Tool", mappedBy="weave", cascade={"all"},
     *     criteria={"deleted": false},
     *     prime={"mediaList"},
     *     storeAs="dbRef"
     * )
     * @ODS\Type(name="ArrayCollection<Rc\Serializer\Type\Tool>")
     */
    protected $toolList;

    /**
     * @var int[]
     * @ODM\Field(type="hash")
     */
    protected $toolCounter;

    /**
     * @var int
     * @ODS\Exclude()
     */
    private $_width;

    /**
     * @var int
     * @ODS\Exclude()
     */
    private $_height;

    public function __construct()
    {
        parent::__construct();
        $this->toolList = new ArrayCollection();
        $this->padding = new PositionStyle();
        $this->margin = new PositionStyle();
        $this->border = new BorderStyle();
        $this->background = new BackgroundStyle();
    }

    /**
     * @return string
     */
    public function getIframe()
    {
        return $this->iframe;
    }

    /**
     * @param string $iframe
     */
    public function setIframe($iframe)
    {
        $this->iframe = $iframe;
    }

    /**
     * @param int $width
     * @param int $height
     * @param bool $useDefaultThumb
     * @return WeaveThumb
     */
    public function getThumb($width = self::DEFAULT_THUMB_WIDTH, $height = self::DEFAULT_THUMB_HEIGHT, $useDefaultThumb = false)
    {
        $conf['maxwidth'] = $width;
        $conf['maxheight'] = $height;
        return new WeaveThumb($this->getItems(), $conf);
    }

    /**
     * @param User $user
     * @return ToolInterface[]
     */
    public function getItems(User $user = null)
    {
        $user = $user ?: Identity::getUserOrNull();

        return $this->toolList->filter(function (ToolInterface $tool) use ($user) {
            $show = $tool->isVisibleTo($user);

            if (!$tool instanceof ContextAwareInterface || !$tool->hasContext()) {
                return $show;
            }

            return $show && $tool->getContext()->isActive();
        })->toArray();
    }

    /**
     * @return array
     */
    public function getDefaultAspect()
    {
        $defaultAspect = $this->defaultAspect;
        $defaultAspect['x'] += $this->getPaddingLeft();
        $defaultAspect['y'] += $this->getPaddingTop();
        return $defaultAspect;
    }

    /**
     * @param int $x
     * @param int $y
     */
    public function setDefaultAspect($x, $y)
    {
        $x -= $this->getPaddingLeft();
        $y -= $this->getPaddingTop();
        $this->defaultAspect = ['x' => $x, 'y' => $y];
    }

    /**
     * @return int
     */
    public function getDefaultAspectX()
    {
        return $this->getDefaultAspect()['x'];
    }

    /**
     * @return int
     */
    public function getDefaultAspectY()
    {
        return $this->getDefaultAspect()['y'];
    }

    /**
     * @return bool
     */
    public function hasItems()
    {
        return count($this->getItems()) > 0;
    }

    /**
     * @return int
     */
    public function getWidth()
    {
        if (!isset($this->_width)) {
            foreach ($this->getItems() as $item) {
                $this->_width = max($this->_width, $item->getLeft() + $item->getOuterWidth());
            }
        }
        return $this->_width;
    }

    /**
     * @return int
     */
    public function getHeight()
    {
        if (!isset($this->_height)) {
            foreach ($this->getItems() as $item) {
                $this->_height = max($this->_height, $item->getTop() + $item->getOuterHeight());
            }
        }
        return $this->_height;
    }

    /**
     * @return int
     */
    public function getMaxZIndex()
    {
        $max = 0;
        foreach ($this->getItems() as $item) {
            $max = max($max, $item->getZIndex());
        }
        return $max;
    }

    /**
     * @return string
     */
    public function getHyperlinkColor()
    {
        return $this->hyperlinkColor;
    }

    /**
     * @param string $hyperlinkColor
     */
    public function setHyperlinkColor($hyperlinkColor)
    {
        $this->hyperlinkColor = $hyperlinkColor;
    }

    /**
     * @return string
     */
    public function getHyperlinkVisitedColor()
    {
        return $this->hyperlinkVisitedColor;
    }

    /**
     * @param string $hyperlinkVisitedColor
     */
    public function setHyperlinkVisitedColor($hyperlinkVisitedColor)
    {
        $this->hyperlinkVisitedColor = $hyperlinkVisitedColor;
    }

    /**
     * @return string
     */
    public function getHyperlinkHoverColor()
    {
        return $this->hyperlinkHoverColor;
    }

    /**
     * @param string $hyperlinkHoverColor
     */
    public function setHyperlinkHoverColor($hyperlinkHoverColor)
    {
        $this->hyperlinkHoverColor = $hyperlinkHoverColor;
    }

    /**
     * @return string
     */
    public function getFootnoteColor()
    {
        return $this->footnoteColor;
    }

    /**
     * @param string $footnoteColor
     */
    public function setFootnoteColor($footnoteColor)
    {
        $this->footnoteColor = $footnoteColor;
    }

    /**
     * @return string
     */
    public function getFootnoteVisitedColor()
    {
        return $this->footnoteVisitedColor;
    }

    /**
     * @param string $footnoteVisitedColor
     */
    public function setFootnoteVisitedColor($footnoteVisitedColor)
    {
        $this->footnoteVisitedColor = $footnoteVisitedColor;
    }

    /**
     * @return string
     */
    public function getFootnoteHoverColor()
    {
        return $this->footnoteHoverColor;
    }

    /**
     * @param string $footnoteHoverColor
     */
    public function setFootnoteHoverColor($footnoteHoverColor)
    {
        $this->footnoteHoverColor = $footnoteHoverColor;
    }

    /**
     * @return string
     */
    public function getRawCss()
    {
        return $this->rawCss;
    }

    /**
     * @param string $rawCss
     */
    public function setRawCss($rawCss)
    {
        $this->rawCss = $rawCss;
    }

    /**
     * @param ModelInterface $model
     * @return int
     */
    public function incrementItemCounter(ModelInterface $model)
    {
        $pEntry = &$this->toolCounter[get_class($model)];
        return ++$pEntry;
    }

    /**
     * @return void
     */
    public function removeExternalDependencies()
    {
        $this->removeExternalDependencyForBackgroundImage();

        foreach ($this->toolList as $tool) {
            $tool->removeExternalDependencies();
        }
    }
}