<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use InsufficientRightsException;
use Rc\Domain\Model\AbstractDilpsModel;
use Rc\Domain\Model\Core\Revision\RevisionAwareTrait;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Popover;
use Rc\Domain\Model\Research\Exposition\SimpleMedia\ImageSimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\AudioAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\ImageAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\StyleAwareInterface;
use Rc\Domain\Model\Research\Exposition\Weave\Generic\StyleAwareTrait;
use Rc\Domain\Model\Research\Exposition\Weave\Style\BackgroundStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Style\BorderStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Style\PositionStyle;
use Rc\Domain\Model\Research\Exposition\Weave\Style\ShadowStyle;
use Rc\Domain\Model\Research\MediaContainer;
use Rc\Domain\Model\Research\MediaContainerInterface;
use Rc\Domain\Model\Research\SimpleMediaAwareTrait;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;

/**
 * @ODM\Document
 * @ODM\InheritanceType("SINGLE_COLLECTION")
 * @ODM\DiscriminatorField("_class")
 * @ODM\Index(keys={"weave.$id"="asc"}, partialFilterExpression={"deleted"={"$eq"=false}})
 */
abstract class Tool extends AbstractDilpsModel implements ToolInterface
{
    use StyleAwareTrait;
    use AudioAwareTrait;
    use ImageAwareTrait;
    use SimpleMediaAwareTrait;
    use RevisionAwareTrait;

    /**
     * @var SimpleMediaInterface[]|ArrayCollection
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition\SimpleMedia", strategy="set", cascade={"persist"}, storeAs="dbRef")
     */
    protected $mediaList;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $left;

    /**
     * @var int
     * @ODM\Field(type="integer")
     */
    protected $top;

    /**
     * @var float
     * @ODM\Field(type="float")
     */
    protected $rotate = 0;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $size;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $position;

    /**
     * @var GraphicalWeave
     * @ODM\ReferenceOne(cascade={"persist"}, storeAs="dbRef")
     */
    protected $weave;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $shapeContent;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $reviewing;

    /**
     * @var bool
     * @ODM\Field(type="boolean")
     */
    protected $locked;

    /**
     * @var Popover
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Exposition\Popover", cascade={"persist"}, storeAs="dbRef")
     */
    protected $popover;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $hoverPopup;

    /**
     * @var boolean
     * @ODM\Field(type="boolean")
     */
    protected $showCopyright;

    /**
     * @var string
     * @ODM\Field(type="string")
     */
    protected $hoverPlainText;

    public function __construct()
    {
        parent::__construct();
        $this->background = new BackgroundStyle();
        $this->margin = new PositionStyle();
        $this->padding = new PositionStyle();
        $this->border = new BorderStyle();
        $this->shadow = new ShadowStyle();
        $this->mediaList = new ArrayCollection();
        $this->audioList = new ArrayCollection();
        $this->imageList = new ArrayCollection();
    }

    /**
     * @return array
     */
    public static function getSizes()
    {
        return [
            self::SIZE_CONTAIN => 'css.size.contain',
            self::SIZE_COVER   => 'css.size.cover',
            self::SIZE_FIT     => 'css.size.fit',
            self::SIZE_AUTO    => 'css.size.auto',
        ];
    }

    /**
     * @return array
     */
    public static function getPositions()
    {
        return [
            'left top'                   => 'css.bg.left-top',
            'left center'                => 'css.bg.left-center',
            'left bottom'                => 'css.bg.left-bottom',
            'center top'                 => 'css.bg.center-top',
            self::POSITION_CENTER_CENTER => 'css.bg.center-center',
            'center bottom'              => 'css.bg.center-bottom',
            'right top'                  => 'css.bg.right-top',
            'right center'               => 'css.bg.right-center',
            'right bottom'               => 'css.bg.right-bottom',
        ];
    }

    /**
     * @return array
     */
    public static function getHoverPopupOptions()
    {
        return [
            ''          => 'hover-popup.none',
            'copyright' => 'hover-popup.copyright',
            'plaintext' => 'hover-popup.plaintext',
            'both'      => 'hover-popup.both',
        ];
    }

    /**
     * @return float
     */
    public function getRotate()
    {
        return round($this->rotate, 2);
    }

    /**
     * @param float $degree
     * @return $this
     */
    public function setRotate($degree)
    {
        $degree = round($degree, 2);
        if ($degree != $this->getRotate()) {
            $this->addRevisionComment('move');
        }

        $this->rotate = $degree;
        return $this;
    }

    /**
     * @return float
     */
    public function getScaleOfRotatedObject()
    {
        return min($this->getWidth() / $this->getWidthOfRotatedObject(), $this->getHeight() / $this->getHeightOfRotatedObject());
    }

    /**
     * @return float
     */
    public function getWidthOfRotatedObject()
    {
        return $this->getWidth() * abs(cos($this->getRadius())) + $this->getHeight() * abs(sin($this->getRadius()));
    }

    /**
     * @return float
     */
    public function getRadius()
    {
        return $this->getRotate() * (pi() / 180);
    }

    /**
     * @return float
     */
    public function getHeightOfRotatedObject()
    {
        return $this->getHeight() * abs(cos($this->getRadius())) + $this->getWidth() * abs(sin($this->getRadius()));
    }

    /**
     * @return string
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * @param string $size
     * @return $this
     */
    public function setSize($size)
    {
        if ($size != $this->getSize()) {
            $this->addRevisionComment('resize');
        }

        $this->size = $size;
        return $this;
    }

    /**
     * @return string
     */
    public function getShapeContent()
    {
        return $this->shapeContent;
    }

    /**
     * @param string $text
     * @return $this
     */
    public function setShapeContent($text)
    {
        $this->setMediaType(self::MEDIA_TYPE_SHAPE);
        $this->shapeContent = $text;
        return $this;
    }

    /**
     * @return bool
     */
    public function hasShapeContent()
    {
        return isset($this->shapeContent);
    }

    /**
     * @return bool
     */
    public function isReviewing()
    {
        return $this->reviewing;
    }

    /**
     * @deprecated
     * @return bool
     */
    public function getReviewing()
    {
        return $this->isReviewing();
    }

    /**
     * @param bool $reviewing
     * @return $this
     */
    public function setReviewing($reviewing)
    {
        $this->reviewing = $reviewing;
        return $this;
    }

    /**
     * @return bool
     */
    public function isLocked()
    {
        return $this->locked;
    }

    /**
     * @param bool $locked
     * @return $this
     */
    public function lock($locked)
    {
        $this->locked = $locked;
        return $this;
    }

    /**
     * @return Popover
     */
    public function getPopover()
    {
        return $this->popover;
    }

    /**
     * @param Popover $popover
     * @return $this
     */
    public function setPopover(Popover $popover = null)
    {
        $this->popover = $popover;
        return $this;
    }

    /**
     * @return bool
     */
    public function hasPopover()
    {
        return isset($this->popover);
    }

    /**
     * @deprecated
     * @return Exposition
     */
    public function getResearch()
    {
        return $this->getExposition();
    }

    /**
     * @return bool
     */
    public function isResourceLimitReached()
    {
        return $this->getResourceLimit() > 0 && $this->getResourceLimit() <= $this->getResourceCount();
    }

    public abstract function getSupportedMediaTypes();

    /**
     * @return array
     */
    public function getSupportedSimpleMedias()
    {
        $simpleMedias = [];
        foreach ($this->getSupportedMediaTypes() as $supportedMediaType) {
            $simpleMedias[] = 'Rc\\Domain\\Model\\Research\\Exposition\\SimpleMedia\\' . ucfirst($supportedMediaType) . 'SimpleMedia';
        }
        return $simpleMedias;
    }

    /**
     * @return string
     */
    public function getHoverPopup()
    {
        return $this->hoverPopup;
    }

    /**
     * @param string $hoverPopup
     * @return $this
     */
    public function setHoverPopup($hoverPopup)
    {
        $this->hoverPopup = $hoverPopup;
        return $this;
    }

    /**
     * @deprecated
     * @return bool
     */
    public function getShowCopyright()
    {
        return $this->showCopyright;
    }

    /**
     * @return bool
     */
    public function isShowCopyright()
    {
        return $this->showCopyright;
    }

    /**
     * @param bool $showCopyright
     * @return $this
     */
    public function setShowCopyright($showCopyright)
    {
        $this->showCopyright = $showCopyright;
        return $this;
    }

    /**
     * @return string
     */
    public function getHoverPlainText()
    {
        return $this->hoverPlainText;
    }

    /**
     * @param string $plainText
     * @return $this
     */
    public function setHoverPlainText($plainText)
    {
        $this->hoverPlainText = $plainText;
        return $this;
    }

    /**
     * @return void
     */
    protected function removeExternalDependencyForMediaList()
    {
        foreach ($this->mediaList as $key => $simpleMedia) {
            if (!$simpleMedia->hasWork()) {
                $this->removeExternalDependencyForAudio($simpleMedia, $simpleMedia);
                continue;
            }

            $newSimpleMedia = clone $simpleMedia;
            $newSimpleMedia->setResearch($this->weave->getExposition());
            $newSimpleMedia->setSourceId($simpleMedia->getId());
            $this->mediaList->offsetSet($key, $newSimpleMedia);

            $this->removeExternalDependencyForAudio($simpleMedia, $newSimpleMedia);
        }
    }

    /**
     * @param int $size
     * @return int
     */
    protected function _getSizeOffset($size)
    {
        $borderWidth = 2 * $this->getBorderWidth() * ($this->getBorderStyle() != StyleAwareInterface::CSS_BORDER_NONE);
        if ($size == self::SIZE_HORIZONTAL) {
            return $borderWidth + $this->getPaddingLeft() + $this->getPaddingRight();
        }
        if ($size == self::SIZE_VERTICAL) {
            return $borderWidth + $this->getPaddingTop() + $this->getPaddingBottom();
        }
        return 0;
    }

    /**
     * @return Exposition
     */
    public function getExposition()
    {
        return $this->getWeave()->getExposition();
    }

    /**
     * @return int
     */
    public function getResourceLimit()
    {
        return static::MEDIA_LIMIT;
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isVisibleTo(User $user = null)
    {
        return true;
    }

    /**
     * @param User|null $user
     * @return bool
     */
    public function isEditableBy(User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        return true;
    }

    /**
     * @inheritDoc
     */
    public function setBackgroundColor($color)
    {
        if ($color != $this->getBackgroundColor()) {
            $this->addRevisionComment('style');
        }

        $this->background->setColor($color);
    }

    /**
     * @inheritDoc
     */
    public function setBackgroundImageStyle($imageStyle)
    {
        if ($imageStyle != $this->getBackgroundImageStyle()) {
            $this->addRevisionComment('style');
        }

        $this->background->setImageStyle($imageStyle);
    }

    /**
     * @inheritDoc
     */
    public function setBackgroundImagePosition($imagePosition)
    {
        if ($imagePosition != $this->getBackgroundImagePosition()) {
            $this->addRevisionComment('style');
        }

        $this->background->setImagePosition($imagePosition);
    }

    /**
     * @inheritDoc
     */
    public function setBackgroundImageSize($imageSize)
    {
        if ($imageSize != $this->getBackgroundImageSize()) {
            $this->addRevisionComment('style');
        }

        $this->background->setImageSize($imageSize);
    }

    /**
     * @inheritDoc
     */
    public function setBackgroundImage(ImageSimpleMedia $image = null)
    {
        if ($image !== $this->getBackgroundImage()) {
            $this->addRevisionComment('style');
        }

        $this->background->setImage($image);
    }

    /**
     * @inheritDoc
     */
    public function setBorderStyle($style)
    {
        if ($style != $this->getBorderStyle()) {
            $this->addRevisionComment('style');
        }

        $this->border->setStyle($style);
    }

    /**
     * @inheritDoc
     */
    public function setBorderColor($color)
    {
        if ($color != $this->getBorderColor()) {
            $this->addRevisionComment('style');
        }

        $this->border->setColor($color);
    }

    /**
     * @inheritDoc
     */
    public function setBorderWidth($width)
    {
        if ($width != $this->getBorderWidth()) {
            $this->addRevisionComment('style');
        }

        $this->border->setWidth($width);
    }

    /**
     * @inheritDoc
     */
    public function setBorderRadius($radius)
    {
        if ($radius != $this->getBorderRadius()) {
            $this->addRevisionComment('style');
        }

        $this->border->setRadius($radius);
    }

    /**
     * @inheritDoc
     */
    public function setMarginTop($top)
    {
        if ($top != $this->getMarginTop()) {
            $this->addRevisionComment('style');
        }

        $this->margin->setTop($top);
    }

    /**
     * @inheritDoc
     */
    public function setMarginRight($right)
    {
        if ($right != $this->getMarginRight()) {
            $this->addRevisionComment('style');
        }

        $this->margin->setRight($right);
    }

    /**
     * @inheritDoc
     */
    public function setMarginBottom($bottom)
    {
        if ($bottom != $this->getMarginBottom()) {
            $this->addRevisionComment('style');
        }

        $this->margin->setBottom($bottom);
    }

    /**
     * @inheritDoc
     */
    public function setMarginLeft($left)
    {
        if ($left != $this->getMarginLeft()) {
            $this->addRevisionComment('style');
        }

        $this->margin->setLeft($left);
    }

    /**
     * @inheritDoc
     */
    public function setPaddingTop($top)
    {
        if ($top != $this->getPaddingTop()) {
            $this->addRevisionComment('style');
        }

        $this->padding->setTop($top);
    }

    /**
     * @inheritDoc
     */
    public function setPaddingRight($right)
    {
        if ($right != $this->getPaddingRight()) {
            $this->addRevisionComment('style');
        }

        $this->padding->setRight($right);
    }

    /**
     * @inheritDoc
     */
    public function setPaddingBottom($bottom)
    {
        if ($bottom != $this->getPaddingBottom()) {
            $this->addRevisionComment('style');
        }

        $this->padding->setBottom($bottom);
    }

    /**
     * @inheritDoc
     */
    public function setPaddingLeft($left)
    {
        if ($left != $this->getPaddingLeft()) {
            $this->addRevisionComment('style');
        }

        $this->padding->setLeft($left);
    }

    /**
     * @inheritDoc
     */
    public function setShadowMarginTop($marginTop)
    {
        if ($marginTop != $this->getShadowMarginTop()) {
            $this->addRevisionComment('style');
        }

        $this->shadow->setMarginTop($marginTop);
    }

    /**
     * @inheritDoc
     */
    public function setShadowMarginLeft($marginLeft)
    {
        if ($marginLeft != $this->getShadowMarginLeft()) {
            $this->addRevisionComment('style');
        }

        $this->shadow->setMarginLeft($marginLeft);
    }

    /**
     * @inheritDoc
     */
    public function setShadowBlur($blur)
    {
        if ($blur != $this->getShadowBlur()) {
            $this->addRevisionComment('style');
        }

        $this->shadow->setBlur($blur);
    }

    /**
     * @inheritDoc
     */
    public function setShadowUnschaerfe($shadowBlur)
    {
        $this->setShadowBlur($shadowBlur);
    }

    /**
     * @inheritDoc
     */
    public function setShadowColor($color)
    {
        if ($color != $this->getShadowColor()) {
            $this->addRevisionComment('style');
        }

        $this->shadow->setColor($color);
    }

    /**
     * @inheritDoc
     */
    public function setOpacity($opacity)
    {
        if ($opacity != $this->getOpacity()) {
            $this->addRevisionComment('style');
        }

        $this->opacity = $opacity;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setZIndex($zIndex)
    {
        if ($zIndex != $this->getOpacity()) {
            $this->addRevisionComment('style');
        }

        $this->zIndex = $zIndex;
        return $this;
    }

    public function removeExternalDependencies()
    {
        $this->removeExternalDependencyForMediaList();
        $this->removeExternalDependencyForBackgroundImage();
    }

    /**
     * @inheritDoc
     */
    public function getAclEntity()
    {
        return $this->getExposition();
    }

    /**
     * @inheritdoc
     */
    public function setHeight($height)
    {
        if ($height != $this->getHeight()) {
            $this->addRevisionComment('resize');
        }

        $this->height = $height;
        return $this;
    }

    /**
     * @inheritdoc
     */
    public function setWidth($width)
    {
        if ($width != $this->getWidth()) {
            $this->addRevisionComment('resize');
        }

        $this->width = $width;
        return $this;
    }

    /**
     * @return void
     */
    public function clearMediaList()
    {
        $this->mediaList->clear();
        $this->audioList->clear();
        $this->imageList->clear();
    }

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @deprecated
     */
    public function addResource(SimpleMediaInterface $simpleMedia)
    {
        if ($this->mediaList->contains($simpleMedia)) {
            return;
        }

        if ($this->isResourceLimitReached()) {
            throw new \LogicException(
                'Tried to add resource #' . $simpleMedia->getId() . ' but the resource limit of ' .
                $this->getResourceLimit() . ' is already reached!',
                1472904240
            );
        }

        $this->addRevisionComment('resource');

        $this->mediaList->add($simpleMedia);

        if ($simpleMedia->hasWork()) {
            $this->getExposition()->addShare($simpleMedia->getResearch());
        }
    }

    /**
     * @param SimpleMediaInterface $resource
     * @deprecated
     */
    public function removeResource(SimpleMediaInterface $resource)
    {
        $this->addRevisionComment('resource');

        $this->mediaList->removeElement($resource);
        $this->audioList->remove($resource->getId());
        $this->imageList->remove($resource->getId());
    }

    /**
     * @inheritdoc
     */
    public function getMediaContainer()
    {
        $media = $this->getMedia();
        if (!isset($media)) {
            return new MediaContainer();
        }
        return $this->getSimpleMediaContainer($media);
    }

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @return MediaContainerInterface
     */
    public function getSimpleMediaContainer(SimpleMediaInterface $simpleMedia)
    {
        if (!$this->mediaList->contains($simpleMedia)) {
            throw new \InvalidArgumentException('The given simple media ' . $simpleMedia->getId() . ' is not known to tool ' . $this->getId() . '!');
        }
        $mediaContainer = $simpleMedia->getMediaContainer();
        if (!$this->hasImage($simpleMedia)) {
            return $mediaContainer;
        }
        $customPreview = $this->getImage($simpleMedia);
        if ($customPreview->hasThumb()) {
            $mediaContainer->setThumb($customPreview->getThumb());
        } elseif ($customPreview->hasMedia()) {
            $mediaContainer->setThumb($customPreview->getMedia());
        }
        return $mediaContainer;
    }

    /**
     * @return int
     */
    public function getLeft()
    {
        if (!$this->hasWeave()) {
            return $this->left;
        }
        return $this->left + $this->getWeave()->getPaddingLeft();
    }

    /**
     * @param int $left
     * @return $this
     */
    public function setLeft($left)
    {
        if ($left != $this->getLeft()) {
            $this->addRevisionComment('move');
        }

        $this->left = $left;
        if ($this->hasWeave()) {
            $this->left -= $this->weave->getPaddingLeft();
        }
        return $this;
    }

    /**
     * @return int
     */
    public function getTop()
    {
        if (!$this->hasWeave()) {
            return $this->top;
        }
        return $this->top + $this->getWeave()->getPaddingTop();
    }

    /**
     * @param int $top
     * @return $this
     */
    public function setTop($top)
    {
        if ($top != $this->getTop()) {
            $this->addRevisionComment('move');
        }

        $this->top = $top;
        if ($this->hasWeave()) {
            $this->top -= $this->weave->getPaddingTop();
        }
        return $this;
    }

    /**
     * @return GraphicalWeave
     */
    public function getWeave()
    {
        return $this->weave;
    }

    /**
     * @param GraphicalWeave $weave
     * @return $this
     */
    public function setWeave(GraphicalWeave $weave)
    {
        $this->weave = $weave;
        return $this;
    }

    /**
     * @return bool
     */
    public function hasWeave()
    {
        return isset($this->weave);
    }

    /**
     * @return int
     */
    public function getOuterWidth()
    {
        return $this->getWidth() + $this->_getSizeOffset(self::SIZE_HORIZONTAL);
    }

    /**
     * @return int
     */
    public function getOuterHeight()
    {
        return $this->getHeight() + $this->_getSizeOffset(self::SIZE_VERTICAL);
    }

    /**
     * @return string
     */
    public function getPosition()
    {
        return $this->position;
    }

    /**
     * @param string $position
     * @return $this
     */
    public function setPosition($position)
    {
        $this->position = $position;
        return $this;
    }

    /**
     * @return void
     */
    public function removeMissingResources()
    {
        foreach ($this->mediaList->toArray() as $key => $value) {
            try {
                $value->getId();
            } catch (DocumentNotFoundException $exception) {
                $this->mediaList->remove($key);
            }
        }

        foreach ($this->audioList as $key => $id) {
            try {
                SimpleMediaRepository::getById($id);
            } catch (InsufficientRightsException $exception) {
                $this->audioList->remove($key);
            }
        }

        foreach ($this->imageList as $key => $id) {
            try {
                SimpleMediaRepository::getById($id);
            } catch (InsufficientRightsException $exception) {
                $this->imageList->remove($key);
            }
        }
    }

    /**
     * @inheritdoc
     */
    public function setTextContent($text)
    {
        if ($text != $this->getTextContent()) {
            $this->addRevisionComment('content');
        }

        parent::setTextContent($text);
    }

    /**
     * @return string
     */
    public function getName()
    {
        $parts = explode('\\', get_called_class());
        return strtolower(substr(end($parts), 0, -4));
    }
}