<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Comment;

use Doctrine\ODM\MongoDB\Mapping\Annotations as ODM;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Exposition\Weave\Tool\Context\Context;
use Rc\Domain\Model\Research\Exposition\Weave\ToolInterface;
use Rc\Domain\Model\User;

/**
 * @ODM\EmbeddedDocument
 */
class ApplicationContext extends Context
{
    const ROLE_ADMIN = 'admin';
    const ROLE_COLLABORATEUR = 'collaborateur';
    const ROLE_OWNER = 'owner';
    const ROLE_REVIEWER = 'reviewer';
    const ROLE_ADVISOR = 'advisor';
    const ROLE_READER = 'reader';

    /**
     * @var Application
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Application", storeAs="dbRef")
     */
    protected $application;

    /**
     * @param Application $application
     * @param array $roles
     */
    public function __construct(Application $application, array $roles)
    {
        parent::__construct($roles);

        $this->application = $application;
    }

    /**
     * @param Application $application
     * @param ToolInterface $tool
     * @return static
     */
    public static function from(Application $application, ToolInterface $tool)
    {
        $user = $tool->getOwner();
        if (!isset($user)) {
            throw new \InvalidArgumentException('Cannot create context without a valid owner for ' . $tool . '!');
        }

        $exposition = $tool->getExposition();
        if (!isset($exposition)) {
            throw new \InvalidArgumentException('Cannot create context without a valid exposition for ' . $tool . '!');
        }

        $roles = [];
        if ($user->isAdmin()) {
            $roles[] = self::ROLE_ADMIN;
        }

        if ($application->isCollaborateur($user)) {
            $roles[] = self::ROLE_COLLABORATEUR;
        }

        if ($application->isOwner($user)) {
            $roles[] = self::ROLE_OWNER;
        }

        if ($application->isReader($user)) {
            $roles[] = self::ROLE_READER;
        }

        if ($application->isReviewer($user)) {
            $roles[] = self::ROLE_REVIEWER;
        }

        if ($application->isSupervisor($user)) {
            $roles[] = self::ROLE_ADVISOR;
        }

        return new static($application, $roles);
    }

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    public function isVisibleTo(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        // notes are not visible when application is published
        if ($this->application->isPublished()) {
            return false;
        }

        // a privileged user sees all comments
        if ($user->isAdmin()) {
            return true;
        }

        // a reviewer sees only his comments
        if ($this->application->isReviewer($user)) {
            return $tool->isOwner($user);
        }

        // a reviewer sees only his comments
        if ($this->application->isReader($user)) {
            return $tool->isOwner($user);
        }

        // special user sees non reviewer comments
        if ($this->application->isOwner($user) ||
            $this->application->isCollaborateur($user) ||
            $this->application->isSupervisor($user)
        ) {
            return !$this->hasReviewerRole() && !$this->hasReaderRole();
        }

        return false;
    }

    /**
     * @param ToolInterface $tool
     * @param User $user
     * @return bool
     */
    public function isEditableBy(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        if ($user->isAdmin() || $tool->isOwner($user)) {
            return true;
        }

        $owner = $tool->getOwner();
        if (!isset($owner)) {
            return false;
        }

        if ($this->application->isOwner($owner) || $this->application->isCollaborateur($owner)) {
            return true;
        }

        return false;
    }

    /**
     * @param ToolInterface $tool
     * @param User|null $user
     * @return bool
     */
    public function isMetaEditableBy(ToolInterface $tool, User $user = null)
    {
        if (!isset($user)) {
            return false;
        }

        return $tool->isOwner($user);
    }

    /**
     * @return bool
     */
    public function isActive()
    {
        $context = self::getCurrentContext();

        if (!isset($context['application'])) {
            return false;
        }

        return $context['application'] === $this->application;
    }

    /**
     * @return bool
     */
    protected function hasReviewerRole()
    {
        return in_array(self::ROLE_REVIEWER, $this->roles);
    }

    /**
     * @return bool
     */
    protected function hasReaderRole()
    {
        return in_array(self::ROLE_READER, $this->roles);
    }
}