<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Model\Research;

use Doctrine\Common\Collections\ArrayCollection;
use Rc\Domain\Model\Media\MediaAwareTrait;
use Rc\Domain\Model\Media\MediaType;
use Rc\Domain\Model\Media\Resource\ImageResource;
use Rc\Domain\Model\MediaInterface;

class MediaContainer implements MediaContainerInterface
{
    use MediaAwareTrait;

    /**
     * @var int
     */
    protected $id;

    /**
     * @var string
     */
    protected $name;

    /**
     * @var string
     */
    protected $contentType;

    /**
     * @var MediaInterface
     */
    protected $thumb;

    /**
     * @var MediaInterface[]|ArrayCollection
     */
    protected $mediaList;

    /**
     * @param MediaInterface|MediaInterface[]|null $mediaList
     * @param MediaInterface|null $thumb
     * @param string $name
     * @param int $id
     */
    public function __construct($mediaList = null, MediaInterface $thumb = null, $name = null, $id = null)
    {
        $this->thumb = $thumb;
        $this->name = $name;
        $this->id = $id;

        if (!isset($mediaList)) {
            $this->mediaList = new ArrayCollection();
        } elseif ($mediaList instanceof MediaInterface) {
            $this->mediaList = new ArrayCollection([$mediaList]);
        } elseif (is_array($mediaList)) {
            $this->mediaList = new ArrayCollection($mediaList);
        } else {
            throw new \InvalidArgumentException(
                'Expected any count of Rc\Domain\Model\MediaInterface got "' . gettype($mediaList) . '"!',
                1472366608
            );
        }
    }

    /**
     * @return MediaContainerInterface
     */
    public function getMediaContainer()
    {
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function hasThumb()
    {
        if (isset($this->thumb) && $this->thumb->hasMaster()) {
            return true;
        }
        if ($this->hasMedia() && $this->getMedia()->hasSplashScreen()) {
            return true;
        }
        return false;
    }

    /**
     * @return ImageResource|null
     */
    public function getThumb()
    {
        if (isset($this->thumb) && $this->thumb->hasMaster()) {
            /** @noinspection PhpIncompatibleReturnTypeInspection */
            return $this->thumb->getMaster();
        }

        if ($this->hasMedia() && $this->getMedia()->hasSplashScreen()) {
            return $this->getMedia()->getSplashScreen();
        }

        return null;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = (int)$id;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = (string)$name;
    }

    /**
     * @return string
     */
    public function getContentType()
    {
        return $this->contentType ?: $this->getMedia()->getContentType();
    }

    /**
     * @param string $contentType
     */
    public function setContentType($contentType)
    {
        $this->contentType = (string)$contentType;
    }

    /**
     * @return bool
     */
    public function isImage()
    {
        if (isset($this->contentType)) {
            return $this->contentType == MediaType::IMAGE;
        } else {
            return $this->getMedia()->isImage();
        }
    }

    /**
     * @return bool
     */
    public function isVideo()
    {
        if (isset($this->contentType)) {
            return $this->contentType == MediaType::VIDEO;
        } else {
            return $this->getMedia()->isVideo();
        }
    }

    /**
     * @return bool
     */
    public function isAudio()
    {
        if (isset($this->contentType)) {
            return $this->contentType == MediaType::AUDIO;
        } else {
            return $this->getMedia()->isAudio();
        }
    }

    /**
     * @return bool
     */
    public function isPdf()
    {
        if (isset($this->contentType)) {
            return $this->contentType == MediaType::PDF;
        } else {
            return $this->getMedia()->isPdf();
        }
    }

    /**
     * @return bool
     */
    public function isArchive()
    {
        if (isset($this->contentType)) {
            return $this->contentType == MediaType::ARCHIVE;
        } else {
            return false;
        }
    }

    /**
     * @return string
     */
    public function getMimeType()
    {
        return $this->getMedia()->getMaster()->getMimeType();
    }

    /**
     * @return bool
     */
    public function isEmpty()
    {
        return !isset($this->thumb) && $this->mediaList->isEmpty();
    }
}