<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */
namespace Rc\Domain\Model\Research;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ODM\MongoDB\DocumentNotFoundException;
use Rc\Domain\Model\MediaInterface;
use Rc\Domain\Model\Research\Exposition\SimpleMedia;
use Rc\Domain\Model\Research\Exposition\SimpleMediaInterface;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;

/**
 * @mixin SimpleMedia
 */
trait SimpleMediaAwareTrait
{
    /**
     * @var SimpleMediaInterface
     * @ODM\ReferenceOne(targetDocument="Rc\Domain\Model\Research\Exposition\SimpleMedia", storeAs="dbRef")
     */
    protected $thumb;

    /**
     * @var SimpleMediaInterface[]|ArrayCollection
     * @ODM\ReferenceMany(targetDocument="Rc\Domain\Model\Research\Exposition\SimpleMedia", cascade={"persist"}, storeAs="dbRef")
     */
    protected $mediaList;

    /**
     * @return boolean
     */
    public function hasThumb()
    {
        return isset($this->thumb);
    }

    /**
     * @return SimpleMediaInterface|null
     */
    public function getThumb()
    {
        return $this->thumb;
    }

    /**
     * @param SimpleMediaInterface|null $thumb
     */
    public function setThumb(SimpleMediaInterface $thumb = null)
    {
        $this->thumb = $thumb;
    }

    /**
     * @return boolean
     */
    public function hasMedia()
    {
        return !$this->mediaList->isEmpty();
    }

    /**
     * @return SimpleMediaInterface|null
     */
    public function getMedia()
    {
        try {
            $media = $this->mediaList->first() ?: null;
            if ($media instanceof SimpleMediaInterface) {
                $media->getName();
            }
        } catch (DocumentNotFoundException $exception) {
            trigger_error($exception->getMessage(), E_USER_WARNING);

            $this->mediaList->remove($this->mediaList->key());
            SimpleMediaRepository::persist($this);
        }

        return $this->mediaList->first() ?: null;
    }

    /**
     * @return SimpleMediaInterface[]
     */
    public function getMediaList()
    {
        return $this->mediaList->toArray();
    }

    /**
     * @param SimpleMediaInterface[] $mediaList
     */
    public function setMediaList(array $mediaList)
    {
        $oldMediaList = $this->mediaList->toArray();
        foreach (array_diff($oldMediaList, $mediaList) as $media) {
            $this->removeResource($media);
        }
        foreach ($mediaList as $media) {
            $this->addResource($media);
        }
    }

    /**
     * @param SimpleMediaInterface|null $media
     */
    public function setMedia(SimpleMediaInterface $media = null)
    {
        $this->addResource($media);
    }

    /**
     * @return SimpleMediaInterface[]
     * @deprecated
     */
    public function getResources()
    {
        return $this->getMediaList();
    }

    /**
     * @param SimpleMediaInterface[] $resources
     * @deprecated
     */
    public function setResources(array $resources)
    {
        $this->mediaList = new ArrayCollection($resources);
    }

    /**
     * @return bool
     * @deprecated
     */
    public function hasResources()
    {
        return !$this->mediaList->isEmpty();
    }

    /**
     * @return int
     */
    public function getResourceCount()
    {
        return $this->mediaList->count();
    }

    /**
     * @param SimpleMediaInterface $simpleMedia
     * @deprecated
     */
    public function addResource(SimpleMediaInterface $simpleMedia)
    {
        if (!$this->mediaList->contains($simpleMedia)) {
            $this->mediaList->add($simpleMedia);
        }
    }

    /**
     * @param SimpleMediaInterface $resource
     * @deprecated
     */
    public function removeResource(SimpleMediaInterface $resource)
    {
        $this->mediaList->removeElement($resource);
    }

    /**
     * @return array
     * @deprecated
     */
    public function getResourceIds()
    {
        return $this->createIdList($this->mediaList);
    }

    /**
     * @param int $resourceId
     * @deprecated
     */
    public function removeResourceId($resourceId)
    {
        $this->mediaList->offsetUnset($resourceId);
    }

    /**
     * @return $this
     */
    public function clearMediaList()
    {
        $this->mediaList->clear();
        return $this;
    }

    /**
     * @return SimpleMediaInterface|null
     * @deprecated
     */
    public function getFirstResource()
    {
        return $this->mediaList->first() ?: null;
    }

    /**
     * @deprecated use getMediaList() instead
     * @return SimpleMediaInterface[]
     */
    public function getFiles()
    {
        return $this->mediaList->toArray();
    }

    /**
     * @param SimpleMediaInterface[] $mediaList
     */
    public function setFiles(array $mediaList)
    {
        $this->mediaList = new ArrayCollection($mediaList);
        $this->setThumbFile($this->mediaList->contains($this->thumb) ? $this->thumb : null);
    }

    /**
     * @param SimpleMediaInterface $simpleMedia
     */
    public function addFile(SimpleMediaInterface $simpleMedia)
    {
        if (!$this->mediaList->contains($simpleMedia)) {
            $this->mediaList->add($simpleMedia);
        }
    }

    /**
     * @return bool
     */
    public function hasFiles()
    {
        return !$this->mediaList->isEmpty();
    }

    /**
     * @return int[]
     */
    public function getFilePositions()
    {
        return $this->createIdList($this->mediaList);
    }

    /**
     * @param int[] $positions
     */
    public function setFilePositions(array $positions)
    {
        $positions = array_flip($positions);
        $simpleMediaList = $this->mediaList->toArray();
        usort($simpleMediaList, function (SimpleMediaInterface $a, SimpleMediaInterface $b) use ($positions) {
            if (!isset($positions[$a->getId()]) || !isset($positions[$b->getId()])) {
                return 0;
            }
            return $positions[$a->getId()] < $positions[$b->getId()] ? -1 : 1;
        });
        $this->mediaList = new ArrayCollection($simpleMediaList);
    }

    /**
     * @param int $fileId
     * @param int $position
     */
    public function setFilePosition($fileId, $position)
    {
        $oldPosition = null;
        foreach ($this->mediaList as $key => $value) {
            if ($fileId == $value->getId()) {
                $oldPosition = $key;
                break;
            }
        }

        $positions = [];
        foreach ($this->mediaList as $key => $value) {
            if ($fileId == $value->getId()) {
                $positions[$position] = $value->getId();
            } elseif ($oldPosition < $position) {
                $positions[$key - ($key <= $position)] = $value->getId();
            } elseif ($oldPosition > $position) {
                $positions[$key + ($key >= $position)] = $value->getId();
            }
        }
        $this->setFilePositions($positions);
    }

    /**
     * @return int|null
     */
    public function getThumbFileId()
    {
        if (!isset($this->thumb)) {
            return null;
        }
        return $this->thumb->getId();
    }

    /**
     * @param int $id
     */
    public function setThumbFileId($id)
    {
        $this->setThumbFile(
            $this->mediaList->filter(function (SimpleMediaInterface $file) use ($id) {
                return $file->getId() == $id;
            })->first() ?: null
        );
    }

    /**
     * @return bool
     */
    public function hasThumbFileId()
    {
        return isset($this->thumb);
    }

    /**
     * @return MediaInterface|null
     */
    public function getThumbFile()
    {
        if (!isset($this->thumb)) {
            return null;
        }
        return $this->thumb->getThumb();
    }

    /**
     * @param SimpleMediaInterface|null $thumb
     */
    public function setThumbFile(SimpleMediaInterface $thumb = null)
    {
        if (isset($thumb) && !$this->mediaList->contains($thumb)) {
            throw new \InvalidArgumentException('Cannot use ' . $thumb->getId() . ' as thumb, because it is not a member of ' . $this->getId() . '!');
        }

        if (!isset($thumb)) {
            $thumb = $this->mediaList->filter(function (SimpleMediaInterface $file) {
                return $this->isMediaTypeAsThumbSupported($file->getMediaType());
            })->first() ?: null;
        }

        $this->thumb = $thumb;
    }

    /**
     * @param string $mediaType
     * @return bool
     */
    public function isMediaTypeAsThumbSupported($mediaType)
    {
        return in_array($mediaType, [self::MEDIA_TYPE_AUDIO, self::MEDIA_TYPE_PDF, self::MEDIA_TYPE_VIDEO, self::MEDIA_TYPE_IMAGE, self::MEDIA_TYPE_ARCHIVE]);
    }

}