<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Model\User;

/**
 * @method static Portal|null findById($id)
 * @method static Portal getById($id)
 * @method static Portal[] findByIds(array $id, array $options = [])
 * @method static Portal[] findAll(array $options = [])
 */
class PortalRepository extends MongoRepository
{

    /**
     * @param int $adminId
     * @param array $options
     * @return mixed
     */
    public static function findByAdminId($adminId, array $options = [])
    {
        return self::findByAdminUuid(IdMappingRepository::getByTypeAndReferenceId(User::class,
            $adminId)->getReferenceUuid(), $options);
    }

    /**
     * @param string $adminId
     * @param array $options
     * @return mixed
     */
    public static function findByAdminUuid($adminId, array $options = [])
    {
        return self::find(['admins.uuid' => (string)$adminId], $options);
    }

    /**
     * @param int $reviewerId
     * @param array $options
     * @return array
     */
    public static function findByReviewerId($reviewerId, array $options = [])
    {
        return self::findByReviewerUuid(IdMappingRepository::getByTypeAndReferenceId(User::class,
            $reviewerId)->getReferenceUuid(), $options);
    }

    /**
     * @param string $reviewerId
     * @param array $options
     * @return array
     */
    public static function findByReviewerUuid($reviewerId, array $options = [])
    {
        return self::find(['reviewers.uuid' => (string)$reviewerId], $options);
    }

    /**
     * @return Portal
     */
    public static function getDefault()
    {
        return self::find(['default' => true], ['require' => 1])[0];
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findAllExceptDefault(array $options = [])
    {
        return self::find(['default' => false], $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowExpositionConnecting(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_EXPOSITION_CONNECTING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowWorkConnecting(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_WORK_CONNECTING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowDegreeConnecting(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_DEGREE_CONNECTING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowProjectConnecting(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_PROJECT_CONNECTING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowExpositionPublishing(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_EXPOSITION_PUBLISHING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowLimitedExpositionPublishing(array $options = [])
    {
        return self::find([
            'features.' . Portal::FEATURE_EXPOSITION_PUBLISHING         => true,
            'features.' . Portal::FEATURE_LIMITED_EXPOSITION_PUBLISHING => true,
        ], $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowDegreePublishing(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_DEGREE_PUBLISHING, $options);
    }

    /**
     * @param array $options
     * @return \Rc\Domain\Model\Portal[]
     */
    public static function findByAllowApplicationPublishing(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_APPLICATION_PUBLISHING, $options);
    }

    /**
     * @param array $options
     * @return Portal[]
     */
    public static function findByAllowProfile(array $options = [])
    {
        return self::findByFeature(Portal::FEATURE_PROFILE, $options);
    }

    /**
     * @param Research $research
     * @return bool
     */
    public static function isResearchUsedAsTemplate(Research $research)
    {
        if (!$research instanceof Exposition || !$research->getDefaultWeave() instanceof TextWeave) {
            return false;
        }

        return self::count([
                'textExpositionTemplates.uuid' => (string)$research->getUuid(),
            ]) > 0;
    }

    /**
     * @param string $feature
     * @param array $options
     * @return Portal[]
     */
    protected static function findByFeature($feature, array $options = [])
    {
        return self::find(['features.' . $feature => true], $options);
    }
}