<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Research;

use Rc\Domain\Model\Application\ApplicationProgram;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Research;
use Rc\Domain\Model\Research\Application;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\StatusAwareInterface;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\SingleCollectionInheritanceInterface;

/**
 * @method static Application|null findById($id)
 * @method static Application|null findOne(array $query = [], array $options = [])
 * @method static Application getById($id)
 * @method static Application[] findByIds(array $id, array $options = [])
 * @method static Application[] findAll(array $options = [])
 */
class ApplicationRepository extends ResearchRepository implements SingleCollectionInheritanceInterface
{

    /**
     * @param User $user
     * @param Exposition $exposition
     * @param array $options
     * @return Application[]
     */
    public static function findByUserForReviewing(User $user, Exposition $exposition = null, array $options = [])
    {
        $query = [
            'status'         => ['$in' => [Application::STATUS_REVIEW, Application::STATUS_PRE_REVIEW]],
            'reviewers.uuid' => ['$in' => self::castListToUuidList([$user])],
        ];

        if (isset($exposition)) {
            $query += [
                'expositions.uuid' => (string)$exposition->getUuid(),
            ];
        }

        return self::find($query, $options);
    }

    /**
     * @param User $user
     * @param Exposition $exposition
     * @param array $options
     * @return Application[]
     */
    public static function findByUserForReading(User $user, Exposition $exposition = null, array $options = [])
    {
        $query = [
            'status'               => Application::STATUS_REVIEW,
            'externalReaders.uuid' => ['$in' => self::castListToUuidList([$user])],
        ];
        if (isset($exposition)) {
            $query += [
                'expositions.uuid' => (string)$exposition->getUuid(),
            ];
        }
        return self::find($query, $options);
    }

    /**
     * @param User $user
     * @param Exposition $exposition
     * @param array $options
     * @return Application[]
     */
    public static function findSupervisedByUser(User $user, Exposition $exposition = null, array $options = [])
    {
        $query = [
            'status'           => Application::STATUS_PROGRESS,
            'supervisors.uuid' => ['$in' => self::castListToUuidList([$user])],
        ];
        if (isset($exposition)) {
            $query += [
                'expositions.uuid' => (string)$exposition->getUuid(),
            ];
        }
        return self::find($query, $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Application[]
     */
    public static function findForeignByPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'applicantPortal.uuid' => (string)$portal->getUuid(),
            'status'               => ['$ne' => StatusAwareInterface::STATUS_PROGRESS],
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Application[]|Research[]
     */
    public static function findApprovedByPortal(Portal $portal, array $options = [])
    {
        return self::findByPortalIdAndStatus($portal->getUuid(), StatusAwareInterface::STATUS_PUBLISHED, $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Application[]|Research[]
     */
    public static function findRejectedByPortal(Portal $portal, array $options = [])
    {
        return self::findByPortalIdAndStatus($portal->getUuid(), StatusAwareInterface::STATUS_DECLINED, $options);
    }

    /**
     * @param ApplicationProgram $program
     * @param array $options
     * @return Application[]
     */
    public static function findByProgram(ApplicationProgram $program, array $options = [])
    {
        return self::findByProgramUuid($program->getUuid(), $options);
    }

    /**
     * @param string $uuid
     * @param array $options
     * @return Application[]
     */
    public static function findByProgramUuid($uuid, array $options = [])
    {
        return self::find([
            'program.uuid' => $uuid,
        ], $options);
    }
}
