<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository\Research;

use Identity;
use Rc\Domain\Model\Portal;
use Rc\Domain\Model\Portal\Issue;
use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave\TextWeave;
use Rc\Domain\Model\Research\Work;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Generic\SearchResult;
use Rc\Domain\Repository\IdMappingRepository;
use Rc\Domain\Repository\ResearchRepository;
use Rc\Domain\Repository\SingleCollectionInheritanceInterface;

/**
 * @method static Exposition|null findById($id)
 * @method static Exposition|null findOne(array $query = [], array $options = [])
 * @method static Exposition getById($id)
 * @method static Exposition[] findByIds(array $id, array $options = [])
 * @method static Exposition[] findAll(array $options = [])
 * @method static Exposition[] findByPortalForReviewing(Portal $portal, array $options = [])
 * @method static Exposition[] findByPortalIdForReviewing($portalId, array $options = [])
 * @method static Exposition[] findPublic(array $options = [])
 */
class ExpositionRepository extends ResearchRepository implements SingleCollectionInheritanceInterface
{
    /**
     * @type array
     */
    protected static $defaultOrder = [
        'publishDate'  => self::SORT_DESC,
        'lastModified' => self::SORT_DESC,
    ];

    /**
     * @var array
     */
    protected static $defaultPrimeProperties = [
        'preview',
        'owner',
        'coAuthors',
        'defaultWeave',
    ];

    /**
     * @param array $options
     * @return Exposition[]
     */
    public static function findPublished(array $options = [])
    {
        return self::find([
            'status'  => Exposition::STATUS_PUBLISHED,
            'blocked' => false,
            'hide'    => false,
            '$and'    => [self::getIsAccessibleCondition()],
        ], $options + ['prime' => true]);
    }

    /**
     * @param array $options
     * @return Exposition[]
     */
    public static function findPublicButNotPublished(array $options = [])
    {
        return self::find([
            'accessLevel' => ['$ne' => Exposition::ACCESS_LEVEL_PRIVATE],
            'status'      => ['$nin' => [Exposition::STATUS_PUBLISHED, Exposition::STATUS_REVIEW_PUBLISHED]],
            'blocked'     => false,
            'hide'        => false,
            '$and'        => [self::getIsAccessibleCondition()],
        ], $options + ['prime' => true]);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Exposition[]
     */
    public static function findPublishedByMainPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'status'      => Exposition::STATUS_PUBLISHED,
            'blocked'     => false,
            'hide'        => false,
            'portal.uuid' => (string)$portal->getUuid(),
            '$and'        => [self::getIsAccessibleCondition()],
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Exposition[]
     */
    public static function findPublishedByPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'status'  => Exposition::STATUS_PUBLISHED,
            'blocked' => false,
            'hide'    => false,
            '$or'     => [
                ['portal.uuid' => (string)$portal->getUuid()],
                ['partOfPortal.uuid' => (string)$portal->getUuid()],
                ['additionalPortals.uuid' => (string)$portal->getUuid()],
            ],
            '$and'    => [self::getIsAccessibleCondition()],
        ], $options);
    }

    /**
     * @param Issue $issue
     * @param array $options
     * @return array
     */
    public static function findByIssue(Issue $issue, array $options = [])
    {
        return self::find([
            'blocked'     => false,
            'hide'        => false,
            'portalIssue' => (string)$issue->getUuid(),
            '$or'         => [
                ['portal.uuid' => (string)$issue->getPortal()->getUuid()],
                ['partOfPortal.uuid' => (string)$issue->getPortal()->getUuid()],
                ['additionalPortals.uuid' => (string)$issue->getPortal()->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param User[] $users
     * @return Exposition[]
     */
    public static function findTextExpositionsByUsers(array $users)
    {
        /** @var Exposition[] $expositions */
        $expositions = self::find([
            'owner.uuid' => ['$in' => self::castListToUuidList($users)],
        ]);

        foreach ($expositions as $key => $exposition) {
            if (!$exposition->getDefaultWeave() instanceof TextWeave) {
                unset($expositions[$key]);
            }
        }

        return $expositions;
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Exposition[]
     */
    public static function findByPortal(Portal $portal, array $options = [])
    {
        return self::find([
            'blocked' => false,
            'hide'    => false,
            '$or'     => [
                ['portal.uuid' => (string)$portal->getUuid()],
                ['partOfPortal.uuid' => (string)$portal->getUuid()],
                ['additionalPortals.uuid' => (string)$portal->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param int $portalId
     * @param array $options
     * @return Exposition[]
     */
    public static function findByPortalId($portalId, array $options = [])
    {
        return self::findByPortalUuid(IdMappingRepository::getByTypeAndReferenceId(Portal::class,
            $portalId)->getReferenceUuid(), $options);
    }

    /**
     * @param string $portalId
     * @param array $options
     * @return Exposition[]
     */
    public static function findByPortalUuid($portalId, array $options = [])
    {
        return self::find([
            '$or' => [
                ['portal.uuid' => (string)$portalId],
                ['partOfPortal.uuid' => (string)$portalId],
                ['additionalPortals.uuid' => (string)$portalId],
            ],
        ], $options);
    }

    /**
     * @param array $options
     * @return Exposition[]
     */
    public static function findPortalLess(array $options = [])
    {
        return self::find([
            'portal.uuid'       => null,
            'partOfPortal.uuid' => null,
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findByUserForReviewing(User $user, array $options = [])
    {
        return self::findByUserIdForReviewing($user->getUuid(), $options);
    }

    /**
     * @param string $userId
     * @param array $options
     * @return Exposition[]
     */
    public static function findByUserIdForReviewing($userId, array $options = [])
    {
        return self::find([
            'status' => ['$in' => [Exposition::STATUS_REVIEW, Exposition::STATUS_REVIEW_PUBLISHED]],
            '$or'    => [
                ['reviewers.uuid' => (string)$userId],
                ['doubleBlindReviewers.uuid' => (string)$userId],
            ],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findCollaboratedByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or'     => [
                ['owner.uuid' => ['$ne' => (string)$user->getUuid()]],
                ['coAuthors.uuid' => (string)$user->getUuid()],
                ['contributors.uuid' => (string)$user->getUuid()],
            ],
            'blocked' => false,
        ], $options + ['sort' => ['created' => self::SORT_DESC]]);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findSupervisedByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or'     => [
                ['owner.uuid' => ['$ne' => (string)$user->getUuid()]],
                ['supervisors.uuid' => (string)$user->getUuid()],
                ['pendingInvites.supervisor.' . $user->getUuid() => ['$ne' => true]],
            ],
            'blocked' => false,
        ], $options + ['sort' => ['created' => self::SORT_DESC]]);
    }

    /**
     * @param Portal $portal
     * @param array $options
     * @return Exposition[]
     */
    public static function findConnected(Portal $portal, array $options = [])
    {
        return self::find([
            '$or' => [
                ['portal.uuid' => (string)$portal->getUuid()],
                ['partOfPortal.uuid' => (string)$portal->getUuid()],
                ['additionalPortals.uuid' => (string)$portal->getUuid()],
            ],
        ], $options);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findOwnAndCollaboratedByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or'     => [
                ['owner.uuid' => (string)$user->getUuid()],
                ['coAuthors.uuid' => (string)$user->getUuid()],
                ['contributors.uuid' => (string)$user->getUuid()],
            ],
            'blocked' => false,
        ], $options + ['sort' => ['created' => self::SORT_DESC]]);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findEditedByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or'     => [
                ['owner.uuid' => ['$ne' => (string)$user->getUuid()]],
                ['editors.uuid' => (string)$user->getUuid()],
            ],
            'blocked' => false,
        ], $options + ['sort' => ['created' => self::SORT_DESC]]);
    }

    /**
     * @param User $user
     * @param array $options
     * @return Exposition[]
     */
    public static function findAttendedByPerson(User $user, array $options = [])
    {
        return self::find([
            '$or'     => [
                ['owner.uuid' => ['$ne' => (string)$user->getUuid()]],
                ['groups' => (int)$user->getId()],
            ],
            'blocked' => false,
        ], $options + ['sort' => ['created' => self::SORT_DESC]]);
    }

    /**
     * @param array $options
     * @return Exposition[]
     */
    public static function findAllNonPublished(array $options = [])
    {
        return self::find([
            'status' => ['$ne' => Exposition::STATUS_PUBLISHED],
        ], $options);
    }

    /**
     * @param array $arguments
     * @param array $options
     * @return Exposition[]|SearchResult
     */
    public static function search(array $arguments, array $options = [])
    {
        $queryBuilder = self::createQueryBuilder([
            'blocked' => false,
            'hide'    => false,
        ], $options);

        // title
        if (!empty($arguments['title'])) {
            $queryBuilder->addAnd(self::buildSearchRegexForTranslationList('title', (string)$arguments['title']));
        }

        // status
        if (!empty($arguments['status'])) {
            $queryBuilder->addAnd(['status' => ['$in' => self::castListToString($arguments['status'])]]);
        }

        // keywords
        if (!empty($arguments['keyword'])) {
            $queryBuilder->addAnd(['keywords' => ['$in' => [(string)$arguments['keyword']]]]);
        }

        // portal
        if (!empty($arguments['portal'])) {
            $portalUuid = IdMappingRepository::getByTypeAndReferenceId(Portal::class, $arguments['portal'])->getReferenceUuid();
            $queryBuilder->addAnd([
                '$or' => [
                    ['portal.uuid' => $portalUuid],
                    ['partOfPortal.uuid' => $portalUuid],
                    ['additionalPortals.uuid' => $portalUuid],
                ],
            ]);
        }

        // author
        if (!empty($arguments['author'])) {
            $authorUuid = IdMappingRepository::getByTypeAndReferenceId(User::class, $arguments['author'])->getReferenceUuid();
            $queryBuilder->addAnd([
                '$or' => [
                    ['author.uuid' => $authorUuid],
                    ['coAuthors.uuid' => $authorUuid],
                ],
            ]);
        }

        // exclude researches for double blind reviewers
        if (Identity::exists()) {
            $queryBuilder->addAnd(['doubleBlindReviewers' => ['$nin' => [Identity::getUserUuid()]]]);
        }

        // fulltext
        if (!empty($arguments['fulltext'])) {
            $queryBuilder->text((string)$arguments['fulltext']);
            $queryBuilder->sortMeta('score', 'textScore');
        }

        // accessible condition
        if (!empty($arguments['accessible'])) {
            $queryBuilder->addAnd(self::getIsAccessibleCondition($arguments['accessible'][0], $arguments['accessible'][1]));
        } else {
            $queryBuilder->addAnd(self::getIsAccessibleCondition());
        }

        return new SearchResult($queryBuilder);
    }

    /**
     * @param Work $work
     * @return array
     */
    public static function findBySharedWork(Work $work)
    {
        return self::find([
            'shares.$id' => new \MongoId($work->getUuid()),
        ]);
    }

}
