<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Domain\Repository;

use Rc\Domain\Model\Portal;
use Rc\Domain\Model\User;
use Rc\Domain\Repository\Generic\SearchResult;
use Rc\Domain\Repository\Generic\Suggest;

/**
 * @method static User|null findById($id)
 * @method static User getById($id)
 * @method static User[] findByIds(array $ids, array $options = [])
 * @method static User[] findByUuids(array $uuids, array $options = [])
 * @method static User[] findAll(array $options = [])
 */
class UserRepository extends MongoRepository
{
    const USER_ID_SYSTEM = 5;
    const USER_ID_USER_SUPPORT = 45585;

    /**
     * @var array
     */
    protected static $suggestPropertyList = [
        'name',
    ];

    /**
     * @var array
     */
    protected static $defaultPrimeProperties = [
        'mediaList',
    ];

    /**
     * @param array $options
     * @return User[]
     * @throws \Doctrine\ODM\MongoDB\MongoDBException
     * @throws \MongoException
     */
    public static function findForLandingPage(array $options = [])
    {
        $media = MediaRepository::findByCheckSum(MediaRepository::CHECKSUM_DEFAULT_PROFILE_IMAGE);
        if (!isset($media)) {
            return [];
        }

        $userIds = self::getClient()
            ->getDocumentCollection(User::class)
            ->aggregate([
                [
                    '$match' => [
                        'hide'             => false,
                        'deleted'          => false,
                        'hasPublicObjects' => true,
                        'mediaList.$id'    => ['$ne' => new \MongoId($media->getUuid())],
                    ],
                ],
                [
                    '$sample' => [
                        'size' => isset($options['limit']) ? $options['limit'] : 9,
                    ],
                ],
                [
                    '$project' => [
                        '_id' => true,
                    ],
                ],
            ], ['cursor' => true])
            ->toArray();

        return static::findByUuids(array_map(function (array $entry) {
            return $entry['_id'];
        }, $userIds), $options + ['prime' => true]);
    }

    /**
     * @param array $options
     * @return User[]
     */
    public static function findVisible(array $options = [])
    {
        return self::find([
            'hide'     => false,
            'deleted'  => false,
            'approved' => true,
        ], $options);
    }

    /**
     * @param int[] $ids
     * @param array $options
     * @return User[]
     */
    public static function findVisibleByIds(array $ids, array $options = [])
    {
        if (empty($ids)) {
            return [];
        }
        return self::find([
            'id'       => ['$in' => self::castListToInteger($ids)],
            'hide'     => false,
            'deleted'  => false,
            'approved' => true,
        ], $options);
    }

    /**
     * @param string $email
     * @param string $password
     * @return User|null
     */
    public static function findByLoginInformation($email, $password)
    {
        return self::findOne([
            'email'    => (string)strtolower($email),
            'password' => (string)$password,
            'deleted'  => false,
        ]);
    }

    /**
     * @param string $email
     * @return User|null
     */
    public static function findByEmail($email)
    {
        return self::findOne([
            'email' => trim((string)$email),
        ]);
    }

    /**
     * @param string $passwordForgottenKey
     * @return User|null
     */
    public static function findByPasswordForgottenKey($passwordForgottenKey)
    {
        return self::findOne([
            'passwordForgottenKey' => (string)$passwordForgottenKey,
        ]);
    }

    /**
     * @param string $role
     * @param array $options
     * @return User[]
     */
    public static function findByRole($role, array $options = [])
    {
        return self::findByRoles([(string)$role], $options);
    }

    /**
     * @param string[] $roles
     * @param array $options
     * @return User[]
     */
    public static function findByRoles(array $roles, array $options = [])
    {
        return self::find([
            'roles'   => ['$in' => self::castListToString($roles)],
            'deleted' => false,
        ], $options);
    }

    /**
     * @param Portal $portal
     * @param bool $deleted
     * @param array $options
     * @return User[]
     */
    public static function findByPortal(Portal $portal, $deleted = false, array $options = [])
    {
        return self::findByPortalUuid($portal->getUuid(), $deleted, $options);
    }

    /**
     * @param string $portalId
     * @param bool $deleted
     * @param array $options
     * @return User[]
     */
    public static function findByPortalUuid($portalId, $deleted = false, array $options = [])
    {
        return self::find([
            'portals.uuid' => (string)$portalId,
            'deleted'      => (bool)$deleted,
        ], $options);
    }

    /**
     * @param int $portalId
     * @param bool $deleted
     * @param array $options
     * @return User[]
     */
    public static function findSarMemberByPortalId($portalId, $deleted = false, array $options = [])
    {
        return self::findSarMemberByPortalUuid(IdMappingRepository::getByTypeAndReferenceId(Portal::class,
            $portalId)->getReferenceUuid(), $deleted, $options);
    }

    /**
     * @param string $portalId
     * @param bool $deleted
     * @param array $options
     * @return User[]
     */
    public static function findSarMemberByPortalUuid($portalId, $deleted = false, array $options = [])
    {
        return self::find([
            'portals.uuid' => (string)$portalId,
            'deleted'      => (bool)$deleted,
            'sarMember'    => true,
        ], $options);
    }

    /**
     * @return User
     */
    public static function findSystemAccount()
    {
        return self::findOne(['id' => self::USER_ID_SYSTEM]);
    }

    /**
     * @return User|null
     */
    public static function findUserSupport()
    {
        return self::findOne(['id' => self::USER_ID_USER_SUPPORT]);
    }

    /**
     * @param array $arguments
     * @param array $options
     * @return User[]|SearchResult
     */
    public static function search(array $arguments, array $options = [])
    {
        $queryBuilder = self::createQueryBuilder([
            'hide'    => false,
            'deleted' => false,
        ], $options);

        // name
        if (isset($arguments['name'])) {
            $queryBuilder->addAnd([
                'name' => [
                    '$regex'   => '(^| )' . preg_quote(strtolower($arguments['name'])) . '.*',
                    '$options' => 'i',
                ],
            ]);
        }

        // country
        if (isset($arguments['country'])) {
            $queryBuilder->addAnd(['countryResidence' => (string)$arguments['country']]);
        }

        // fulltext
        if (isset($arguments['fulltext'])) {
            $queryBuilder->text((string)$arguments['fulltext']);
            $queryBuilder->sortMeta('score', 'textScore');
        }

        return new SearchResult($queryBuilder);
    }

    /**
     * @param string $field
     * @param string $searchTerm
     * @param array $options
     * @return Suggest[]
     */
    public static function suggest($field, $searchTerm, array $options = [])
    {
        return parent::suggest($field, $searchTerm, $options + ['approvedOnly' => true]);
    }
}