<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Export;

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Model\Research\Exposition\Weave;
use Rc\Domain\Model\Research\Exposition\Weave\GraphicalWeave;
use Rc\Domain\Repository\Research\Exposition\SimpleMediaRepository;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;
use Zend_Controller_Front;
use Zend_View;
use ZipArchive;

/**
 * @category  SAR.RC
 * @copyright Copyright (c) 2017 CHECK24 Vergleichsportal Flüge GmbH
 */
class ExpositionHtmlExport
{

    /**
     * @param Exposition $exposition
     * @return string
     */
    static public function export(Exposition $exposition)
    {
        $files = [];
        $content = [];

        foreach ($exposition->getWeaves() as $weave) {
            $weaveFilename = str_replace('/', '-', utf8_decode($weave->getTitle())) . '.html';

            $content[$weaveFilename] = self::exportWeave($weave, $files, $content);

            if ($weave === $exposition->getDefaultWeave()) {
                $content['index.html'] = $content[$weaveFilename];
            }
        }

        // create archive with all media files
        $file = tempnam(temp_path(), 'research_export_');

        $archive = new ZipArchive();
        $archive->open($file);

        $localRootDirectory = str_replace('/', '-', utf8_decode($exposition->getTitle())) . '/';
        foreach ($content as $localFileName => $tmpContent) {
            $archive->addFromString($localRootDirectory . $localFileName, $tmpContent);
        }

        $localAssetDirectory = $localRootDirectory . 'assets/';
        foreach ($files as $localFileName => $tmpFile) {
            if (is_dir($tmpFile)) {
                foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($tmpFile)) as $tmpSubFile) {
                    if (!is_file($tmpSubFile)) {
                        continue;
                    }

                    $tmpLocalFileName = substr($tmpSubFile, strpos($tmpSubFile, $localFileName));
                    if (isset($files[$tmpLocalFileName])) {
                        continue;
                    }

                    $archive->addFile($tmpSubFile, $localAssetDirectory . $tmpLocalFileName);
                }
            } else {
                $archive->addFile($tmpFile, $localAssetDirectory . $localFileName);
            }
        }
        $archive->close();

        return $file;
    }

    /**
     * @param Weave $weave
     * @param array $files
     * @param array $content
     * @return string
     */
    protected static function exportWeave(Weave $weave, array &$files, array &$content)
    {
        $exposition = $weave->getExposition();

        $viewContent = self::renderView('index', 'view', [
            'research' => $exposition->getId(),
            'weave'    => $weave->getId(),
            'identity' => 'anonymous',
        ]);

        // navigation thumb
        if ($weave instanceof GraphicalWeave) {
            $thumbContent = self::renderView('thumb', 'view', [
                'research' => $exposition->getId(),
                'weave'    => $weave->getId(),
                'identity' => 'anonymous',
            ]);
            $content['assets/navigation.png'] = $thumbContent;
            $viewContent = preg_replace('~data-src=".*"~i', 'src="assets/navigation.png"', $viewContent);
        }

        // chapter list and other view links
        preg_match_all('~"[^"]*(/view/\d+/\d+(/\d+(/\d+)?)?)"~iU', $viewContent, $matches);
        foreach ($matches[1] as $key => $viewLink) {
            list(, , $expositionId, $weaveId, $x, $y) = explode('/', $viewLink . '/0/0');
            if ($expositionId != $exposition->getId()) {
                continue;
            }

            $tmpWeave = array_values(array_filter($exposition->getWeaves(), function (Weave $weave) use ($weaveId) {
                return $weave->getId() == $weaveId;
            }));

            // link to an external weave
            if (empty($tmpWeave[0])) {
                continue;
            }

            $viewContent = str_replace($matches[0][$key], '"' . $tmpWeave[0]->getTitle() . '.html?x=' . $x . '&y=' . $y . '"', $viewContent);
        }

        // replace relative rc links to global
        $viewContent = preg_replace('~"(/((profile)|(portal)|(view)).*)"~iU', '"' . config('app.url') . '$1"', $viewContent);

        // popovers
        preg_match_all('~data-popover="(\d+)"~', $viewContent, $matches);
        foreach ($matches[1] as $popoverId) {
            $_SERVER['REDIRECT_URL'] = '/view/popover/' . $exposition->getId() . '/' . $popoverId;
            $popoverFilename = 'popover_' . $exposition->getId() . '_' . $popoverId . '.html';

            $content[$popoverFilename] = self::parseAssets(self::renderView('popover', 'view', [
                'research' => $exposition->getId(),
                'popover'  => $popoverId,
                'identity' => 'anonymous',
            ]), $files, $content);
        }

        $viewContent = self::parseAssets($viewContent, $files, $content);

        return $viewContent;
    }

    /**
     * @param string $viewContent
     * @param array $files
     * @param array $content
     * @param string $prefix
     * @return string
     */
    protected static function parseAssets($viewContent, array &$files, array &$content, $prefix = 'assets/')
    {
        // fetch all assets
        preg_match_all('~"(/rc/.*(.js|.css))"~iU', $viewContent, $matches);
        foreach ($matches[1] as $assetUrl) {
            $assetContent = file_get_contents(APPLICATION_PATH . '/../public/' . $assetUrl);
            preg_match_all('~(\'|"|url\()/?(rc/.*)(\'|"|\))~iU', $assetContent, $cssMatches);
            foreach ($cssMatches[2] as $embeddedAssetUrl) {
                $files[$embeddedAssetUrl] = APPLICATION_PATH . '/../public/' . ltrim($embeddedAssetUrl, '/');
            }

            $localAssetFilename = 'assets/' . str_replace(['/rc/minified/', '/rc/'], ['', 'rc/'], $assetUrl);
            $content[$localAssetFilename] = str_replace('/rc/', substr($assetUrl, -4) == '.css' ? 'rc/' : 'assets/rc/', $assetContent);
        }

        $mediaUri = config('media.url');

        // fetch all media files
        preg_match_all('~"' . $mediaUri . '/(.*)\?.*"~iU', $viewContent, $matches);
        foreach ($matches[1] as $mediaUrl) {
            $files[$mediaUrl] = config('media.storage_path') . '/' . $mediaUrl;
        }

        // fetch all media files (text-exposition)
        $viewContent = preg_replace_callback('~"/text-editor/simple-media-resource\?research=\d+&amp;simple-media=(\d+)"~iU', function (array $matches) use (&$files) {
            $simpleMedia = SimpleMediaRepository::getById($matches[1]);
            if ($simpleMedia->getMedia()->isPdf()) {
                $resource = $simpleMedia->getMedia()->getMaster();
            } else {
                $resource = $simpleMedia->getMedia()->getDefaultDerivative();
            }

            if (!isset($resource)) {
                return $matches[0];
            }

            $localPath = str_replace(config('media.storage_path') . '/', '', $resource->getUri());

            $files[$localPath] = $resource->getUri();

            return '"assets/' . $localPath . '"';
        }, $viewContent);

        // prepare view content
        $viewContent = str_replace('"' . $mediaUri, '"assets/', $viewContent);
        $viewContent = str_replace('"/rc/minified/', '"assets/', $viewContent);
        $viewContent = str_replace('"/rc/', '"assets/rc/', $viewContent);
        $viewContent = str_replace('"//', '"https://', $viewContent);

        return $viewContent;
    }

    /**
     * @param string $action
     * @param string $controller
     * @param array $parameters
     * @return string
     */
    protected static function renderView($action, $controller, array $parameters = [])
    {
        $baseDir = Zend_Controller_Front::getInstance()->getModuleDirectory() . DIRECTORY_SEPARATOR . 'views';
        $view = new Zend_View([
            'basePath'   => $baseDir,
            'helperPath' => config('zend.resources.view.helperPath'),
        ]);
        return $view->getHelper('action')->action($action, $controller, '', $parameters);
    }
}
