<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service;

use Rc\Domain\Model\Research\Exposition;
use Rc\Domain\Repository\LogRepository;
use Zend_Controller_Front;
use Zend_Http_Client;
use Zend_View;

class CrossRefService
{
    /**
     * @var string
     */
    static protected $doiPrefix = '10.22501';

    /**
     * @var string
     */
    static protected $loginId = 'sarr';

    /**
     * @var string
     */
    static protected $loginPasswd = 'art_17';

    /**
     * @param Exposition $exposition
     * @param bool $force
     * @return \Zend_Http_Response
     */
    static public function deposit(Exposition $exposition, $force = false)
    {
        if (!$exposition->getPortal()->getAllowExpositionDoiDeposit()) {
            return null;
        }

        if (!$force && $exposition->getDoi()) {
            return null;
        }

        self::assignDoi($exposition);

        if (!$exposition->getDoi()) {
            trigger_error('Could not generate doi for exposition #' . $exposition->getId() . '!', E_USER_WARNING);
            return null;
        }

        if ($exposition->getPortalEdition()) {
            return self::depositAsJournalArticle($exposition);
        } else {
            return self::depositAsPostedContent($exposition);
        }
    }

    /**
     * @param Exposition $exposition
     * @return \Zend_Http_Response
     */
    static protected function depositAsJournalArticle(Exposition $exposition)
    {
        return self::executeDeposit('deposit_journal_article', $exposition);
    }

    /**
     * @param Exposition $exposition
     * @return \Zend_Http_Response
     */
    static protected function depositAsPostedContent(Exposition $exposition)
    {
        return self::executeDeposit('deposit_posted_content', $exposition);
    }

    /**
     * @param string $template
     * @param Exposition $exposition
     * @return \Zend_Http_Response
     */
    static protected function executeDeposit($template, Exposition $exposition)
    {
        $revision = date('YmdHis') . str_pad(explode('.', microtime(true))[1], 4, '0', STR_PAD_RIGHT);
        $rawData = self::generateXmlFor($template, ['exposition' => $exposition, 'revision' => $revision]);

        LogRepository::add('CrossRef', [
            'exposition' => $exposition->getId(),
            'revision'   => $revision,
            'created'    => date(DATE_W3C),
            'data'       => $rawData,
        ]);

        $client = new Zend_Http_Client(null, ['timeout' => 60]);
        $client->setUri(config('services.crossref.uri'));
        $client->setHeaders('Content-Type', 'application/vnd.crossref.deposit+xml');
        $client->setHeaders('Authorization', 'Basic ' . base64_encode(self::$loginId . ':' . self::$loginPasswd));
        $client->setRawData($rawData);
        $response = $client->request(Zend_Http_Client::POST);

        return $response;
    }

    /**
     * @param string $template
     * @param array $data
     * @return string
     */
    static protected function generateXmlFor($template, array $data = [])
    {
        $baseDir = Zend_Controller_Front::getInstance()->getModuleDirectory() . DIRECTORY_SEPARATOR . 'views';
        $view = new Zend_View([
            'basePath'   => $baseDir,
            'helperPath' => config('zend.resources.view.helperPath'),
        ]);
        $view->assign($data);
        return $view->render('partials/crossref/' . $template . '.phtml');
    }

    /**
     * @param Exposition $exposition
     */
    static protected function assignDoi(Exposition $exposition)
    {
        if ($exposition->getDoi()) {
            return;
        }

        $exposition->setDoi(self::generateDoi($exposition));
    }

    /**
     * @param Exposition $exposition
     * @return string
     */
    static protected function generateDoi(Exposition $exposition)
    {
        $portal = $exposition->getPortal();
        if (empty($portal->getDoiAbbreviation())) {
            return null;
        }

        return self::$doiPrefix . '/' . $portal->getDoiAbbreviation() . '.' . $exposition->getId();
    }
}