<?php
/**
 * Copyright (c) 2018 by Society for Artistic Research
 *
 * All rights reserved.
 *
 * Modification, distribution, commercial or non-commercial reuse is prohibited without explicit written permission by the copyright holder.
 */

namespace Rc\Service;

use GuzzleHttp\Client;
use GuzzleHttp\Promise\EachPromise;
use Psr\Http\Message\ResponseInterface;
use Rc\Service\Ingest\IngestService;
use Rc\Service\Ingest\MediaTypes;
use Rc\Service\Madek\MadekMedia;
use Rc\Utility\Environment;

class MadekService
{

    /**
     * @var Client
     */
    protected $client;

    /**
     * @var array
     */
    protected $options;

    /**
     * @param array $options
     */
    public function __construct(array $options = [])
    {
        $this->options = $options + [
                'base_uri' => 'https://medienarchiv.zhdk.ch',
            ];
    }

    /**
     * @return bool
     */
    public function auth()
    {
        try {
            return $this->userId() !== null;
        } catch (\Exception $exception) {
            return false;
        }
    }

    /**
     * @return string
     * @throws GatewayException
     */
    public function userId()
    {
        return $this->get('/api/auth-info', 'id');
    }

    /**
     * @throws GatewayException
     */
    public function mediaEntities()
    {
        $mediaEntities = [];

        $url = $baseUrl = '/api/media-entries/?public_get_metadata_and_previews=true&public_get_full_size=true&filter_by=' . urlencode(json_encode([
                'permissions' => [
                    [
                        'key'   => 'public',
                        'value' => 'true',
                    ],
                    [
                        'key'   => 'responsible_user',
                        'value' => $this->userId(),
                    ],
                ],
            ]));

        do {
            $response = $this->get($url);

            $mediaEntities = array_merge($mediaEntities, $response['media-entries']);

            if (!isset($response['_json-roa']['collection']['next']['href'])) {
                break;
            }

            parse_str(parse_url($response['_json-roa']['collection']['next']['href'], PHP_URL_QUERY), $queryParams);

            $url = $baseUrl . '&page=' . $queryParams['page'];
        } while (true);

        foreach ($mediaEntities as $key => $mediaEntity) {
            $mediaEntities[$key] = $this->mediaEntity($mediaEntity['id'], $mediaEntity);
        }

        return $mediaEntities;
    }

    /**
     * @param string $uuid
     * @param array $data
     * @return MadekMedia
     * @throws GatewayException
     */
    public function mediaEntity($uuid, array $data = [])
    {
        $entity = array_merge(
            $data,
            $this->mediaAttributes($uuid),
            ['id' => $uuid]
        );

        $roa = $this->get('/api/media-entries/' . $uuid, '_json-roa');
        if (isset($roa['relations']['media-file']['href'])) {
            $media = $this->get($roa['relations']['media-file']['href']);
            $entity['media_type'] = MediaTypes::getContentTypeFromFilename($media['filename']);
        }

        return new MadekMedia($entity);
    }

    /**
     * @param string $uuid
     * @return string
     * @throws GatewayException
     */
    public function downloadMediaFile($uuid)
    {
        $roa = $this->get('/api/media-entries/' . $uuid, '_json-roa');
        if (!isset($roa['relations']['media-file']['href'])) {
            throw new \BadMethodCallException('Cannot download media file of madek entity ' . $uuid . ', because it is not accessible!');
        }

        $originalFilename = $this->get($roa['relations']['media-file']['href'], 'filename');

        $file = temp_path($uuid . '.' . uniqid(true) . '.' . str_replace(['/', '\\'], '_', $originalFilename));
        $resource = fopen($file, 'w');
        $stream = \GuzzleHttp\Psr7\stream_for($resource);

        $this->client()->get($roa['relations']['media-file']['href'] . '/data-stream', ['sink' => $stream]);

        return $file;
    }

    /**
     * @param string $uuid
     * @param bool $replace
     * @return \Rc\Domain\Model\MediaInterface
     * @throws GatewayException
     */
    public function importMediaFile($uuid, $replace = false)
    {
        return IngestService::masterFromFile($this->downloadMediaFile($uuid), false, $replace);
    }

    /**
     * @param string $uuid
     * @return array
     * @throws GatewayException
     */
    protected function mediaAttributes($uuid)
    {
        $urls = [];
        foreach ($this->get('/api/media-entries/' . $uuid . '/meta-data/', 'meta-data') as $metaDatum) {
            $urls[] = '/api/meta-data/' . $metaDatum['id'];
        }

        $metaData = [];
        foreach ($this->getMultiple($urls) as $metaDatum) {
            $metaData[$metaDatum['meta_key_id']] = $metaDatum['value'];
        }

        return $metaData;
    }

    /**
     * @return Client
     */
    protected function client()
    {
        if (!isset($this->client)) {
            $this->client = new Client([
                'base_uri' => $this->options['base_uri'],
                'auth'     => $this->options['auth'],
                'headers'  => [
                    'Accept' => 'application/json-roa+json',
                ],
            ]);
        }

        return $this->client;
    }

    /**
     * @param string $path
     * @param string|null $key
     * @return mixed
     * @throws GatewayException
     */
    protected function get($path, $key = null)
    {
        $response = $this->client()->get($path);

        return $this->responseData($response, $key);
    }

    /**
     * @param array $paths
     * @param string $key
     * @return array
     */
    protected function getMultiple(array $paths, $key = null)
    {
        $data = [];

        $promise = new EachPromise(
            array_map(function ($path) {
                return $this->client()->getAsync($path);
            }, $paths),
            [
                'concurrency' => 1,
                'fulfilled'   => function (ResponseInterface $response) use (&$data, $key) {
                    $data[] = $this->responseData($response, $key);
                },
            ]
        );

        $promise->promise()->wait(true);

        return $data;
    }

    /**
     * @param string $mimeType
     * @return null|string
     */
    protected function mediaType($mimeType)
    {
        if (!isset($mimeType)) {
            return null;
        }

        if ($mimeType == 'application/pdf') {
            return 'pdf';
        }

        $pieces = explode('/', $mimeType);
        if (!isset($pieces[0])) {
            return null;
        }

        switch ($pieces[0]) {
            case 'audio':
            case 'image':
            case 'video':
                return $pieces[0];
            default:
                return null;
        }
    }

    /**
     * @param ResponseInterface $response
     * @param string $key
     * @return mixed
     * @throws GatewayException
     */
    protected function responseData(ResponseInterface $response, $key = null)
    {
        if ($response->getStatusCode() != 200) {
            throw new GatewayException($response->getReasonPhrase());
        }

        $data = json_decode($response->getBody(), true);
        if (!isset($data)) {
            throw new GatewayException('Invalid response body!');
        }

        if (!isset($key)) {
            return $data;
        }

        if (!isset($data[$key])) {
            throw new GatewayException('Expected key ' . $key . ' does not exist in the response!');
        }

        return $data[$key];
    }

}